// ==============================================================
//                 ORBITER MODULE: ShuttlePB
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// ShuttlePB.cpp
// Control module for ShuttlePB vessel class
//
// Notes:
// This is an example for a "minimal" vessel implementation which
// only overloads the clbkSetClassCaps method to define vessel
// capabilities and otherwise uses the default VESSEL class
// behaviour.
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#define ENG 				1	
#define GEAR				2	
#define BRAKE               3
#define CANOPY				4	
#define START               5
#define STOP                6

#include <orbitersdk.h>
#include <cstdio>
#include <cmath>

#include <string>
#include "OrbiterSoundSDK35.h"

#include <stdio.h>
#include <stdlib.h>

using namespace std;


// ==============================================================
// Some vessel parameters
// ==============================================================


const double PB_FUELMASS = 10000.0;
const double PB_ISP = 50000.0;
const double PB_MAXMAINTH =0;

double iop = 0;
double st = 0;
double pld = 0;
double cen = 0;
double gear_proc = 0;
double gear_procl = 0;
double speedbrake = 0;
double power_proc = 0;
double flaps = 0;
double exproc = 0;
double Lights = 0;
double switched = 0;
double aoa;
double dil = 0;
double canopy = 0;
double engstart = 0;
double engrot = 0;
double start = 0;
double full = 0.001;
double i = 0;
double i1 = 0;
double i2 = 0;
double i3 = 0;
double i4 = 0;
double i5 = 0;
double i7 = 0;
double i9 = 0;
double i13 = 0;
double i14 = 0;
double i15 = 0;
double i16 = 0;
double i17 = 0;
double i18 = 0;
double i19 = 0;
double eng_pitch = 22000;
double refuel = 0;
double i1a = 0;
double i2a = 0;
double i3a = 0;
double i4a = 0;
double i5a = 0;
double s1 = 0;
double s2 = 0;
double s3 = 0;
double s4 = 0;
double s5 = 0;
double ga = 0;
double diff = 0.1;
double k3;
int k4;
char k3buffer [33];
double amet = 0;
double dee = 0;
double iopt = 0;

double i20 = 0;
double i21 = 0;
double secondtime = 0;


UINT animactive;

UINT animactivel;

UINT animactive2;

UINT animactive3;

UINT animactive4;

UINT animactive5;

UINT animactive6;

UINT animactive7;

UINT animactive9;


const VECTOR3 offset = {0,0.4,4.1};

const VECTOR3 offsetalt = {0.0733,0.785,4.34};

const VECTOR3 offsetaoa = {-0.085,0.723,4.34};

const VECTOR3 offsetdirect = {-0.005,0.645,4.34};

const VECTOR3 offsetspeed = {-0.0875,0.782,4.34};

const VECTOR3 offsetbank = {-0.005,0.74,4.34};

const VECTOR3 offsetclock = {-0.18,0.722,4.34};

const VECTOR3 offsetfuel = {0.0745,0.73,4.34};


THRUSTER_HANDLE th_main;


BEACONLIGHTSPEC beacon1;    

BEACONLIGHTSPEC beacon2; 
  
MESHHANDLE f150;
MESHHANDLE f151;
MESHHANDLE f152;
MESHHANDLE f153;
MESHHANDLE f154;
MESHHANDLE f155;
MESHHANDLE f156;
MESHHANDLE f157;
MESHHANDLE f158;
MESHHANDLE f159;
MESHHANDLE f160;
MESHHANDLE f161;
MESHHANDLE f162;
MESHHANDLE f163;
MESHHANDLE f164;
MESHHANDLE f165;
MESHHANDLE f166;
MESHHANDLE f167;
MESHHANDLE f168;


// 1. vertical lift component (wings and body)

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	static const double CL[nabsc] = {0.1, 0.17, 0.2, 0.2, 0.17, 0.1, 0, -0.11, -0.24, -0.38,  -0.5,  -0.5, -0.02, 0.6355,    0.63,   0.46, 0.28, 0.13, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,    0,0.002,0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = CM[idx] + (CM[idx+1]-CM[idx])*d;
	*cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.02 + oapiGetInducedDrag (*cl, 1.5, 0.6);
}


// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class ShuttlePB: public VESSEL2 {
public:
	ShuttlePB (OBJHANDLE hVessel, int flightmodel);
	//	: VESSEL2 (hVessel, flightmodel) {}
	void clbkSetClassCaps (FILEHANDLE cfg);
	void DefineAnimations(void);
	void Steering();
	void Lighting();
	void clbkPostCreation (void);
    void clbkPostStep (double simt, double simdt, double mjd);
    int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
   
    UINT anim_wheel;
    UINT anim_eng;
    UINT anim_gear;
    UINT anim_gearlever;
    UINT anim_rudder;
    UINT anim_elevator;
    UINT anim_laileron;
    UINT anim_raileron;
    UINT anim_speedbrake;
    
    UINT anim_flaps;
    UINT anim_canopy;
    UINT anim_power;
    UINT anim_dial44;

    
    UINT anim_wheel0;
    UINT anim_wheel1;
    UINT anim_wheel2;
    UINT anim_wheel3;
    UINT anim_wheel4;
    UINT anim_wheel5;
    
    UINT anim_dial1;
    UINT anim_dial2;
    UINT anim_dial3;
    UINT anim_bank;
    UINT anim_clock_s;
    UINT anim_clock_m;
    UINT anim_clock_h;

    
    bool clbkLoadVC (int id);
    
    int MySoundID;	


};

ShuttlePB::ShuttlePB (OBJHANDLE hVessel, int flightmodel)
		: VESSEL2 (hVessel, flightmodel)
{
	
	// preload meshes
	
    f150                 = oapiLoadMeshGlobal ("K33\\K-A-10A");
    f151                 = oapiLoadMeshGlobal ("K33\\K-A-10A-Cpit");
    f152                 = oapiLoadMeshGlobal ("K33\\K-A-10A-Cpit-Alt");
    f153                 = oapiLoadMeshGlobal ("K33\\K-A-10A-Cpit-AOA");
    f154                 = oapiLoadMeshGlobal ("K33\\K-A-10A-Cpit-Direct");
    f155                 = oapiLoadMeshGlobal ("K33\\K-A-10A-Cpit_n");
    f156                 = oapiLoadMeshGlobal ("K33\\K-A-10A-Cpit-Alt_n");
    f157                 = oapiLoadMeshGlobal ("K33\\K-A-10A-Cpit-AOA_n");
    f158                 = oapiLoadMeshGlobal ("K33\\K-A-10A-Cpit-Direct_n");
	f159                 = oapiLoadMeshGlobal ("K33\\K-A-10A-Cpit-Speed");
    f160                 = oapiLoadMeshGlobal ("K33\\K-A-10A-Cpit-Speed_n");
    f161                 = oapiLoadMeshGlobal ("K33\\K-A-10A-Cpit-Bank");
    f162                 = oapiLoadMeshGlobal ("K33\\K-A-10A-Cpit-Bank_n");
    f163                 = oapiLoadMeshGlobal ("K33\\K-A-10A-Cpit-Clock");
    f164                 = oapiLoadMeshGlobal ("K33\\K-A-10A-Cpit-Clock_n");
    f165                 = oapiLoadMeshGlobal ("K33\\K-A-10A-Cpit-Fuel");
    f166                 = oapiLoadMeshGlobal ("K33\\K-A-10A-Cpit-Fuel_n");
    f167                 = oapiLoadMeshGlobal ("K33\\K-A-10A-Tank");
    f168                 = oapiLoadMeshGlobal ("K33\\K-A-10A-Tankd");


	
	DefineAnimations ();
}


void ShuttlePB::clbkPostCreation (void)
{

int MySoundID=ConnectToOrbiterSoundDLL3(GetHandle());

//ReplaceStockSound3(MySoundID,"Sound\\Vessel\\K-Test-Car\\K-Test-Car.wav",	REPLACE_MAIN_THRUST);

RequestLoadVesselWave3(MySoundID,ENG,"Sound\\Vessel\\K-A-10A\\K-A-10A-Eng.wav",BOTHVIEW_FADED_FAR);

RequestLoadVesselWave3(MySoundID,BRAKE,"Sound\\Vessel\\K-A-10A\\K-A-10A-Brake.wav",BOTHVIEW_FADED_FAR);

RequestLoadVesselWave3(MySoundID,GEAR,"Sound\\Vessel\\K-A-10A\\K-A-10A-Gear.wav",BOTHVIEW_FADED_FAR);

RequestLoadVesselWave3(MySoundID,CANOPY,"Sound\\Vessel\\K-A-10A\\K-A-10A-Canopy.wav",BOTHVIEW_FADED_FAR);

RequestLoadVesselWave3(MySoundID,START,"Sound\\Vessel\\K-A-10A\\K-A-10A-Start.wav",BOTHVIEW_FADED_FAR);

RequestLoadVesselWave3(MySoundID,STOP,"Sound\\Vessel\\K-A-10A\\K-A-10A-Stop.wav",BOTHVIEW_FADED_FAR);

SoundOptionOnOff3(MySoundID,PLAYMAINTHRUST,FALSE);

SoundOptionOnOff3(MySoundID,PLAYHOVERTHRUST,FALSE);


SoundOptionOnOff3(MySoundID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);

SoundOptionOnOff3(MySoundID,PLAYCABINAIRCONDITIONING,FALSE);

SoundOptionOnOff3(MySoundID,PLAYCABINRANDOMAMBIANCE,FALSE);

SoundOptionOnOff3(MySoundID,PLAYRADIOATC,FALSE);

SoundOptionOnOff3(MySoundID,DISPLAYTIMER,FALSE);

}


// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------

void ShuttlePB::clbkSetClassCaps (FILEHANDLE cfg)
{
	// THRUSTER_HANDLE th_main;


	// physical specs
	SetSize (8);
	SetEmptyMass (15000.0);
	SetCW (0.3, 0.3, 0.6, 0.9);
	SetWingAspect (1.7);
	SetWingEffectiveness (3.1);
	SetCrossSections (_V(5,5,15));
	SetRotDrag (_V(3.5,3.5,3.5));
	if (GetFlightModel() >= 1) {
		SetPitchMomentScale (1e-4);
		SetBankMomentScale (1e-4);
	}
	SetPMI (_V(15.5,22.1,7.7));
	SetTrimScale (0.05);
	SetCameraOffset (_V(0,0.9,3.72));
	
	SetTouchdownPoints (_V(0,-1.84,4.21241), _V(-2.197965,-1.84,-0.3728825), _V(2.197965,-1.84,-0.3728825));

	// **************************** aerodynamics ***********************************

	CreateAirfoil (LIFT_VERTICAL,   _V(0,0,-0.561), VLiftCoeff, 20, 270, 2.266);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,-4), HLiftCoeff, 20,  50, 1.5);





	CreateControlSurface (AIRCTRL_ELEVATOR,     7, 0.7, _V(   0,0,-17), AIRCTRL_AXIS_XPOS,anim_elevator);
	CreateControlSurface (AIRCTRL_RUDDER,       2, 0.5, _V(   0,2,-15), AIRCTRL_AXIS_YPOS,anim_rudder);
	CreateControlSurface (AIRCTRL_AILERON,      4.8, 0.2, _V( 17.8,0,-17.8), AIRCTRL_AXIS_XPOS,anim_raileron);
	CreateControlSurface (AIRCTRL_AILERON,      4.8, 0.2, _V(-17.8,0,-17.8), AIRCTRL_AXIS_XNEG,anim_laileron);
	CreateControlSurface (AIRCTRL_ELEVATORTRIM, 5, 0.1, _V(   0,0,-15), AIRCTRL_AXIS_XPOS,anim_elevator);
    CreateControlSurface (AIRCTRL_FLAP,         5, 5, _V( 0,0,-1), AIRCTRL_AXIS_XPOS,anim_flaps);
	
	CreateVariableDragElement( &speedbrake, 8.0, _V(0,0,0) );

	//CreateVariableDragElement (&gear_proc, 0.8, _V(0, -1, 0));

	
	// propellant resources
	PROPELLANT_HANDLE hpr = CreatePropellantResource (PB_FUELMASS);
	
	
	//SURFHANDLE tex_main = oapiRegisterExhaustTexture ("K-F-15A-Exhaust");



	// ***************** thruster definitions *******************

	PARTICLESTREAMSPEC contrail_main = {
		0, 0.22, 111, 33, 0.03, 0.03, 1.3, 1.3, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 2,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};
	
	th_main = CreateThruster (_V(0,0,-4), _V(0,0,1), PB_MAXMAINTH, hpr, PB_ISP);
	
	
	
	CreateThrusterGroup (&th_main, 1, THGROUP_MAIN);
	
	
	
	//AddExhaust (th_main, 5.4, 0.55, _V(-0.55,-0.555,-6), _V(0,0,-1) , tex_main);
	//AddExhaust (th_main, 5.4, 0.55, _V(0.55,-0.555,-6), _V(0,0,-1) , tex_main);

    
    AddExhaustStream (th_main, _V(-1.163313,0.6339008,-4.155573), &contrail_main);
    AddExhaustStream (th_main, _V(1.163313,0.6339008,-4.155573), &contrail_main);
    
    
    
    
    // Strobe lights
	
	//Blue strobe
	
	static VECTOR3 beacon1pos = {7.3,0.2174522,-0.4028853};
	static VECTOR3 beacon1col = {0.043, 0.161, 0.992};
	
		beacon1.shape = (BEACONSHAPE_STAR);
		beacon1.pos = &beacon1pos;
		beacon1.col = &beacon1col;
		beacon1.size = (0.12);
		beacon1.falloff = (0.6);
		beacon1.period = (1.13);
		beacon1.duration = (0.05);
		beacon1.tofs = (0.6);
		beacon1.active = false;
		
		AddBeacon (&beacon1);
		
		//Red strobe
	
	static VECTOR3 beacon2pos = {-7.3,0.2174522,-0.4028853};
	static VECTOR3 beacon2col = {0.988, 0.133, 0.133};
	
		beacon2.shape = (BEACONSHAPE_STAR);
		beacon2.pos = &beacon2pos;
		beacon2.col = &beacon2col;
		beacon2.size = (0.12);
		beacon2.falloff = (0.6);
		beacon2.period = (1.13);
		beacon2.duration = (0.05);
		beacon2.tofs = (0.6);
		beacon2.active = false;
		
		AddBeacon (&beacon2);
		

    
    

	// visual specs
	
	AddMesh (f150);
    SetMeshVisibilityMode( AddMesh (f151,&offset), MESHVIS_VC );
    SetMeshVisibilityMode( AddMesh (f152,&offsetalt), MESHVIS_VC );
    SetMeshVisibilityMode( AddMesh (f153,&offsetaoa), MESHVIS_VC );
    SetMeshVisibilityMode( AddMesh (f154,&offsetdirect), MESHVIS_VC );
    SetMeshVisibilityMode( AddMesh (f159,&offsetspeed), MESHVIS_VC );
    SetMeshVisibilityMode( AddMesh (f161,&offsetbank), MESHVIS_VC );
    SetMeshVisibilityMode( AddMesh (f163,&offsetclock), MESHVIS_VC );
    SetMeshVisibilityMode( AddMesh (f165,&offsetfuel), MESHVIS_VC );


}

// ==============================================================
// API callback interface
// ==============================================================

void ShuttlePB::clbkPostStep (double simt, double simdt, double mjd)

{
	
	if( oapiGetTimeAcceleration() > 10.0 ) oapiSetTimeAcceleration(10);
	
	double vsAirSpeed = GetAirspeed();
	
	
	
	
    //speed and altitude control
    
    if (engstart == 0) 
	{
		SetThrusterGroupLevel(th_main,0);
		
		SetThrusterLevel(th_main,0);
		
		SetThrusterMax0 (th_main ,0);

	
}
    
    
	
	if (GetAirspeed() >= 333) 
	{
		SetCW (333, 333, 333, 333);
		SetThrusterMax0 (th_main ,0);

	
}

if (GetAltitude() >= 17000) 
	{
		SetCW (333, 333, 333, 333);
		SetThrusterMax0 (th_main ,0);

	
}



if (GetAirspeed() < 333 && GetAltitude() < 17000 && engstart ==2) 
	{
		SetCW (0.3, 0.3, 0.67, 0.92);
		SetThrusterMax0 (th_main ,3.8e5);

	
}

	double vsAirSpeed2 = (vsAirSpeed/1000)*3600;
    
    //Ground Steering
    
    if (GroundContact () ) Steering();
    
    
    //aoa = GetAOA();
    
    //sprintf(oapiDebugString(),"anim %2.2f", aoa );
    
  

    //Anims 
    
    if (iop > 1)iop=0;
    
    if (vsAirSpeed > 0) {
    
	double aspeed = vsAirSpeed;
    if (iop > 1)iop=0;
    if (vsAirSpeed > 30) aspeed = 30; 
    iop = iop + (aspeed /300);
     
    if (vsAirSpeed ==0) iop=0;
    
    
    if ( GroundContact() && gear_proc < 0.05) SetAnimation (anim_wheel, iop);
    
    

    

   }
	
	// Turbine Anims
	
	// Engine start
	
	if (engstart == 1)
	{
	if (iopt >1 )iopt = 0;
	iopt = iopt + engrot;
	
	engrot = engrot + 0.0005;
	
	if (engrot > 0.1333)
	{
		engstart = 2;
		
		engrot = 0.1333;
	}
	
	
	SetAnimation (anim_eng, iopt);
}
	
	// Engine on
	
	if (engstart == 2)
	{
	if (iopt >1 )iopt = 0;
	iopt = iopt + 0.1333;
	
	SetAnimation (anim_eng, iopt);
}

   // Engine stop
	
	if (engstart == 3)
	{
	if (iopt >1 )iopt = 0;
	iopt = iopt + engrot;
	
	engrot = engrot - 0.0005;
	
	if (engrot <= 0)
	{
		engstart = 0;
		
		engrot = 0;
	}

	SetAnimation (anim_eng, iopt);
}
   
// Gear animations


   if( animactive && !GroundContact()) {
		if( animactive == 1 ) {
			gear_proc += 0.00242;
			if( gear_proc > 1.0 ) {
				gear_proc = 1.0;
				animactive = 0;
			}
		}
		else  {
			gear_proc -= 0.00242;
			if( gear_proc < 0.0 ) {
				gear_proc = 0.0;
				animactive = 0;
			}
		}
		SetAnimation( anim_gear, gear_proc );
	}

	
	// Gear Lever animations


   if( animactivel && !GroundContact()) {
		if( animactivel == 1 ) {
			gear_procl += 0.00242;
			if( gear_procl > 1.0 ) {
				gear_procl = 1.0;
				animactivel = 0;
			}
		}
		else  {
			gear_procl -= 0.00242;
			if( gear_procl < 0.0 ) {
				gear_procl = 0.0;
				animactivel = 0;
			}
		}
		SetAnimation( anim_gearlever, gear_procl );
	}

	
	

   // Speedbrake animations


   if( animactive2 ) {
		if( animactive2 == 1 ) {
			speedbrake += 0.006;
			if( speedbrake > 1.0 ) {
				speedbrake = 1.0;
				animactive2 = 0;
			}
		}
		else  {
			speedbrake -= 0.006;
			if( speedbrake < 0.0 ) {
				speedbrake = 0.0;
				animactive2 = 0;
			}
		}
		SetAnimation( anim_speedbrake, speedbrake );
	}

	
	// Canopy Animations


   if( animactive5 ) {
		if( animactive5 == 1 ) {
			canopy += 0.003;
			if( canopy > 1.0 ) {
				canopy = 1.0;
				animactive5 = 0;
			}
		}
		else  {
			canopy -= 0.003;
			if( canopy < 0.0 ) {
				canopy = 0.0;
				animactive5 = 0;
			}
		}
		SetAnimation( anim_canopy, canopy );
	}
	
	// Throttle animation

        power_proc = GetThrusterLevel(th_main);
	
		SetAnimation( anim_power, power_proc );


	
   
   // Flaps


   if( animactive4 ) {
		if( animactive4 == 1 ) {
			flaps += 0.006;
			if( flaps > 1.0 ) {
				flaps = 1.0;
				animactive4 = 0;
			}
		}
		else  {
			flaps -= 0.006;
			if( flaps < 0.0 ) {
				flaps = 0.0;
				animactive4 = 0;
			}
		}
		SetAnimation( anim_flaps, flaps );
	}

   SetControlSurfaceLevel(AIRCTRL_FLAP,flaps);

   

// Engine sound
	
	
	
	
	if (engstart == 2) 
	{
		
	double pit = ((GetThrusterLevel (th_main)*6000));
	
	eng_pitch = 13000 + pit;
	
	if (eng_pitch < 11500 ) eng_pitch = 11500;
	
	if (eng_pitch > 22500 ) eng_pitch = 22500;
	
	//sprintf(oapiDebugString(),"anim %7.7f", eng_pitch );
		
	PlayVesselWave3(MySoundID,ENG,LOOP,255,eng_pitch);

}


    if (engstart == 0) 
	{
	
	StopVesselWave3(MySoundID,ENG);
}

if (engstart == 3) 
	{
	
	StopVesselWave3(MySoundID,ENG);
}
   
 if (Lights == 1 && switched == 1)
 {
	 beacon1.active = true;
	 beacon2.active = true;
	 switched = 0;
	 Lighting();
}

if (Lights == 0 && switched == 1)
{
	beacon1.active = false;
	beacon2.active = false;
    switched = 0;
    Lighting();
}



// Altimeter animation
   
   
    //Anims 
    
     k3 = GetAltitude();
    
     k4 = int (k3);
      
     gcvt (k3,6,k3buffer);
     
     

   
     string stringOne(k3buffer);
     
     if (k3 >= 0 )
     {

     string is1 = stringOne.substr(0 + s2 + s3 + s4 + s5, 1) ;
       
     char const *cString1 = is1.c_str();
     
     i1a = atol(cString1);
     
     i1 = i1a * 0.1;
     
 }
 
 if (k3 >= 10 ) 
 {
	 
     string is2 = stringOne.substr(0 + s3 + s4 + s5 , 1) ;
       
     char const *cString2 = is2.c_str();
     
     i2a = atol(cString2);
     
     i2 = i2a * 0.1;
      s2 = 1;
 }
 
 if (k3 >= 100 )
 {
     string is3 = stringOne.substr(0 + s4 + s5, 1) ;
       
     char const *cString3 = is3.c_str();
     
     i3a = atol(cString3);
     
     i3 = i3a * 0.1;
     
     s3 = 1;
 }
 
 if (k3 >= 1000)
 {
     string is4 = stringOne.substr(0 + s5, 1) ;
       
     char const *cString4 = is4.c_str();
     
     i4a = atol(cString4);
     
     i4 = i4a * 0.1;
     
     s4 = 1;
 }
 
 if (k3 >= 10000)
 {
     string is5 = stringOne.substr(0, 1) ;
       
     char const *cString5 = is5.c_str();
     
     i5a = atol(cString5);
     
     i5 = i5a * 0.1;
     
     s5 = 1;
}

      if(k3<10)
      {
	      s2=0;
	      i2=0;
	      s3=0;
	      i3=0;
	      s4=0;
	      i4=0;
	      s5=0;
	      i5=0;
    }
         
    if(k3<100)
      {
	     
	      s3=0;
	      i3=0;
	      s4=0;
	      i4=0;
	      s5=0;
	      i5=0;
    }
    
    if(k3<1000)
      {
	      
	      s4=0;
	      i4=0;
	      s5=0;
	      i5=0;
    }
   
    if(k3<10000)
      {
	     
	      s5=0;
	      i5=0;
	      
    }
    
    
   i=(k3 * 0.01)- dee;
   
   if (i>=1)
   {
	   dee = dee + 1;
   }
   
   if (i <= 0)
   {
	   dee = dee - 1;
	   
	   if (dee < 0)dee = 0;
   }
   
    
	//sprintf(oapiDebugString(),"force %2.2f", i1 );
	
	
	SetAnimation (anim_wheel0, i);
	SetAnimation (anim_wheel1, i1);
	SetAnimation (anim_wheel2, i2);
	SetAnimation (anim_wheel3, i3);
	SetAnimation (anim_wheel4, i4);
	SetAnimation (anim_wheel5, i5);
	
	


// AOA Dial Animation

    aoa = GetAOA();
    
    
   i7 = aoa * 6;

   SetAnimation (anim_dial1, i7);
   
   
   // Direction Dial Animation

   oapiGetFocusHeading(&dil);
    
   i9 = (dil / 6.25) + 0.001;
   
   //sprintf(oapiDebugString(),"hed %2.2f", dil );

   SetAnimation (anim_dial2, i9);
   
    // Refuel
   
   if (refuel == 1)
   {
	   
	  SetFuelMass (10000.0);

	   refuel = 0;
   }


   // Speed Dial Animation
    
   i13 = GetAirspeed();
   
   if (i13 > 600) i13 = 600;
   
   if (i13 < 0) i13 = 0;
   
   i14 = (-i13 + 600)/600;
   
   if (i14 < 0) i14 = 0;
   
   if (i14 > 1) i14 = 1;
   
   SetAnimation (anim_dial3, i14);
   
    // Bank Guage Animation
    
   i15 = GetBank();
   
   i16 = (i15 + 3) / 6;
   
   if (i16 < 0) i16 = 0;
   
   if (i16 > 1) i16 = 1;
   
   SetAnimation (anim_bank, i16);
   
    //sprintf(oapiDebugString(),"hed %5.5f", i15 );
    
    
     // Clock Dial Animation
     
     //Second Hand
     
    
   i17 = i17 + ( 0.0002 * oapiGetTimeAcceleration() );
   
   if (i17 < 0) i17 = 0;
   
   if (i17 >= 1) 
   {
	   i17 = 0;
	   i18 = i18 + 0.016666666;
}
   SetAnimation (anim_clock_s, i17);
   
   //sprintf(oapiDebugString(),"mouse %19.19f", mx1 );
   
    //Minute Hand
    
   if (i18 < 0) i18 = 0;
   
   if (i18 >= 1) 
   {
	   i18 = 0;
	   i19 = i19 + 0.016666666;
   }
   
   
   SetAnimation (anim_clock_m, i18);
   
    //Hour Hand
   
   if (i19 < 0) i19 = 0;
   
   if (i19 >= 1) i19 = 0;
   
   SetAnimation (anim_clock_h, i19);

 // Fuel Guage Animation
   
   i20 = (-GetFuelMass() + 10000)/10000; 

   i21 = i20 / 3;
   
   if (i21 < 0) i21 = 0;
   
   if (i21 > 1) i21 = 1;
   
   SetAnimation (anim_dial44, i21);

   
   
   
   
   
   
   
}

//Ground Steering


void ShuttlePB::Steering()
 {

	double pos = GetControlSurfaceLevel(AIRCTRL_RUDDER);
	double lev = 1e6 * pos * GetAirspeed() / 10.0;

	if( oapiGetTimeAcceleration() > 10.0 )
		return;

	AddForce( _V( lev,0,0 ), _V( 0,0,0.33 ) );
	AddForce( _V( -1.0*lev,0,0 ), _V( 0,0,-0.33 ) );
	
}

// Lighting


void ShuttlePB::Lighting()
 {
    if(Lights == 1)
    {
	
	SetMeshVisibilityMode( InsertMesh (f155,1,&offset), MESHVIS_VC );
	SetMeshVisibilityMode( InsertMesh (f156,2,&offsetalt), MESHVIS_VC );
	SetMeshVisibilityMode( InsertMesh (f157,3,&offsetaoa), MESHVIS_VC );
	SetMeshVisibilityMode( InsertMesh (f158,4,&offsetdirect), MESHVIS_VC );
	SetMeshVisibilityMode( InsertMesh (f160,5,&offsetspeed), MESHVIS_VC );
	SetMeshVisibilityMode( InsertMesh (f162,6,&offsetbank), MESHVIS_VC );
	SetMeshVisibilityMode( InsertMesh (f164,7,&offsetclock), MESHVIS_VC );
    SetMeshVisibilityMode( InsertMesh (f166,8,&offsetfuel), MESHVIS_VC );

	
}

    if(Lights == 0)
    {
	
	SetMeshVisibilityMode( InsertMesh (f151,1,&offset), MESHVIS_VC );
	SetMeshVisibilityMode( InsertMesh (f152,2,&offsetalt), MESHVIS_VC );
	SetMeshVisibilityMode( InsertMesh (f153,3,&offsetaoa), MESHVIS_VC );
	SetMeshVisibilityMode( InsertMesh (f154,4,&offsetdirect), MESHVIS_VC );
	SetMeshVisibilityMode( InsertMesh (f159,5,&offsetspeed), MESHVIS_VC );
	SetMeshVisibilityMode( InsertMesh (f161,6,&offsetbank), MESHVIS_VC );
	SetMeshVisibilityMode( InsertMesh (f163,7,&offsetclock), MESHVIS_VC );
    SetMeshVisibilityMode( InsertMesh (f165,8,&offsetfuel), MESHVIS_VC );

	
}

}


// --------------------------------------------------------------
// Keyboard interface handler (buffered key events)
// --------------------------------------------------------------
int ShuttlePB::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate)
{
	if (!down) return 0; // only process keydown events

	// if (KEYMOD_SHIFT (kstate)) {

		switch (key) {
		case OAPI_KEY_G: // Gear
		
		if( gear_proc > 0.5 )
					animactive = 2;
				else
					animactive = 1;

		if ( !GroundContact()) PlayVesselWave3(MySoundID,GEAR,NOLOOP,255);
		
		
		if( gear_procl > 0.5 )
					animactivel = 2;
				else
					animactivel = 1;

			
			return 1;
			
			
			case OAPI_KEY_B: // Speedbrake
		
		if( speedbrake > 0.5 )
					animactive2 = 2;
				else
					animactive2 = 1;

		PlayVesselWave3(MySoundID,BRAKE,NOLOOP,255);
		
			
			return 1;
			
			case OAPI_KEY_8: // Canopy
		
		if( canopy > 0.5 )
					animactive5 = 2;
				else
					animactive5 = 1;

		PlayVesselWave3(MySoundID,CANOPY,NOLOOP,255);
		
			
			return 1;
			
			case OAPI_KEY_J: // Refuel
		
		if ( GroundContact() && GetAirspeed() == 0)
		{
			refuel = 1;
								
			return 1;
}
			else
     		{
			refuel = 0;
								
			return 1;
}

			
			case OAPI_KEY_9: // Engine start
		
		if( engstart == 0 )
		{
			engstart = 1;
			
			PlayVesselWave3(MySoundID,START,NOLOOP,255);
			
			return 1;
		}
					
			if( engstart == 1 )
		{
			engstart = 1;
			
			return 1;
		}
		
		if( engstart == 2 )
		{
			engstart = 3;
			
			PlayVesselWave3(MySoundID,STOP,NOLOOP,255);
			
			return 1;
		}

			
			case OAPI_KEY_F: // Flaps
		
		if( flaps > 0.5 )
					animactive4 = 2;
				else
					animactive4 = 1;

		PlayVesselWave3(MySoundID,BRAKE,NOLOOP,255);
		
			
			return 1;
			
			case OAPI_KEY_K: // Lights
		
		if( Lights == 0 )
		{
			Lights = 1;
			switched =1;
					
			return 1;
}

		if( Lights == 1 )
		{
			Lights = 0;
			switched = 1;
					
			return 1;
}
	
    } 
	return 0;
}


void ShuttlePB::DefineAnimations(void)
{
   // Front Wheel
   static UINT WGrp1[1] = {224};
   
   static MGROUP_ROTATE wheel1 (0,WGrp1, 1, _V(0.2399261,-1.569521,4.206994), _V(1,0,0), (float)(360*RAD));
   
   //Rear left Wheel
   static UINT WGrp3[1] = {222};
   
   static MGROUP_ROTATE wheel2 (0,WGrp3, 1, _V(-2.194767,-1.527298,-0.383905), _V(1,0,0), (float)(360*RAD));
   
   //Rear Right wheel
   static UINT WGrp4[1] = {223};
   
   static MGROUP_ROTATE wheel3 (0,WGrp4, 1, _V(2.194767,-1.527298,-0.383905), _V(1,0,0), (float)(360*RAD));
   
   anim_wheel = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel1);
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel2);
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel3);
   
   
   
   
   // Gear anims
  
   
   //Front Gear
   static UINT GGrp1[12] = {45,46,47,48,49,50,51,52,69,196,197,224};
   
   static MGROUP_ROTATE gear1 (0,GGrp1, 12, _V(0,-0.3482238,3.616336), _V(1,0,0), (float)(-70*RAD));
   
   //Rear Gear Left
   static UINT GGrp2[12] = {53,54,55,56,57,58,59,60,70,222,239,240};
   
   static MGROUP_ROTATE gear2 (0,GGrp2, 12, _V(-2.109135,-0.45,-0.509176), _V(1,0,0), (float)(-75*RAD));
   
   //Rear Gear Right
   static UINT GGrp3[12] = {61,62,63,64,65,66,67,68,71,223,271,272};
   
   static MGROUP_ROTATE gear3 (0,GGrp3, 12, _V(2.109135,-0.45,-0.509176), _V(1,0,0), (float)(-75*RAD));
   
   //Front Gear Cover
   static UINT GGrp4[1] = {201};
   
   static MGROUP_ROTATE gear4 (0,GGrp4, 1, _V(0.3680046,-0.4644772,4.972941), _V(0,0,1), (float)(-90*RAD));
   
   
   
   anim_gear = CreateAnimation (0);
   
   AddAnimationComponent  (anim_gear, 0, 0.5, &gear1);
   AddAnimationComponent  (anim_gear, 0, 0.5, &gear2);
   AddAnimationComponent  (anim_gear, 0, 0.5, &gear3);
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear4);
   
   
   
   //Rudder left
   static UINT RGrp1[2] = {203,205};
   
   static MGROUP_ROTATE rudderl (0,RGrp1, 2, _V(-2.393519,0.9688046,-5.869215), _V(0,1,0), (float)(-33*RAD));
   
   
   //Rudder Pedals left
   static UINT RGrp3[2] = {83,85};
   
   static MGROUP_ROTATE rudderlp (1,RGrp3, 2, _V(0,0.34,0.4480002), _V(1,0,0), (float)(12*RAD));
   
   
   //Rudder right
   static UINT RGrp2[2] = {206,207};
   
   static MGROUP_ROTATE rudderr (0,RGrp2, 2, _V(2.393519,0.9688046,-5.869215), _V(0,1,0), (float)(-33*RAD));
   
   //Rudder Padals right
   static UINT RGrp4[2] = {84,86};
   
   static MGROUP_ROTATE rudderrp (1,RGrp4, 2, _V(0,0.34,0.4480002), _V(1,0,0), (float)(-12*RAD));
   
   
   anim_rudder = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderl);
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderr);
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderlp);
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderrp);
   
   
   //Elevator
   static UINT EGrp1[2] = {140,142};
   
   static MGROUP_ROTATE elevator (0,EGrp1, 2, _V(0,-7.355354E-02,-5.920062), _V(1,0,0), (float)(33*RAD));
   
   
   anim_elevator = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_elevator, 0, 1, &elevator);
   
   
   
   
   //Left Aleron
   static UINT LAGrp1[4] = {253,254,255,256};
   
   static MGROUP_ROTATE laileron (0,LAGrp1, 4, _V(-5.313836,-3.248717E-04,-0.8004674), _V(1,-0.1,0), (float)(-33*RAD));
   
   
   anim_laileron = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_laileron, 0, 1, &laileron);
   
   
   //Right Aleron
   static UINT RAGrp1[4] = {266,267,268,269};
   
   static MGROUP_ROTATE raileron (0,RAGrp1, 4, _V(5.313836,-3.248717E-04,-0.8004674), _V(1,0.1,0), (float)(33*RAD));
   
   
   anim_raileron = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_raileron, 0, 1, &raileron);
   
   
   //Speedbrake
   
   //Upper Left
   static UINT SBGrp1[2] = {253,254};
   
   static MGROUP_ROTATE speedbrake (0,SBGrp1, 2, _V(-5.313836,-3.248717E-04,-0.8004674), _V(1,-0.1,0), (float)(-33*RAD));
   
   //Lower Left
   static UINT SBGrp2[2] = {255,256};
   
   static MGROUP_ROTATE speedbrake2(0,SBGrp2, 2, _V(-5.313836,-3.248717E-04,-0.8004674), _V(1,-0.1,0), (float)(33*RAD));
   
   //Upper Right
   static UINT SBGrp3[2] = {268,269};
   
   static MGROUP_ROTATE speedbrake3 (0,SBGrp3, 2, _V(5.313836,-3.248717E-04,-0.8004674), _V(1,0.1,0), (float)(-33*RAD));
   
   //Lower Right
   static UINT SBGrp4[2] = {266,267};
   
   static MGROUP_ROTATE speedbrake4(0,SBGrp4, 2, _V(5.313836,-3.248717E-04,-0.8004674), _V(1,0.1,0), (float)(33*RAD));
   
   
   anim_speedbrake = CreateAnimation (0);
   
   AddAnimationComponent  (anim_speedbrake, 0, 1, &speedbrake);
   AddAnimationComponent  (anim_speedbrake, 0, 1, &speedbrake2);
   AddAnimationComponent  (anim_speedbrake, 0, 1, &speedbrake3);
   AddAnimationComponent  (anim_speedbrake, 0, 1, &speedbrake4);
  
   
   //Flaps animation
   
   //Left Flaps
   static UINT FLGrp1[2] = {257,258};
   
   static MGROUP_ROTATE flapsl (0,FLGrp1, 2, _V(-2.294927,-1,-0.9197722), _V(1,0,0), (float)(-33*RAD));
   
   
   //Right Flaps
   static UINT FRGrp1[2] = {259,260};
   
   static MGROUP_ROTATE flapsr (0,FRGrp1, 2, _V(2.294927,-1,-0.9197722), _V(1,0,0), (float)(-33*RAD));
   
   //Front Flaps
   static UINT FFGrp1[2] = {242,261};
   
   static MGROUP_ROTATE flapsf (0,FFGrp1, 2, _V(0,-1,0.5), _V(1,0,0), (float)(7*RAD));
   
   
   anim_flaps = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_flaps, 0, 1, &flapsl);
   AddAnimationComponent  (anim_flaps, 0, 1, &flapsr);
   AddAnimationComponent  (anim_flaps, 0, 1, &flapsf);
   
   
   //Canopy
   
   //Outside
   static UINT CGrp1[6] = {163,164,179,180,308,309};
   
   static MGROUP_ROTATE can1 (0,CGrp1, 6, _V(0,0.9169314,2.67), _V(1,0,0), (float)(-35*RAD));
   
   //Inside
   static UINT CGrp2[8] = {25,45,46,47,48,49,51,52};
   
   static MGROUP_ROTATE can2 (1,CGrp2, 8, _V(0,0.5660748,-1.054566), _V(1,0,0), (float)(-35*RAD));
   
   
   anim_canopy = CreateAnimation (0);
   
   AddAnimationComponent  (anim_canopy, 0, 1, &can1);
   AddAnimationComponent  (anim_canopy, 0, 1, &can2);

   
   
   // Altimeter Animations
   
   // Dial
   static UINT wWGrp0[1] = {1};
   
   static MGROUP_ROTATE wwheel0 (2,wWGrp0, 1, _V(0,0,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_wheel0 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel0, 0, 1, &wwheel0);
   
   // 1st dial from Left
   static UINT wWGrp1[10] = {23,24,25,26,27,28,29,30,31,32};
   
   static MGROUP_ROTATE wwheel1 (2,wWGrp1, 10, _V(0,7.199557e-03,0), _V(1,0,0), (float)(360*RAD));
   
   
   anim_wheel1 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel1, 0, 1, &wwheel1);
   
   // 2nd dial from Left
   static UINT wWGrp2[10] = {13,14,15,16,17,18,19,20,21,22};
   
   static MGROUP_ROTATE wwheel2 (2,wWGrp2, 10, _V(0,7.199557e-03,0), _V(1,0,0), (float)(360*RAD));
   
   
   anim_wheel2 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel2, 0, 1, &wwheel2);
   
   // 3rd dial from Left
   static UINT wWGrp3[10] = {3,4,5,6,7,8,9,10,11,12};
   
   static MGROUP_ROTATE wwheel3 (2,wWGrp3, 10, _V(0,7.199557e-03,0), _V(1,0,0), (float)(360*RAD));
   
   
   anim_wheel3 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel3, 0, 1, &wwheel3);
   
   // 4th dial from Left
   static UINT wWGrp4[10] = {43,44,45,46,47,48,49,50,51,52};
   
   static MGROUP_ROTATE wwheel4 (2,wWGrp4, 10, _V(0,7.199557e-03,0.785718e-03), _V(1,0,0), (float)(360*RAD));
   
   
   anim_wheel4 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel4, 0, 1, &wwheel4);
   
   // 5th dial from Left
   static UINT wWGrp5[10] = {33,34,35,36,37,38,39,40,41,42};
   
   static MGROUP_ROTATE wwheel5 (2,wWGrp5, 10, _V(0,7.199557e-03,0.785718e-03), _V(1,0,0), (float)(360*RAD));
   
   
   anim_wheel5 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel5, 0, 1, &wwheel5);
   
   
   
   // AOA Animations
   
   // Dial1
   static UINT dGrp0[1] = {1};
   
   static MGROUP_ROTATE dial1 (3,dGrp0, 1, _V(0,0,0), _V(0,0,1), (float)(311*RAD));
   
   
   anim_dial1 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_dial1, 0, 1, &dial1);
   

    // Direction Dial Animations
   
   // Dial2
   static UINT d2Grp0[1] = {0};
   
   static MGROUP_ROTATE dial2 (4,d2Grp0, 1, _V(0,0,0), _V(0,0,1), (float)(360*RAD));
   
   
   anim_dial2 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_dial2, 0, 1, &dial2);
   
   
   // Gear Lever Animation
   
   //Gear Lever in Cockpit
   static UINT Glev0[2] = {3,4};
   
   static MGROUP_ROTATE gearlev (1,Glev0, 2, _V(-0.2657341,0.1866311,0.2689739), _V(1,0,0), (float)(41*RAD));
   
   anim_gearlever = CreateAnimation (0);
   
   AddAnimationComponent  (anim_gearlever, 0, 1, &gearlev);
   
    // Engine turbines
    
   // Left Eng
   static UINT TGrp1[2] = {111,112};
   
   static MGROUP_ROTATE eng1 (0,TGrp1, 2, _V(-1.17378,0.7121599,-1.407674), _V(0,0,1), (float)(360*RAD));
   
   // Right Eng
   static UINT TGrp2[2] = {113,114};
   
   static MGROUP_ROTATE eng2 (0,TGrp2, 2, _V(1.17378,0.7121599,-1.407674), _V(0,0,1), (float)(360*RAD));
   
   
   anim_eng = CreateAnimation (0);
   
   AddAnimationComponent  (anim_eng, 0, 1, &eng1);
   AddAnimationComponent  (anim_eng, 0, 1, &eng2);

   // Throttle Animation
   
   static UINT Pow0[2] = {87,88};
   
   static MGROUP_TRANSLATE powg (1,Pow0, 2, _V(0,0,0.06));
   
   anim_power = CreateAnimation (0);
   
   AddAnimationComponent  (anim_power, 0, 1, &powg);
   
   
   // Speed Dial Animations
   
   // Dial3
   static UINT d3Grp0[2] = {0,1};
   
   static MGROUP_ROTATE dial3 (5,d3Grp0, 2, _V(0,0,0), _V(0,0,1), (float)(360*RAD));
   
   
   anim_dial3 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_dial3, 0, 1, &dial3);
   
   
   // Bank Guage Animations
   
   static UINT BAGrp0[2] = {0,2};
   
   static MGROUP_ROTATE bank (6,BAGrp0, 2, _V(0,0,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_bank = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_bank, 0, 1, &bank);
   
   // Clock
   
   // Second Hand
   static UINT dGrp7[1] = {4};
   
   static MGROUP_ROTATE dial7 (7,dGrp7, 1, _V(0,0,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_clock_s = CreateAnimation (0);
   
   AddAnimationComponent  (anim_clock_s, 0, 1, &dial7);
   
   // Minute Hand
   static UINT dGrp8[1] = {3};
   
   static MGROUP_ROTATE dial8 (7,dGrp8, 1, _V(0,0,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_clock_m = CreateAnimation (0);
   
   AddAnimationComponent  (anim_clock_m, 0, 1, &dial8);
   
   // Hour Hand
   static UINT dGrp9[1] = {1};
   
   static MGROUP_ROTATE dial9 (7,dGrp9, 1, _V(0,0,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_clock_h = CreateAnimation (0);
   
   AddAnimationComponent  (anim_clock_h, 0, 1, &dial9);


   // Fuel Guage Animations
   
   // Dial4
   static UINT dGrp44[1] = {0};
   
   static MGROUP_ROTATE dial44 (8,dGrp44, 1, _V(0,0,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_dial44 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_dial44, 0, 1, &dial44);
   


   
}
	

bool ShuttlePB::clbkLoadVC (int id)
{
	
	SetCameraOffset (_V(0,0.9,3.7));
	
    SetCameraDefaultDirection (_V(0,0,1));

    SetCameraRotationRange (RAD*65, RAD*65, RAD*45, RAD*45);
	
	static VCHUDSPEC hud_pilot  = {1, 91,{0,0.95,4.5},0.5};
	

	
		
		
		
		
		static VCMFDSPEC mfds_right= {1, 89};
		
       
        oapiVCRegisterMFD (MFD_RIGHT, &mfds_right);
        oapiVCRegisterHUD (&hud_pilot);

		
	
	
 return true;

}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new ShuttlePB (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (ShuttlePB*)vessel;
}
