// ==============================================================
//                 ORBITER MODULE: ShuttlePB
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// ShuttlePB.cpp
// Control module for ShuttlePB vessel class
//
// Notes:
// This is an example for a "minimal" vessel implementation which
// only overloads the clbkSetClassCaps method to define vessel
// capabilities and otherwise uses the default VESSEL class
// behaviour.
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#define ENG 				10	
#define GEAR				11	
#define SWING				12	

#include <orbitersdk.h>
#include <cstdio>
#include <cmath>


#include <string>
#include "OrbiterSoundSDK35.h"

#include <stdio.h>
#include <stdlib.h>

using namespace std;


// ==============================================================
// Some vessel parameters
// ==============================================================
const double PB_FUELMASS = 20000.0;
const double PB_ISP = 50000.0;
const double PB_MAXMAINTH =0;

double iop = 0;
double st = 0;
double pld = 0;
double cen = 0;
double gear_proc = 0;
double gear_proc2 = 0;
double gear_procl = 0;
double power_proc = 0;
double fuel_proc = 0;
//double speedbrake = 0;
double swingwing = 0;
double swingwing2 = 0;
double swing = 0;
double swing3 = 0;
//double hook = 0;
//double canopy = 0;
//double flaps = 0;
//double flaps2 = 0;
double exproc = 0;
double Lights = 0;
double switched = 0;
double aoa;
double aoa2 = 0;

double refuel = 0;


double dil = 0;
double eng_pitch = 22000;

double start = 0;
double full = 0.001;
double i = 0;
double i1 = 0;
double i2 = 0;
double i3 = 0;
double i4 = 0;
double i5 = 0;
double i7 = 0;
double i9 = 0;
double i1a = 0;
double i2a = 0;
double i3a = 0;
double i4a = 0;
double i5a = 0;
double s1 = 0;
double s2 = 0;
double s3 = 0;
double s4 = 0;
double s5 = 0;
double ga = 0;
double diff = 0.1;
double k3;
int k4;
char k3buffer [33];
double amet = 0;
double dee = 0;
double i13 = 0;
double i14 = 0;
double i15 = 0;
double i16 = 0;
double i17 = 0;
double i18 = 0;
double i19 = 0;
double secondtime = 0;

double tpp = 0.5;

double stick3 = 0;
double stick4 = 0;

double stick3b = 0;
double stick4b = 0;

UINT animactive;

UINT animactivel;

UINT animactive2;

UINT animactive3;

UINT animactive4;

UINT animactive5;

UINT animactive7;

UINT animactive9;

const VECTOR3 offset = {0,0,10};


THRUSTER_HANDLE th_main;

BEACONLIGHTSPEC beacon1;    

//BEACONLIGHTSPEC beacon2;   

MESHHANDLE f150;
MESHHANDLE f151;

MESHHANDLE f155;



// 1. vertical lift component (wings and body)

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	static const double CL[nabsc] = {0.1, 0.17, 0.2, 0.2, 0.17, 0.1, 0, -0.11, -0.24, -0.38,  -0.5,  -0.5, -0.02, 0.6355,    0.63,   0.46, 0.28, 0.13, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,    0,0.002,0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = CM[idx] + (CM[idx+1]-CM[idx])*d;
	*cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.02 + oapiGetInducedDrag (*cl, 1.5, 0.6);
}


// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class ShuttlePB: public VESSEL2 {
public:
	ShuttlePB (OBJHANDLE hVessel, int flightmodel);
	//	: VESSEL2 (hVessel, flightmodel) {}
	void clbkSetClassCaps (FILEHANDLE cfg);
	
	~ShuttlePB();
	
	void DefineAnimations(void);
	void Steering();
	void Lighting();
	void clbkPostCreation (void);
    void clbkPostStep (double simt, double simdt, double mjd);
    int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
   
    UINT anim_wheel;
    UINT anim_gear;
    UINT anim_gearlever;
    UINT anim_rudder;
    UINT anim_elevator;
    UINT anim_laileron;
    UINT anim_raileron;
    //UINT anim_speedbrake;
    UINT anim_swingwing;
   // UINT anim_flaps;
    //UINT anim_hook;
   // UINT anim_canopy;
    UINT anim_power;
    UINT anim_fuel;
    
    UINT anim_wheel0;
    UINT anim_wheel1;
    UINT anim_wheel2;
    UINT anim_wheel3;
    UINT anim_wheel4;
    UINT anim_wheel5;
    
    UINT anim_dial1;
    UINT anim_dial2;
    UINT anim_dial3;
    UINT anim_bank;
    UINT anim_clock_s;
    UINT anim_clock_m;
    UINT anim_clock_h;
    
    UINT anim_turn;
    
    UINT anim_stick1;
    UINT anim_stick2;
    
    UINT anim_stick1b;
    UINT anim_stick2b;

     MGROUP_TRANSFORM *wheel1;
    
     MGROUP_TRANSFORM *stick1;
     MGROUP_TRANSFORM *stick1b;
    
    bool clbkLoadVC (int id);
    
    int MySoundID;	


};

ShuttlePB::ShuttlePB (OBJHANDLE hVessel, int flightmodel)
		: VESSEL2 (hVessel, flightmodel)
{
	
	// preload meshes
	
    f150                 = oapiLoadMeshGlobal ("K33\\K-B-1B");
    f151                 = oapiLoadMeshGlobal ("K33\\K-B-1B-Cpit");
    f155                 = oapiLoadMeshGlobal ("K33\\K-B-1B-Cpit_n");
   
	
	DefineAnimations ();
}


void ShuttlePB::clbkPostCreation (void)
{

int MySoundID=ConnectToOrbiterSoundDLL3(GetHandle());


RequestLoadVesselWave3(MySoundID,ENG,"Sound\\K-B-1B\\K-B-1B-Eng.wav",BOTHVIEW_FADED_FAR);


RequestLoadVesselWave3(MySoundID,GEAR,"Sound\\K-B-1B\\K-B-1B-Gear.wav",BOTHVIEW_FADED_FAR);

RequestLoadVesselWave3(MySoundID,SWING,"Sound\\K-B-1B\\K-B-1B-Swing.wav",BOTHVIEW_FADED_FAR);

SoundOptionOnOff3(MySoundID,PLAYMAINTHRUST,FALSE);

SoundOptionOnOff3(MySoundID,PLAYHOVERTHRUST,FALSE);


SoundOptionOnOff3(MySoundID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);

SoundOptionOnOff3(MySoundID,PLAYCABINAIRCONDITIONING,FALSE);

SoundOptionOnOff3(MySoundID,PLAYCABINRANDOMAMBIANCE,FALSE);

SoundOptionOnOff3(MySoundID,PLAYRADIOATC,FALSE);

SoundOptionOnOff3(MySoundID,DISPLAYTIMER,FALSE);

}


// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------

void ShuttlePB::clbkSetClassCaps (FILEHANDLE cfg)
{
	// THRUSTER_HANDLE th_main;


	// physical specs
	SetSize (8);
	SetEmptyMass (12000.0);
	SetCW (0.3, 0.3, 0.6, 0.9);
	SetWingAspect (1.7);
	SetWingEffectiveness (3.1);
	SetCrossSections (_V(5,5,15));
	SetRotDrag (_V(3.5,3.5,3.5));
	if (GetFlightModel() >= 1) {
		SetPitchMomentScale (1e-4);
		SetBankMomentScale (1e-4);
	}
	SetPMI (_V(15.5,22.1,7.7));
	SetTrimScale (0.05);
	SetCameraOffset (_V(-1.8,0,5.2));
	
	SetTouchdownPoints (_V(0,-3.50421,9.771303), _V(-1.4,-3.50421,-2.77804), _V(1.4,-3.50421,-2.77804));

	// **************************** aerodynamics ***********************************

	CreateAirfoil (LIFT_VERTICAL,   _V(0,0,-0.5), VLiftCoeff, 20, 270, 2.266);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,-4), HLiftCoeff, 20,  50, 1.5);





	CreateControlSurface (AIRCTRL_ELEVATOR,     7, 0.7, _V(   0,0,-17), AIRCTRL_AXIS_XPOS,anim_elevator);
	CreateControlSurface (AIRCTRL_RUDDER,       2, 0.5, _V(   0,2,-15), AIRCTRL_AXIS_YPOS,anim_rudder);
	CreateControlSurface (AIRCTRL_AILERON,      4.8, 0.2, _V( 17.8,0,-17.8), AIRCTRL_AXIS_XPOS,anim_raileron);
	CreateControlSurface (AIRCTRL_AILERON,      4.8, 0.2, _V(-17.8,0,-17.8), AIRCTRL_AXIS_XNEG,anim_laileron);
	CreateControlSurface (AIRCTRL_ELEVATORTRIM, 5, 0.1, _V(   0,0,-15), AIRCTRL_AXIS_XPOS,anim_elevator);
    CreateControlSurface (AIRCTRL_FLAP,         5, 5, _V( 0,0,-1), AIRCTRL_AXIS_XPOS,anim_swingwing);
	
	//CreateVariableDragElement( &speedbrake, 8.0, _V(0,0,0) );

	CreateVariableDragElement (&gear_proc2, 0.8, _V(0, -1, 0));
	
	CreateVariableDragElement (&swing3, 0.8, _V(0,0,0));
	
	//CreateVariableDragElement (&canopy, 3.33, _V(0,1,0));

	
	// propellant resources
	PROPELLANT_HANDLE hpr = CreatePropellantResource (PB_FUELMASS);
	
	
	SURFHANDLE tex_main = oapiRegisterExhaustTexture ("K-B-1B-Exhaust");



	// ***************** thruster definitions *******************

	PARTICLESTREAMSPEC contrail_main = {
		0, 0.34, 165, 19, 0.019, 0.111, 1.11, 1.11, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 2,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};
	
	th_main = CreateThruster (_V(0,0,-5), _V(0,0,1), PB_MAXMAINTH, hpr, PB_ISP);
	
	
	
	CreateThrusterGroup (&th_main, 1, THGROUP_MAIN);
	
	
	
	AddExhaust (th_main, 7, 0.55, _V(-2.642622,-1.770356,-7.34), _V(0,0,-1) , tex_main);
	AddExhaust (th_main, 7, 0.55, _V(2.642622,-1.770356,-7.34), _V(0,0,-1) , tex_main);
    AddExhaust (th_main, 7, 0.55, _V(-1.655584,-1.770356,-7.34), _V(0,0,-1) , tex_main);
	AddExhaust (th_main, 7, 0.55, _V(1.655584,-1.770356,-7.34), _V(0,0,-1) , tex_main);
    
    AddExhaustStream (th_main, _V(-2.642622,-1.770356,-7.4), &contrail_main);
    AddExhaustStream (th_main, _V(2.642622,-1.770356,-7.4), &contrail_main);
    AddExhaustStream (th_main, _V(-1.655584,-1.770356,-7.4), &contrail_main);
    AddExhaustStream (th_main, _V(1.655584,-1.770356,-7.4), &contrail_main);
    
    
     // Strobe lights
	
	//Red Tail strobe
	
	static VECTOR3 beacon1pos = {0,3.40047,-12.92826};
	static VECTOR3 beacon1col = {0.988, 0.133, 0.133};
	
		beacon1.shape = (BEACONSHAPE_STAR);
		beacon1.pos = &beacon1pos;
		beacon1.col = &beacon1col;
		beacon1.size = (0.22);
		beacon1.falloff = (0.6);
		beacon1.period = (1.13);
		beacon1.duration = (0.05);
		beacon1.tofs = (0.6);
		beacon1.active = false;
		
		AddBeacon (&beacon1);
		
		

    
    
    
    
	// visual specs
	
	AddMesh (f150);
	
    SetMeshVisibilityMode( AddMesh (f151,&offset), MESHVIS_VC );
    

}

// ==============================================================
// API callback interface
// ==============================================================

void ShuttlePB::clbkPostStep (double simt, double simdt, double mjd)

{
	
	if( oapiGetTimeAcceleration() > 10.0 ) oapiSetTimeAcceleration(10);
	
	double vsAirSpeed = GetAirspeed();
	
	
	
	
    //speed and altitude control
	
	if (GetAirspeed() >= 700) 
	{
		SetCW (999, 999, 999, 999);
		SetThrusterMax0 (th_main ,0);

	
}

if (GetAltitude() >= 17000) 
	{
		SetCW (999, 999, 999, 999);
		SetThrusterMax0 (th_main ,0);

	
}



if (GetAirspeed() < 700 && GetAltitude() < 17000 ) 
	{
		SetCW (0.3, 0.3, 0.67, 0.92);
		SetThrusterMax0 (th_main ,4.4e5);

	
}

	double vsAirSpeed2 = (vsAirSpeed/1000)*3600;
    
    //Ground Steering
    
    if (GroundContact () ) Steering();
    
    
    //aoa = GetAOA();
    
    //sprintf(oapiDebugString(),"anim %2.2f", aoa );
    
  
    if (  gear_proc < 0.05 ) SetTouchdownPoints (_V(0,-3.50421,9.771303), _V(-1.4,-3.50421,-2.77804), _V(1.4,-3.50421,-2.77804));

    


    if (  gear_proc > 0.95 ) SetTouchdownPoints (_V(0,-1.622193,9.771303), _V(-1.4,-1.622193,-2.77804), _V(1.4,-1.622193,-2.77804));

    

    //Anims 
    
    if (iop > 1)iop=0;
    
    if (vsAirSpeed > 0) {
    
	double aspeed = vsAirSpeed;
    if (iop > 1)iop=0;
    if (vsAirSpeed > 30) aspeed = 30; 
    iop = iop + (aspeed /300);
     
    if (vsAirSpeed ==0) iop=0;
    
    
    if ( GroundContact() && gear_proc < 0.05) SetAnimation (anim_wheel, iop);
    

   }
	
	 // Front Gear Turn Anims1
   if (!GroundContact() && gear_proc < 1.0 && vsAirSpeed > 0.6)
   {
	   tpp = 0.5;
	   SetAnimation( anim_turn, tpp );
}

   // Front Gear Turn Anims2
   if (GroundContact() && gear_proc < 0.05 )
   {
	   
   tpp = (GetControlSurfaceLevel (AIRCTRL_RUDDER) +1) *0.5;
   if (!GroundContact())tpp = 0.5;
   SetAnimation( anim_turn, tpp );
   
}

   
   
   
   
// Gear animations


   if( animactive && !GroundContact()) {
		if( animactive == 1 ) {
			gear_proc += 0.002;
			if( gear_proc > 1.0 ) {
				gear_proc = 1.0;
				animactive = 0;
			}
		}
		else  {
			gear_proc -= 0.002;
			if( gear_proc < 0.0 ) {
				gear_proc = 0.0;
				animactive = 0;
			}
		}
		SetAnimation( anim_gear, gear_proc );
		gear_proc2 = -gear_proc + 1;
	}

	
	// Gear Lever animations


   if( animactivel && !GroundContact()) {
		if( animactivel == 1 ) {
			gear_procl += 0.00242;
			if( gear_procl > 1.0 ) {
				gear_procl = 1.0;
				animactivel = 0;
			}
		}
		else  {
			gear_procl -= 0.00242;
			if( gear_procl < 0.0 ) {
				gear_procl = 0.0;
				animactivel = 0;
			}
		}
		SetAnimation( anim_gearlever, gear_procl );
	}

	
	// Power Guage animation

        power_proc = GetThrusterLevel(th_main);
	
		SetAnimation( anim_power, power_proc );
	
  
   
     // Swingwing animations


   if( vsAirSpeed <= 150 ) {
		
			swingwing += 0.00133;
			if( swingwing > 1.0 )swingwing = 1.0;
			
				
				swingwing2 = swingwing;
				SetControlSurfaceLevel(AIRCTRL_FLAP,swingwing2);
			}
			
	if( vsAirSpeed > 150 ) {
		
			swingwing -= 0.00133;
			if( swingwing < 0.0 )swingwing = 0.0;
			
			
				swingwing2 = swingwing;
				SetControlSurfaceLevel(AIRCTRL_FLAP,swingwing2);
			}

		SetAnimation( anim_swingwing, swingwing );
		swingwing2 = swingwing;
		swing3 = -swingwing + 1;
		SetControlSurfaceLevel(AIRCTRL_FLAP,swingwing2);
		if (swingwing > 0 && swingwing < 1) PlayVesselWave3(MySoundID,SWING,LOOP,255,300);
		if (swingwing == 0)StopVesselWave3(MySoundID,SWING);
		if (swingwing == 1)StopVesselWave3(MySoundID,SWING);

		

// Engine sound
	
	
	
	
	if (GetThrusterLevel (th_main) > 0) 
	{
		
	double pit = ((GetThrusterLevel (th_main)*6000));
	
	eng_pitch = 14000 + pit;
	
	if (eng_pitch < 11500 ) eng_pitch = 11500;
	
	if (eng_pitch > 22500 ) eng_pitch = 22500;
	
	//sprintf(oapiDebugString(),"anim %7.7f", eng_pitch );
		
	if ( oapiCameraInternal ())
	{
	PlayVesselWave3(MySoundID,ENG,LOOP,220,eng_pitch - 800);
}

else

	PlayVesselWave3(MySoundID,ENG,LOOP,255,eng_pitch);
	
}


    if (GetThrusterLevel (th_main) == 0) 
	{
	
	StopVesselWave3(MySoundID,ENG);
	
}
   

// Lights

 if (Lights == 1 && switched == 1)
 {
	 beacon1.active = true;
	// beacon2.active = true;

	 switched = 0;
	 Lighting();
}

if (Lights == 0 && switched == 1)
{
	beacon1.active = false;
	//beacon2.active = false;

    switched = 0;
    Lighting();
}



// Altimeter animation
   
   
    //Anims 
    
     k3 = GetAltitude();
    
     k4 = int (k3);
      
     gcvt (k3,6,k3buffer);
     
     

   
     string stringOne(k3buffer);
     
     if (k3 >= 0 )
     {

     string is1 = stringOne.substr(0 + s2 + s3 + s4 + s5, 1) ;
       
     char const *cString1 = is1.c_str();
     
     i1a = atol(cString1);
     
     i1 = i1a * 0.1;
     
 }
 
 if (k3 >= 10 ) 
 {
	 
     string is2 = stringOne.substr(0 + s3 + s4 + s5 , 1) ;
       
     char const *cString2 = is2.c_str();
     
     i2a = atol(cString2);
     
     i2 = i2a * 0.1;
      s2 = 1;
 }
 
 if (k3 >= 100 )
 {
     string is3 = stringOne.substr(0 + s4 + s5, 1) ;
       
     char const *cString3 = is3.c_str();
     
     i3a = atol(cString3);
     
     i3 = i3a * 0.1;
     
     s3 = 1;
 }
 
 if (k3 >= 1000)
 {
     string is4 = stringOne.substr(0 + s5, 1) ;
       
     char const *cString4 = is4.c_str();
     
     i4a = atol(cString4);
     
     i4 = i4a * 0.1;
     
     s4 = 1;
 }
 
 if (k3 >= 10000)
 {
     string is5 = stringOne.substr(0, 1) ;
       
     char const *cString5 = is5.c_str();
     
     i5a = atol(cString5);
     
     i5 = i5a * 0.1;
     
     s5 = 1;
}

      if(k3<10)
      {
	      s2=0;
	      i2=0;
	      s3=0;
	      i3=0;
	      s4=0;
	      i4=0;
	      s5=0;
	      i5=0;
    }
         
    if(k3<100)
      {
	     
	      s3=0;
	      i3=0;
	      s4=0;
	      i4=0;
	      s5=0;
	      i5=0;
    }
    
    if(k3<1000)
      {
	      
	      s4=0;
	      i4=0;
	      s5=0;
	      i5=0;
    }
   
    if(k3<10000)
      {
	     
	      s5=0;
	      i5=0;
	      
    }
    
    
   i=(k3 * 0.01)- dee;
   
   if (i>=1)
   {
	   dee = dee + 1;
   }
   
   if (i <= 0)
   {
	   dee = dee - 1;
	   
	   if (dee < 0)dee = 0;
   }
   
    
	//sprintf(oapiDebugString(),"force %2.2f", i1 );
	
	
	SetAnimation (anim_wheel0, i);
	//SetAnimation (anim_wheel1, i1);
	//SetAnimation (anim_wheel2, i2);
	//SetAnimation (anim_wheel3, i3);
	//SetAnimation (anim_wheel4, i4);
	//SetAnimation (anim_wheel5, i5);
	
	



   
   
   // Speed Dial Animation
    
   i13 = GetAirspeed();
   
   if (i13 > 600) i13 = 600;
   
   if (i13 < 0) i13 = 0;
   
   i14 = (-i13 + 600)/600;
   
   if (i14 < 0) i14 = 0;
   
   if (i14 > 1) i14 = 1;
   
   SetAnimation (anim_dial3, i14);
   
    // Bank Guage Animation
    
   i15 = GetBank();
   
   i16 = (i15 + 3) / 6;
   
   if (i16 < 0) i16 = 0;
   
   if (i16 > 1) i16 = 1;
   
   SetAnimation (anim_bank, i16);
   
    //sprintf(oapiDebugString(),"hed %5.5f", i15 );
    
    
    // Refuel
   
   if (refuel == 1)
   {
	   
	  SetFuelMass (20000.0);

	   refuel = 0;
   }
 

   // Head Gyroscopic and G-Force movement
   
    if (vsAirSpeed > 0.65) {
		aoa2 = (GetAOA()* 0.5);
		
		if (aoa2 > 0.1333) aoa2 = 0.1333;
		
		if (aoa2 < -0.1333) aoa2 = -0.1333;
		
		//SetCameraOffset (_V(-1.8,0,5.2));
		SetCameraOffset (_V(-1.8,0 + aoa2,5.2));
		
}
   
//sprintf(oapiDebugString(),"aoa %19.19f", aoa2 );

if (vsAirSpeed <= 0.65) {
	
	aoa2 = 0;
			
	
		//SetCameraOffset (_V(-1.8,0,5.2));
		SetCameraOffset (_V(-1.8,0 + aoa2,5.2));
	}


	
	// Stick Animation
		
		
		stick3 = (GetControlSurfaceLevel (AIRCTRL_ELEVATOR) + 1) * 0.5;
		stick4 = (GetControlSurfaceLevel (AIRCTRL_AILERON) + 1) * 0.5 ;
		
		SetAnimation( anim_stick1, stick4 );
		SetAnimation( anim_stick2, stick3 );
		
		stick3b = (GetControlSurfaceLevel (AIRCTRL_ELEVATOR) + 1) * 0.5;
		stick4b = (GetControlSurfaceLevel (AIRCTRL_AILERON) + 1) * 0.5 ;
		
		SetAnimation( anim_stick1b, stick4b );
		SetAnimation( anim_stick2b, stick3b );



}

//Ground Steering


void ShuttlePB::Steering()
 {

	double pos = GetControlSurfaceLevel(AIRCTRL_RUDDER);
	double lev = 3e6 * pos ;

	if( oapiGetTimeAcceleration() > 10.0 )
		return;

	if( GetAirspeed() < 1.6 )
		return;
		
		if( GetAirspeed() >= 70 )
		return;
		
	AddForce( _V( lev,0,0 ), _V( 0,0,3.33 ) );
	AddForce( _V( -1.0*lev,0,0 ), _V( 0,0,-3.33 ) );

	
}

// Lighting


void ShuttlePB::Lighting()
 {
    if(Lights == 1)
    {
	
	SetMeshVisibilityMode( InsertMesh (f155,1,&offset), MESHVIS_VC );
	
}

    if(Lights == 0)
    {
	
	SetMeshVisibilityMode( InsertMesh (f151,1,&offset), MESHVIS_VC );
	
}

}


// --------------------------------------------------------------
// Keyboard interface handler (buffered key events)
// --------------------------------------------------------------
int ShuttlePB::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate)
{
	if (!down) return 0; // only process keydown events

	// if (KEYMOD_SHIFT (kstate)) {

		switch (key) {
		case OAPI_KEY_G: // Gear
		
		if( gear_proc > 0.5 )
					animactive = 2;
				else
					animactive = 1;

		if ( !GroundContact()) PlayVesselWave3(MySoundID,GEAR,NOLOOP,255);
		
		
		if( gear_procl > 0.5 )
					animactivel = 2;
				else
					animactivel = 1;

			
			return 1;
		
    case OAPI_KEY_J: // Refuel
			
			
		if (refuel == 0 && GetAirspeed() == 0)
		{
			refuel = 1;
		}
		
			
			return 1;


			case OAPI_KEY_K: // Lights
		
		if( Lights == 0 )
		{
			Lights = 1;
			switched =1;
					
			return 1;
}

		if( Lights == 1 )
		{
			Lights = 0;
			switched = 1;
					
			return 1;
}
	
    } 
	return 0;
}


// --------------------------------------------------------------
// Destructor
// --------------------------------------------------------------
ShuttlePB::~ShuttlePB ()
{
	delete wheel1;
	delete stick1;
	delete stick1b;
	

}



void ShuttlePB::DefineAnimations(void)
{
	
	ANIMATIONCOMPONENT_HANDLE parent1;
	
    ANIMATIONCOMPONENT_HANDLE parent3;
    
    ANIMATIONCOMPONENT_HANDLE parent3b;

	
   // Front Wheel
   static UINT WGrp1[2] = {58,59};
   
   wheel1 = new MGROUP_ROTATE  (0,WGrp1, 2, _V(0,-3.193553,9.772344), _V(1,0,0), (float)(360*RAD));
   
   //Turn Front Gear
   static UINT Trn1[4] = {0,3,4,9};
   
   static MGROUP_ROTATE turn1 (0,Trn1, 4, _V(0,-1.536483,9.840845), _V(0,1,0), (float)(70*RAD));
   

   
   
   //Rear left Wheel
   static UINT WGrp3[4] = {60,61,66,67};
   
   static MGROUP_ROTATE wheel2 (0,WGrp3, 4, _V(0,-3.08134,-2.77363), _V(1,0,0), (float)(360*RAD));
   
   //Rear Right wheel
   static UINT WGrp4[4] = {62,63,64,65};
   
   static MGROUP_ROTATE wheel3 (0,WGrp4, 4, _V(0,-3.08134,-1.765635), _V(1,0,0), (float)(360*RAD));
   
   anim_turn = CreateAnimation (0.5);

   anim_wheel = CreateAnimation (0);
   
   parent1 = AddAnimationComponent  (anim_turn, 0, 1, &turn1);

   AddAnimationComponent  (anim_wheel, 0, 1, wheel1, parent1);
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel2);
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel3);
   
   
   
   
   // Gear anims
  
   
   //Front Gear
   static UINT GGrp1[9] = {0,1,2,3,4,9,34,58,59};
   
   static MGROUP_ROTATE gear1 (0,GGrp1, 9, _V(0,-1.520341,9.744677), _V(1,0,0), (float)(-111*RAD));
   
   
   //Rear Gear Left
   static UINT GGrp2[7] = {5,6,10,60,61,62,63};
   
   static MGROUP_ROTATE gear2 (0,GGrp2, 7, _V(-1.105511,-0.9846452,-2.238096), _V(0,0,1), (float)(20*RAD));
   
   //Rear Gear Right
   static UINT GGrp3[7] = {7,8,11,64,65,66,67};
   
   static MGROUP_ROTATE gear3 (0,GGrp3, 7, _V(1.105511,-0.9846452,-2.238096), _V(0,0,1), (float)(-20*RAD));
   
   //Front Gear Cover Left
   static UINT GGrp4[1] = {32};
   
   static MGROUP_ROTATE gear4 (0,GGrp4, 1, _V(-0.3352448,-1.451328,0), _V(0,0,1), (float)(90*RAD));
   
   //Front Gear Cover Right
   static UINT GGrp5[1] = {33};
   
   static MGROUP_ROTATE gear5 (0,GGrp5, 1, _V(0.3352448,-1.451328,0), _V(0,0,1), (float)(-90*RAD));
   
   //Rear Gear Left2
   static UINT GGrp8[7] = {5,6,10,60,61,62,63};
   
   static MGROUP_ROTATE gear8 (0,GGrp8, 7, _V(-1.105511,-0.9846452,-2.238096), _V(1,0,0), (float)(-108.8*RAD));
   
   //Rear Gear Right2
   static UINT GGrp9[7] = {7,8,11,64,65,66,67};
   
   static MGROUP_ROTATE gear9 (0,GGrp9, 7, _V(1.105511,-0.9846452,-2.238096), _V(1,0,0), (float)(-108.8*RAD));
   
   
   //Rear Gear Left3
   static UINT GGrp11[7] = {5,6,10,60,61,62,63};
   
   static MGROUP_ROTATE gear11 (0,GGrp11, 7, _V(-1.105511,-0.9846452,-2.238096), _V(0,1,0), (float)(68*RAD));
   
   //Rear Gear Right3
   static UINT GGrp12[7] = {7,8,11,64,65,66,67};
   
   static MGROUP_ROTATE gear12 (0,GGrp12, 7, _V(1.105511,-0.9846452,-2.238096), _V(0,1,0), (float)(-68*RAD));
   
   
   //Rear Gear Left4
   static UINT GGrp14[1] = {6};
   
   static MGROUP_ROTATE gear14 (0,GGrp14, 1, _V(-1.105511,-1.391714,-1.00101), _V(1,0,0), (float)(45*RAD));
   
   //Rear Gear Right5
   static UINT GGrp15[1] = {8};
   
   static MGROUP_ROTATE gear15 (0,GGrp15, 1, _V(1.105511,-1.391714,-1.00101), _V(1,0,0), (float)(45*RAD));
   
   
   anim_gear = CreateAnimation (0);
   
   AddAnimationComponent  (anim_gear, 0, 0.5, &gear1);
   AddAnimationComponent  (anim_gear, 0, 0.25, &gear2);
   AddAnimationComponent  (anim_gear, 0.25, 0.5, &gear3);
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear4);
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear5);
  
   AddAnimationComponent  (anim_gear, 0.25, 0.5, &gear8);
   AddAnimationComponent  (anim_gear, 0.5, 0.75, &gear9);
   AddAnimationComponent  (anim_gear, 0, 0.5, &gear11);
   AddAnimationComponent  (anim_gear, 0.25, 0.75, &gear12);
   AddAnimationComponent  (anim_gear, 0.5, 0.6, &gear14);
   AddAnimationComponent  (anim_gear, 0.75, 0.85, &gear15);
   
   
   
   
   
   //Rudder Top
   static UINT RGrp1[2] = {41,42};
   
   static MGROUP_ROTATE rudderl (0,RGrp1, 2, _V(0,0.6946283,-12.32847), _V(0,1,-0.5), (float)(-22*RAD));
   
   //Rudder pedals Left
   static UINT RGrp4[4] = {17,19,40,43};
   
   static MGROUP_ROTATE rudderl4 (1,RGrp4, 4, _V(0,-2,1.964862), _V(1,0,0), (float)(25*RAD));
   
   //Rudder pedals Right
   static UINT RGrp5[4] = {18,20,41,42};
   
   static MGROUP_ROTATE rudderl5 (1,RGrp5, 4, _V(0,-2,1.964862), _V(1,0,0), (float)(-25*RAD));
   
   
   //Rudder Bottom
   static UINT RGrp2[2] = {45,46};
   
   static MGROUP_ROTATE rudderr (0,RGrp2, 2, _V(0,0.6946283,-12.32847), _V(0,1,-0.5), (float)(-22*RAD));
   
   anim_rudder = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderl);
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderr);
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderl4);
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderl5);
   
   //Elevator
   static UINT EGrp1[2] = {37,53};
   
   static MGROUP_ROTATE elevator (0,EGrp1, 2, _V(0,0.7390389,-11.60014), _V(1,0,0), (float)(18*RAD));
   
   
   anim_elevator = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_elevator, 0, 1, &elevator);
   
   
   
   
   //Left Aleron
   static UINT LAGrp1[1] = {37};
   
   static MGROUP_ROTATE laileron (0,LAGrp1, 1, _V(0,0.7390389,-11.60014), _V(1,0,0), (float)(-11*RAD));
   
   //Stick Left
   //static UINT LAGrp2[2] = {46,48};
   
   //static MGROUP_ROTATE laileron2 (1,LAGrp2, 2, _V(-1.989425,-3.099091,0.205239), _V(0,0,1), (float)(-22*RAD));
   
   //Stick Right
   //static UINT LAGrp3[2] = {47,49};
   
   //static MGROUP_ROTATE laileron3 (1,LAGrp3, 2, _V(1.989425,-3.099091,0.205239), _V(0,0,1), (float)(-22*RAD));
   
   
   anim_laileron = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_laileron, 0, 1, &laileron);
   //AddAnimationComponent  (anim_laileron, 0, 1, &laileron2);
   //AddAnimationComponent  (anim_laileron, 0, 1, &laileron3);
   
   //Right Aleron
   static UINT RAGrp1[1] = {53};
   
   static MGROUP_ROTATE raileron (0,RAGrp1, 1, _V(0,0.7390389,-11.60014), _V(1,0,0), (float)(11*RAD));
   
   
   anim_raileron = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_raileron, 0, 1, &raileron);
   
   
   
   
   
   //Swingwing Left
   static UINT SWGrp1[6] = {68,69,70,71,72,73};
   
   static MGROUP_ROTATE swing (0,SWGrp1, 6, _V(-3.127486,0,0.2150946), _V(0,1,0), (float)(90*RAD));
   
   //Swingwing Right
   static UINT SWGrp2[6] = {74,75,76,77,78,79};
   
   static MGROUP_ROTATE swing2 (0,SWGrp2, 6, _V(3.127486,0,0.2150946), _V(0,1,0), (float)(-90*RAD));
  
   //Flaps Left
   static UINT SWGrp3[2] = {70,71};
   
   static MGROUP_ROTATE swing3 (0,SWGrp3, 2, _V(-4.942456,0.1215288,-3.412283), _V(1,0.2,0.5), (float)(-12*RAD));
   
   //Flaps Right
   static UINT SWGrp4[2] = {76,77};
   
   static MGROUP_ROTATE swing4 (0,SWGrp4, 2, _V(4.942456,0.1215288,-3.412283), _V(1,-0.2,-0.5), (float)(-12*RAD));
   
   
   anim_swingwing = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_swingwing, 0, 0.8, &swing);
   AddAnimationComponent  (anim_swingwing, 0, 0.8, &swing2);
   AddAnimationComponent  (anim_swingwing, 0.8, 1, &swing3);
   AddAnimationComponent  (anim_swingwing, 0.8, 1, &swing4);
   
   
   
   // Altimeter Animations
   
   // Dial
   static UINT w2WGrp0[1] = {0};
   
   static MGROUP_ROTATE wwheel0 (1,w2WGrp0, 1, _V(-0.7440445,-1.227473,0.971967), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_wheel0 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel0, 0, 1, &wwheel0);
   
   
   
   
   
   
   // Gear Lever Animation
   
   //Gear Lever in Cockpit
   static UINT Glev0[2] = {2,3};
   
   static MGROUP_ROTATE gearlev (1,Glev0, 2, _V(3.734309E-03,-1.705995,0.941926), _V(1,0,0), (float)(55*RAD));
   
   anim_gearlever = CreateAnimation (0);
   
   AddAnimationComponent  (anim_gearlever, 0, 1, &gearlev);
   
   
   // Power Guage Animation
   
   static UINT Pow0[4] = {36,37,38,39};
   
   static MGROUP_ROTATE powg (1,Pow0, 4, _V(0,-2.861104,0.1), _V(1,0,0), (float)(33*RAD));
   
   anim_power = CreateAnimation (0);
   
   AddAnimationComponent  (anim_power, 0, 1, &powg);
   
  
   
   // Speed Dial Animations
   
   // Dial3
   static UINT d3Grp0[1] = {44};
   
   static MGROUP_ROTATE dial3 (1,d3Grp0, 1, _V(-1.48839,-0.7086169,0.971968), _V(0,0,1), (float)(360*RAD));
   
   
   anim_dial3 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_dial3, 0, 1, &dial3);
   
   
   // Bank Guage Animations
   
   static UINT BAGrp0[1] = {4};
   
   static MGROUP_ROTATE bank (1,BAGrp0, 1, _V(-1.117939,-0.8521763,0.9941239), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_bank = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_bank, 0, 1, &bank);
   
  
   
    // Stick Animations
   
    //Left Stick
   static UINT STGrp0[2] = {46,48};
   
   stick1 = new MGROUP_ROTATE (1,STGrp0, 2, _V(-1.989425,-3.099091,0.205239), _V(0,0,1), (float)(-33*RAD));
   
   static UINT STGrp1[1] = {21};
   
   static MGROUP_ROTATE stick2 (1,STGrp1, 1, _V(-1.989425,-3.099091,0.205239), _V(1,0,0), (float)(-33*RAD));
   
   
   anim_stick1 = CreateAnimation (0.5);
   anim_stick2 = CreateAnimation (0.5);
   
   parent3 = AddAnimationComponent  (anim_stick2, 0, 1, &stick2);
   AddAnimationComponent  (anim_stick1, 0, 1, stick1, parent3);

    //Right Stick
   static UINT STGrp0b[2] = {47,49};
   
   stick1b = new MGROUP_ROTATE (1,STGrp0b, 2, _V(1.989425,-3.099091,0.205239), _V(0,0,1), (float)(-33*RAD));
   
   static UINT STGrp1b[1] = {22};
   
   static MGROUP_ROTATE stick2b (1,STGrp1b, 1, _V(1.989425,-3.099091,0.205239), _V(1,0,0), (float)(-33*RAD));
   
   
   anim_stick1b = CreateAnimation (0.5);
   anim_stick2b = CreateAnimation (0.5);
   
   parent3b = AddAnimationComponent  (anim_stick2b, 0, 1, &stick2b);
   AddAnimationComponent  (anim_stick1b, 0, 1, stick1b, parent3b);
   
   
   
}
	

bool ShuttlePB::clbkLoadVC (int id)
{
	
	SetCameraOffset (_V(-1.8,0,5.2));
	
    SetCameraDefaultDirection (_V(0,0,1));

    SetCameraRotationRange (RAD*55, RAD*55, RAD*33, RAD*33);
    
    
    static VCMFDSPEC mfds_left = {1, 58};
    static VCMFDSPEC mfds_right = {1, 59};

    oapiVCRegisterMFD (MFD_LEFT, &mfds_left);
    oapiVCRegisterMFD (MFD_RIGHT, &mfds_right);

    
	
	//static VCHUDSPEC hud_pilot  = {1, 78,{0,0.7,4.7},0.5};
	
    //oapiVCRegisterHUD (&hud_pilot);

		
	
	
 return true;

}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new ShuttlePB (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (ShuttlePB*)vessel;
}
