// ==============================================================
//                 ORBITER MODULE: ShuttlePB
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// ShuttlePB.cpp
// Control module for ShuttlePB vessel class
//
// Notes:
// This is an example for a "minimal" vessel implementation which
// only overloads the clbkSetClassCaps method to define vessel
// capabilities and otherwise uses the default VESSEL class
// behaviour.
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#define ENG 				1	
#define GEAR				2	
#define FLAPS               3

#include <orbitersdk.h>
#include <cstdio>
#include <cmath>


#include <string>
#include "OrbiterSoundSDK35.h"

#include <stdio.h>
#include <stdlib.h>

using namespace std;


// ==============================================================
// Some vessel parameters
// ==============================================================
const double PB_FUELMASS = 20000.0;
const double PB_ISP = 50000.0;
const double PB_MAXMAINTH =0;



double t0;

double tvec = 0;
double tvec2 = 0;

double mx1 = 0;
double iop = 0;
double st = 0;
double pld = 0;
double cen = 0;
double gear_proc = 0;
double gear_proc2 = 0;
double gear_procl = 0;
double power_proc = 0;
double fuel_proc = 0;
double speedbrake = 0;
//double swingwing = 0;
//double swing = 0;
//double swing3 = 0;
//double hook = 0;
//double canopy = 0;
double flaps = 0;
double flaps2 = 0;
double exproc = 0;
double Lights = 0;
double switched = 0;
double aoa;
double dil = 0;
double eng_pitch = 22000;

double tpp = 0;


double start = 0;
double full = 0.001;
double i = 0;
double i1 = 0;
double i2 = 0;
double i3 = 0;
double i4 = 0;
double i5 = 0;
double i7 = 0;
double i9 = 0;
double i1a = 0;
double i2a = 0;
double i3a = 0;
double i4a = 0;
double i5a = 0;
double s1 = 0;
double s2 = 0;
double s3 = 0;
double s4 = 0;
double s5 = 0;
double ga = 0;
double diff = 0.1;
double k3;
int k4;
char k3buffer [33];
double amet = 0;
double dee = 0;
double i13 = 0;
double i14 = 0;
double i15 = 0;
double i16 = 0;
double i17 = 0;
double i18 = 0;
double i19 = 0;
double i20 = 0;
double i21 = 0;

double aoa2 = 0;

//double droptank = 0;

double i25 = 0;
double i26 = 0;


double secondtime = 0;

double refuel = 0;

UINT animactive;

UINT animactivel;

UINT animactive2;

UINT animactive3;

UINT animactive4;

UINT animactive5;

UINT animactive7;

UINT animactive9;

const VECTOR3 offset = {0,0.2,4.5};


THRUSTER_HANDLE th_main;

BEACONLIGHTSPEC beacon1;    

BEACONLIGHTSPEC beacon2;   

MESHHANDLE f150;
MESHHANDLE f151;
MESHHANDLE f152;




// 1. vertical lift component (wings and body)

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	static const double CL[nabsc] = {0.1, 0.17, 0.2, 0.2, 0.17, 0.1, 0, -0.11, -0.24, -0.38,  -0.5,  -0.5, -0.02, 0.6355,    0.63,   0.46, 0.28, 0.13, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,    0,0.002,0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = CM[idx] + (CM[idx+1]-CM[idx])*d;
	*cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.02 + oapiGetInducedDrag (*cl, 1.5, 0.6);
}


// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class ShuttlePB: public VESSEL2 {
public:
	ShuttlePB (OBJHANDLE hVessel, int flightmodel);
	//	: VESSEL2 (hVessel, flightmodel) {}
	void clbkSetClassCaps (FILEHANDLE cfg);
	
	~ShuttlePB();

	
	void DefineAnimations(void);
	void Steering();
	//void Lighting();
	void clbkPostCreation (void);
    void clbkPostStep (double simt, double simdt, double mjd);
    int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
   
    UINT anim_wheel;
    UINT anim_gear;
    UINT anim_gearlever;
    UINT anim_rudder;
    UINT anim_elevator;
    UINT anim_laileron;
    UINT anim_raileron;
    UINT anim_speedbrake;
    
    UINT anim_flaps;
    //UINT anim_hook;
    
    UINT anim_power;
    UINT anim_dial44;
    
    UINT anim_wheel0;
    UINT anim_wheel1;
    UINT anim_wheel2;
    UINT anim_wheel3;
    UINT anim_wheel4;
    UINT anim_wheel5;
    UINT anim_fuel;

    UINT anim_turn;

    UINT anim_dial1;
    UINT anim_dial2;
    UINT anim_dial3;
    UINT anim_bank;
    UINT anim_clock_s;
    UINT anim_clock_m;
    UINT anim_clock_h;
    
    
     MGROUP_TRANSFORM *wheel1;

    
    //bool clbkVCMouseEvent (int id, int event, VECTOR3 &p);

    bool clbkLoadVC (int id);
    
    int MySoundID;	


};

ShuttlePB::ShuttlePB (OBJHANDLE hVessel, int flightmodel)
		: VESSEL2 (hVessel, flightmodel)
{
	
	// preload meshes
	
    f150                 = oapiLoadMeshGlobal ("K33\\K-F-117-v5");
    f151                 = oapiLoadMeshGlobal ("K33\\K-F-117-v5-Cpit");
    f152                 = oapiLoadMeshGlobal ("K33\\K-F-117-v5-Cpit_n");
   

	
	DefineAnimations ();
	
}


void ShuttlePB::clbkPostCreation (void)
{

int MySoundID=ConnectToOrbiterSoundDLL3(GetHandle());

//ReplaceStockSound3(MySoundID,"Sound\\Vessel\\K-Test-Car\\K-Test-Car.wav",	REPLACE_MAIN_THRUST);

RequestLoadVesselWave3(MySoundID,ENG,"Sound\\K-F-117-v5\\K-F-117-v5-Eng.wav",BOTHVIEW_FADED_FAR);

RequestLoadVesselWave3(MySoundID,FLAPS,"Sound\\K-F-117-v5\\K-F-117-v5-Flaps.wav",BOTHVIEW_FADED_FAR);

RequestLoadVesselWave3(MySoundID,GEAR,"Sound\\K-F-117-v5\\K-F-117-v5-Gear.wav",BOTHVIEW_FADED_FAR);


SoundOptionOnOff3(MySoundID,PLAYMAINTHRUST,FALSE);

SoundOptionOnOff3(MySoundID,PLAYHOVERTHRUST,FALSE);


SoundOptionOnOff3(MySoundID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);

SoundOptionOnOff3(MySoundID,PLAYCABINAIRCONDITIONING,FALSE);

SoundOptionOnOff3(MySoundID,PLAYCABINRANDOMAMBIANCE,FALSE);

SoundOptionOnOff3(MySoundID,PLAYRADIOATC,FALSE);

SoundOptionOnOff3(MySoundID,DISPLAYTIMER,FALSE);

}


// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------

void ShuttlePB::clbkSetClassCaps (FILEHANDLE cfg)
{
	// THRUSTER_HANDLE th_main;


	// physical specs
	SetSize (8);
	SetEmptyMass (12000.0);
	SetCW (0.3, 0.3, 0.6, 0.9);
	SetWingAspect (1.7);
	SetWingEffectiveness (3.1);
	SetCrossSections (_V(5,5,15));
	SetRotDrag (_V(3.5,3.5,3.5));
	if (GetFlightModel() >= 1) {
		SetPitchMomentScale (1e-4);
		SetBankMomentScale (1e-4);
	}
	SetPMI (_V(15.5,22.1,7.7));
	SetTrimScale (0.05);
	SetCameraOffset (_V(0,0.64,2.71));
	
	SetTouchdownPoints (_V(0,-1.55953,4.604074), _V(-1.740703,-1.55953,-0.3588331), _V(1.740703,-1.55953,-0.3588331));

	// **************************** aerodynamics ***********************************

	CreateAirfoil (LIFT_VERTICAL,   _V(0,0,-0.5), VLiftCoeff, 20, 270, 2.266);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,-4), HLiftCoeff, 20,  50, 1.5);





	CreateControlSurface (AIRCTRL_ELEVATOR,     7, 0.7, _V(   0,0,-17), AIRCTRL_AXIS_XPOS,anim_elevator);
	CreateControlSurface (AIRCTRL_RUDDER,       2, 0.5, _V(   0,2,-15), AIRCTRL_AXIS_YPOS,anim_rudder);
	CreateControlSurface (AIRCTRL_AILERON,      4.8, 0.2, _V( 17.8,0,-17.8), AIRCTRL_AXIS_XPOS,anim_raileron);
	CreateControlSurface (AIRCTRL_AILERON,      4.8, 0.2, _V(-17.8,0,-17.8), AIRCTRL_AXIS_XNEG,anim_laileron);
	CreateControlSurface (AIRCTRL_ELEVATORTRIM, 5, 0.1, _V(   0,0,-15), AIRCTRL_AXIS_XPOS,anim_elevator);
    CreateControlSurface (AIRCTRL_FLAP,         5, 5, _V( 0,0,-1), AIRCTRL_AXIS_XPOS,anim_flaps);
	
	CreateVariableDragElement( &speedbrake, 5.0, _V(0,0,0) );

	CreateVariableDragElement (&gear_proc2, 0.8, _V(0, -1, 0));
	
	//CreateVariableDragElement (&hook, 3.33, _V(0,1,0));


	
	// propellant resources
	PROPELLANT_HANDLE hpr = CreatePropellantResource (PB_FUELMASS);
	
	
	//SURFHANDLE tex_main = oapiRegisterExhaustTexture ("K-T-38-v4-Exhaust");



	// ***************** thruster definitions *******************

	PARTICLESTREAMSPEC contrail_main = {
		0, 0.3, 165, 19, 0.019, 0.111, 1.11, 1.11, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 2,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};
	
	th_main = CreateThruster (_V(0,0,-4.3), _V(0,0,1), PB_MAXMAINTH, hpr, PB_ISP);
	
	
	
	CreateThrusterGroup (&th_main, 1, THGROUP_MAIN);
	
	
	
	//AddExhaust (th_main, 4, 0.4, _V(-0.1870975,-0.4776336,-6.6), _V(0,0,-1) , tex_main);
	//AddExhaust (th_main, 4, 0.4, _V(0.1870975,-0.4776336,-6.6), _V(0,0,-1) , tex_main);

    
    //AddExhaustStream (th_main, _V(-0.1870975,-0.4776336,-6.87), &contrail_main);
    //AddExhaustStream (th_main, _V(0.1870975,-0.4776336,-6.87), &contrail_main);
    
    
      // Strobe lights
	
	//Red1 strobe
	
	static VECTOR3 beacon1pos = {-5.327872,-0.325484,-5.451247};
	static VECTOR3 beacon1col = {0.988, 0.133, 0.133};
	
		beacon1.shape = (BEACONSHAPE_STAR);
		beacon1.pos = &beacon1pos;
		beacon1.col = &beacon1col;
		beacon1.size = (0.13);
		beacon1.falloff = (0.6);
		beacon1.period = (1.13);
		beacon1.duration = (0.05);
		beacon1.tofs = (0.6);
		beacon1.active = false;
		
		AddBeacon (&beacon1);
		
		//Red2 strobe
	
	static VECTOR3 beacon2pos = {5.327872,-0.325484,-5.451247};
	static VECTOR3 beacon2col = {0.988, 0.133, 0.133};
	
		beacon2.shape = (BEACONSHAPE_STAR);
		beacon2.pos = &beacon2pos;
		beacon2.col = &beacon2col;
		beacon2.size = (0.13);
		beacon2.falloff = (0.6);
		beacon2.period = (1.13);
		beacon2.duration = (0.05);
		beacon2.tofs = (0.6);
		beacon2.active = false;
		
		AddBeacon (&beacon2);
		

    

    
    
    
	// visual specs
	
	AddMesh (f150);
	
	SetMeshVisibilityMode( AddMesh (f151,&offset), MESHVIS_VC );

    

}

// ==============================================================
// API callback interface
// ==============================================================

void ShuttlePB::clbkPostStep (double simt, double simdt, double mjd)

{
	
	if( oapiGetTimeAcceleration() > 10.0 ) oapiSetTimeAcceleration(10);
	
	double vsAirSpeed = GetAirspeed();
	
	

	
	
    //speed and altitude control
	
	if (GetAirspeed() >= 700) 
	{
		SetCW (999, 999, 999, 999);
		SetThrusterMax0 (th_main ,0);

	
}

if (GetAltitude() >= 17000) 
	{
		SetCW (999, 999, 999, 999);
		SetThrusterMax0 (th_main ,0);

	
}



if (GetAirspeed() < 700 && GetAltitude() < 17000 ) 
	{
		SetCW (0.3, 0.3, 0.67, 0.92);
		SetThrusterMax0 (th_main ,4.4e5);

	
}

	double vsAirSpeed2 = (vsAirSpeed/1000)*3600;
    
    //Ground Steering
    
    if (GroundContact () && vsAirSpeed > 0.6 && vsAirSpeed < 90 &&  gear_proc < 0.05) Steering();
    
    
    //aoa = GetAOA();
    
    //sprintf(oapiDebugString(),"anim %2.2f", aoa );
    
  
    
    if (  gear_proc < 0.05 ) SetTouchdownPoints (_V(0,-1.55953,4.604074), _V(-1.740703,-1.55953,-0.3588331), _V(1.740703,-1.55953,-0.3588331));


    if (  gear_proc > 0.95 ) SetTouchdownPoints (_V(0,-0.4268042,4.013654), _V(-1.518829,-0.4268042,-1.547508), _V(1.518829,-0.4268042,-1.547508));

    
    //Anims 
    
    if (iop > 1)iop=0;
    
    if (vsAirSpeed > 0.6) {
    
	double aspeed = vsAirSpeed;
    if (iop > 1)iop=0;
    if (vsAirSpeed > 30) aspeed = 30; 
    iop = iop + (aspeed /300);
     
    if (vsAirSpeed ==0) iop=0;
    
    
    if ( GroundContact() && gear_proc < 0.05) SetAnimation (anim_wheel, iop);
    

   }
	
	 // Front Gear Turn Anims1
   if (!GroundContact() && gear_proc < 1.0 && vsAirSpeed > 0.6)
   {
	   tpp = 0.5;
	   SetAnimation( anim_turn, tpp );
}

   // Front Gear Turn Anims2
   if (GroundContact() && gear_proc < 0.05 )
   {
	   
   tpp = (GetControlSurfaceLevel (AIRCTRL_RUDDER) +1) *0.5;
   if (!GroundContact())tpp = 0.5;
   SetAnimation( anim_turn, tpp );
   
}

   
   
   
// Gear animations


   if( animactive && !GroundContact()) {
		if( animactive == 1 ) {
			gear_proc += 0.00242;
			if( gear_proc > 1.0 ) {
				gear_proc = 1.0;
				animactive = 0;
			}
		}
		else  {
			gear_proc -= 0.00242;
			if( gear_proc < 0.0 ) {
				gear_proc = 0.0;
				animactive = 0;
			}
		}
		SetAnimation( anim_gear, gear_proc );
		gear_proc2 = -gear_proc + 1;
	}

	
	// Gear Lever animations


   if( animactivel && !GroundContact()) {
		if( animactivel == 1 ) {
			gear_procl += 0.00242;
			if( gear_procl > 1.0 ) {
				gear_procl = 1.0;
				animactivel = 0;
			}
		}
		else  {
			gear_procl -= 0.00242;
			if( gear_procl < 0.0 ) {
				gear_procl = 0.0;
				animactivel = 0;
			}
		}
		SetAnimation( anim_gearlever, gear_procl );
	}

	
	// Throttle animation

        power_proc = GetThrusterLevel(th_main);
	
		SetAnimation( anim_power, power_proc );
		
	
   
   // Speedbrake animations

   if( animactive2 ) {
		if( animactive2 == 1 ) {
			speedbrake += 0.003;
			if( speedbrake > 1.0 ) {
				speedbrake = 1.0;
				animactive2 = 0;
			}
		}
		else  {
			speedbrake -= 0.003;
			if( speedbrake < 0.0 ) {
				speedbrake = 0.0;
				animactive2 = 0;
			}
		}
		SetAnimation( anim_speedbrake, speedbrake );
	}

	
	
   // Flaps


   if( animactive4 ) {
		if( animactive4 == 1 ) {
			flaps += 0.006;
			if( flaps > 1.0 ) {
				flaps = 1.0;
				animactive4 = 0;
			}
		}
		else  {
			flaps -= 0.006;
			if( flaps < 0.0 ) {
				flaps = 0.0;
				animactive4 = 0;
			}
		}
		SetAnimation( anim_flaps, flaps );
	}

   SetControlSurfaceLevel(AIRCTRL_FLAP,flaps);

 

// Engine sound
	
	
	
	
	if (GetThrusterLevel (th_main) > 0) 
	{
		
	double pit = ((GetThrusterLevel (th_main)*6000));
	
	eng_pitch = 13000 + pit;
	
	if (eng_pitch < 11500 ) eng_pitch = 11500;
	
	if (eng_pitch > 22500 ) eng_pitch = 22500;
	
	//sprintf(oapiDebugString(),"anim %7.7f", eng_pitch );
		
	if ( oapiCameraInternal ())
	{
	PlayVesselWave3(MySoundID,ENG,LOOP,220,eng_pitch - 1800);
}

else

	PlayVesselWave3(MySoundID,ENG,LOOP,255,eng_pitch);
	
}


    if (GetThrusterLevel (th_main) == 0) 
	{
	
	StopVesselWave3(MySoundID,ENG);
}
   



 if (Lights == 1 && switched == 1)
 {
	 beacon1.active = true;
	 beacon2.active = true;

	 switched = 0;
	 SetMeshVisibilityMode( InsertMesh (f152,1,&offset), MESHVIS_VC );
}

if (Lights == 0 && switched == 1)
{
	beacon1.active = false;
	beacon2.active = false;

    switched = 0;
    SetMeshVisibilityMode( InsertMesh (f151,1,&offset), MESHVIS_VC );
}


// Altimeter animation
   
   
    //Anims 
    
     k3 = GetAltitude();
    
     k4 = int (k3);
      
     gcvt (k3,6,k3buffer);
     
     

   
     string stringOne(k3buffer);
     
     if (k3 >= 0 )
     {

     string is1 = stringOne.substr(0 + s2 + s3 + s4 + s5, 1) ;
       
     char const *cString1 = is1.c_str();
     
     i1a = atol(cString1);
     
     i1 = i1a * 0.1;
     
 }
 
 if (k3 >= 10 ) 
 {
	 
     string is2 = stringOne.substr(0 + s3 + s4 + s5 , 1) ;
       
     char const *cString2 = is2.c_str();
     
     i2a = atol(cString2);
     
     i2 = i2a * 0.1;
      s2 = 1;
 }
 
 if (k3 >= 100 )
 {
     string is3 = stringOne.substr(0 + s4 + s5, 1) ;
       
     char const *cString3 = is3.c_str();
     
     i3a = atol(cString3);
     
     i3 = i3a * 0.1;
     
     s3 = 1;
 }
 
 if (k3 >= 1000)
 {
     string is4 = stringOne.substr(0 + s5, 1) ;
       
     char const *cString4 = is4.c_str();
     
     i4a = atol(cString4);
     
     i4 = i4a * 0.1;
     
     s4 = 1;
 }
 
 if (k3 >= 10000)
 {
     string is5 = stringOne.substr(0, 1) ;
       
     char const *cString5 = is5.c_str();
     
     i5a = atol(cString5);
     
     i5 = i5a * 0.1;
     
     s5 = 1;
}

      if(k3<10)
      {
	      s2=0;
	      i2=0;
	      s3=0;
	      i3=0;
	      s4=0;
	      i4=0;
	      s5=0;
	      i5=0;
    }
         
    if(k3<100)
      {
	     
	      s3=0;
	      i3=0;
	      s4=0;
	      i4=0;
	      s5=0;
	      i5=0;
    }
    
    if(k3<1000)
      {
	      
	      s4=0;
	      i4=0;
	      s5=0;
	      i5=0;
    }
   
    if(k3<10000)
      {
	     
	      s5=0;
	      i5=0;
	      
    }
    
    
   i=(k3 * 0.01)- dee;
   
   if (i>=1)
   {
	   dee = dee + 1;
   }
   
   if (i <= 0)
   {
	   dee = dee - 1;
	   
	   if (dee < 0)dee = 0;
   }
   
    
	//sprintf(oapiDebugString(),"force %2.2f", i1 );
	
	
	SetAnimation (anim_wheel0, i);
	SetAnimation (anim_wheel1, i1);
	SetAnimation (anim_wheel2, i2);
	SetAnimation (anim_wheel3, i3);
	SetAnimation (anim_wheel4, i4);
	SetAnimation (anim_wheel5, i5);
	
	


// AOA Dial Animation

    aoa = GetAOA();
    
   if (vsAirSpeed > 0.6) {
   i7 = aoa * 6;

   SetAnimation (anim_dial1, i7);
   
}

if (vsAirSpeed <= 0.6) {
	i7 = 0;

   SetAnimation (anim_dial1, i7);
   
}

   // Direction Dial Animation

   oapiGetFocusHeading(&dil);
    
   i9 = (dil / 6.25) + 0.001;
   
   //sprintf(oapiDebugString(),"hed %2.2f", dil );

   SetAnimation (anim_dial2, i9);
   
   
   // Speed Dial Animation
    
   i13 = GetAirspeed();
   
   if (i13 > 600) i13 = 600;
   
   if (i13 < 0) i13 = 0;
   
   i14 = (-i13 + 600)/600;
   
   if (i14 < 0) i14 = 0;
   
   if (i14 > 1) i14 = 1;
   
   SetAnimation (anim_dial3, i14);
   
    // Bank Guage Animation
    
   i15 = GetBank();
   
   i16 = (i15 + 3) / 6;
   
   if (i16 < 0) i16 = 0;
   
   if (i16 > 1) i16 = 1;
   
   SetAnimation (anim_bank, i16);
   
 
    
    
     // Clock Dial Animation
     
     //Second Hand
     
    
   i17 = i17 + ( 0.0002 * oapiGetTimeAcceleration() );
   
   if (i17 < 0) i17 = 0;
   
   if (i17 >= 1) 
   {
	   i17 = 0;
	   i18 = i18 + 0.016666666;
}
   SetAnimation (anim_clock_s, i17);
   
  // sprintf(oapiDebugString(),"time %19.19f", i17 );
   
    //Minute Hand
    
   if (i18 < 0) i18 = 0;
   
   if (i18 >= 1) 
   {
	   i18 = 0;
	   i19 = i19 + 0.016666666;
   }
   
   
   SetAnimation (anim_clock_m, i18);
   
    //Hour Hand
   
   if (i19 < 0) i19 = 0;
   
   if (i19 >= 1) i19 = 0;
   
   SetAnimation (anim_clock_h, i19);
   
   
   
    
	// Fuel Guage Animation
   
   fuel_proc = (-GetFuelMass() + 20000)/20000; 

   i21 = fuel_proc / 3;
   
   if (i21 < 0) i21 = 0;
   
   if (i21 > 1) i21 = 1;
   
   SetAnimation (anim_fuel, i21);
   


   // Refuel
   
   if (refuel == 1)
   {
	   
	  SetFuelMass (20000.0);

	   refuel = 0;
   }
 

   // Head Gyroscopic and G-Force movement
   
    if (vsAirSpeed > 0.6) {
		aoa2 = (GetAOA()* 0.5);
		
		if (aoa2 > 0.333) aoa2 = 0.333;
		
		if (aoa2 < -0.333) aoa2 = -0.333;
		
		//SetCameraOffset (_V(0,1.433,2.1));
		SetCameraOffset (_V(0,0.64 + aoa2,2.71));
		
}
   
if (vsAirSpeed <= 0.6) {
	aoa2 = 0;
			
		//SetCameraOffset (_V(0,1.433,2.1));
		SetCameraOffset (_V(0,0.64 + aoa2,2.71));
	}


}

//Ground Steering


void ShuttlePB::Steering()
 {

	double pos = GetControlSurfaceLevel(AIRCTRL_RUDDER);
	double lev = 3e6 * pos ;

	if( oapiGetTimeAcceleration() > 10.0 )
		return;

	AddForce( _V( lev,0,0 ), _V( 0,0,3.33 ) );
	AddForce( _V( -1.0*lev,0,0 ), _V( 0,0,-3.33 ) );
	
}



// --------------------------------------------------------------
// Keyboard interface handler (buffered key events)
// --------------------------------------------------------------
int ShuttlePB::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate)
{
	if (!down) return 0; // only process keydown events

	// if (KEYMOD_SHIFT (kstate)) {

		switch (key) {
		case OAPI_KEY_G: // Gear
		
		if ( GroundContact() ) return 1;
		
		if( gear_proc > 0.5 )
					animactive = 2;
				else
					animactive = 1;

		if ( !GroundContact()) PlayVesselWave3(MySoundID,GEAR,NOLOOP,255);
		
		
		if( gear_procl > 0.5 )
					animactivel = 2;
				else
					animactivel = 1;

			
			return 1;
				
			case OAPI_KEY_0: // Canopy
		
		if( speedbrake > 0.5 )
					animactive2 = 2;
				else
					animactive2 = 1;

		PlayVesselWave3(MySoundID,FLAPS,NOLOOP,255);
		
			
			return 1;

			
			case OAPI_KEY_F: // Flaps
			
			
		if( flaps > 0.5 )
					animactive4 = 2;
				else
					animactive4 = 1;

		PlayVesselWave3(MySoundID,FLAPS,NOLOOP,255);
		
			
			return 1;
			
			
			case OAPI_KEY_J: // Refuel
			
			
		if (refuel == 0 && GetAirspeed() == 0)
		{
			refuel = 1;
		}
		
			
			return 1;
		
		

			case OAPI_KEY_K: // Lights
		
		if( Lights == 0 )
		{
			Lights = 1;
			switched =1;
					
			return 1;
}

		if( Lights == 1 )
		{
			Lights = 0;
			switched = 1;
					
			return 1;
}
	
    } 
	return 0;
}


// --------------------------------------------------------------
// Destructor
// --------------------------------------------------------------
ShuttlePB::~ShuttlePB ()
{
	delete wheel1;
	
}



void ShuttlePB::DefineAnimations(void)
{
	
	ANIMATIONCOMPONENT_HANDLE parent1;

	
   // Front Wheel
   static UINT WGrp1[1] = {46};
   
   wheel1 = new MGROUP_ROTATE  (0,WGrp1, 1, _V(0,-1.3201,4.606005), _V(1,0,0), (float)(360*RAD));
   
   
    //Turn Front Gear
   static UINT Trn1[9] = {36,37,38,39,40,41,42,43,44};
   
   static MGROUP_ROTATE turn1 (0,Trn1, 9, _V(0,-0.2135228,4.660895), _V(0,1,0), (float)(90*RAD));
   

   
   //Rear left Wheel
   static UINT WGrp3[1] = {329};
   
   static MGROUP_ROTATE wheel2 (0,WGrp3, 1, _V(0,-1.237542,-0.357916), _V(1,0,0), (float)(360*RAD));
   
   //Rear Right wheel
   static UINT WGrp4[1] = {330};
   
   static MGROUP_ROTATE wheel3 (0,WGrp4, 1, _V(0,-1.237542,-0.357916), _V(1,0,0), (float)(360*RAD));
   
   anim_turn = CreateAnimation (0.5);
   anim_wheel = CreateAnimation (0);
   
   parent1 = AddAnimationComponent  (anim_turn, 0, 1, &turn1);
   AddAnimationComponent  (anim_wheel, 0, 1, wheel1, parent1);
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel2);
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel3);
   
   
   
   
   // Gear anims
  
   
   //Front Gear
   static UINT GGrp1[11] = {36,37,38,39,40,41,42,43,44,45,46};
   
   static MGROUP_ROTATE gear1 (0,GGrp1, 11, _V(0,-0.2189643,4.584713), _V(1,0,0), (float)(-105*RAD));
   
   
   //Rear Gear Left
   static UINT GGrp2[11] = {309,310,311,312,317,319,320,321,322,323,329};
   
   static MGROUP_ROTATE gear2 (0,GGrp2, 11, _V(-1.470682,-0.2578211,-0.3803951), _V(1,0,0), (float)(-105*RAD));
   
   //Rear Gear Right
   static UINT GGrp3[11] = {313,314,315,316,318,324,325,326,327,328,330};
   
   static MGROUP_ROTATE gear3 (0,GGrp3, 11, _V(1.470682,-0.2578211,-0.3803951), _V(1,0,0), (float)(-105*RAD));
   
    
    //Front Gear Cover
   static UINT GGrp5[10] = {6,7,8,9,10,21,22,23,24,25};
   
   static MGROUP_ROTATE gear5 (0,GGrp5, 10, _V(-0.2335168,-0.1487122,6.08281), _V(0,0,1), (float)(90*RAD));
   
   
   //Rear Cover Left
   static UINT GGrp12[6] = {14,15,16,29,30,31};
   
   static MGROUP_ROTATE gear12 (0,GGrp12, 6, _V(-1.459599,-0.3626664,1.36434), _V(0,0,1), (float)(-90*RAD));
   
   //Rear Cover Right
   static UINT GGrp13[6] = {11,12,13,26,27,28};
   
   static MGROUP_ROTATE gear13 (0,GGrp13, 6, _V(1.459599,-0.3626664,1.36434), _V(0,0,1), (float)(90*RAD));
   
   //Rear Cover Mid Left
   static UINT GGrp15[4] = {17,18,32,33};
   
   static MGROUP_ROTATE gear15 (0,GGrp15, 4, _V(-1.470682,-0.2578211,-0.6107915), _V(1,0,0), (float)(-105*RAD));
   
   //Rear Cover Mid Right
   static UINT GGrp16[4] = {19,20,34,35};
   
   static MGROUP_ROTATE gear16 (0,GGrp16, 4, _V(1.470682,-0.2578211,-0.6107915), _V(1,0,0), (float)(-105*RAD));
   
   anim_gear = CreateAnimation (0);
   
   AddAnimationComponent  (anim_gear, 0, 0.5, &gear1);
   AddAnimationComponent  (anim_gear, 0, 0.5, &gear2);
   AddAnimationComponent  (anim_gear, 0, 0.5, &gear3);
   
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear5);
   
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear12);
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear13);
   
   AddAnimationComponent  (anim_gear, 0.1, 0.6, &gear15);
   AddAnimationComponent  (anim_gear, 0.1, 0.6, &gear16);
  
   
   
   //Rudder Left
   static UINT RGrp1[6] = {335,336,337,338,339,340};
   
   static MGROUP_ROTATE rudderl (0,RGrp1, 6, _V(-0.6047022,0.3360359,-5.595755), _V(0.143,0.975,-0.172), (float)(-20*RAD));
   
   //Rudder Right
   static UINT RGrp2[6] = {341,342,343,344,345,346};
   
   static MGROUP_ROTATE rudderr (0,RGrp2, 6, _V(0.6047022,0.3360359,-5.595755), _V(0.143,0.975,-0.172), (float)(-20*RAD));
   
   
   
   
   //Rudder Pedals left
   static UINT RGrp3[2] = {175,177};
   
   static MGROUP_ROTATE rudderlp (1,RGrp3, 2, _V(0,-0.2587604,0.1553893), _V(1,0,0), (float)(33*RAD));
   
   
   
   
   //Rudder Padals right
   static UINT RGrp4[2] = {174,176};
   
   static MGROUP_ROTATE rudderrp (1,RGrp4, 2, _V(0,-0.2587604,0.1553893), _V(1,0,0), (float)(-33*RAD));
   
   
   
   anim_rudder = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderl);
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderr);
   
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderlp);
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderrp);
   
   
   //Elevator Left
   static UINT EGrp1[6] = {335,336,337,338,339,340};
   
   static MGROUP_ROTATE elevator (0,EGrp1, 6, _V(-0.6047022,0.3360359,-5.595755), _V(0.143,0.975,-0.172), (float)(-20*RAD));
   
   //Elevator Right
   static UINT EGrp2[6] = {341,342,343,344,345,346};
   
   static MGROUP_ROTATE elevator2 (0,EGrp2, 6, _V(0.6047022,0.3360359,-5.595755), _V(0.143,0.975,-0.172), (float)(20*RAD));
   
   
   anim_elevator = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_elevator, 0, 1, &elevator);
   AddAnimationComponent  (anim_elevator, 0, 1, &elevator2);
   
   
   
   
   
   //Left Aleron
   static UINT LAGrp1[1] = {1};
   
   static MGROUP_ROTATE laileron (0,LAGrp1, 1, _V(-3.86218,-0.1992677,-4.336602), _V(1,0,1), (float)(-25*RAD));
   
   
   anim_laileron = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_laileron, 0, 1, &laileron);
   
   
   //Right Aleron
   static UINT RAGrp1[1] = {2};
   
   static MGROUP_ROTATE raileron (0,RAGrp1, 1, _V(3.86218,-0.1992677,-4.336602), _V(1,0,-1), (float)(25*RAD));
   
   
   anim_raileron = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_raileron, 0, 1, &raileron);
   
   
  //External Canopy
   static UINT SBGrp1[98] = {75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127,128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143,144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159,160,161,162,163,164,165,166,167,415,416,417,418,419};
   
   static MGROUP_ROTATE speedbrake1 (0,SBGrp1, 98, _V(0,1.389644,3.87134), _V(1,0,0), (float)(-39*RAD));
   
   //Internal Canopy
   static UINT SBGrp3[93] = {75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127,128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143,144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159,160,161,162,163,164,165,166,167};
   
 
   static MGROUP_ROTATE speedbrake3 (1,SBGrp3, 93, _V(0,0.9444854,-2.137072), _V(1,0,0), (float)(-39*RAD));
   
   
   //Canopy Lever
   static UINT SBGrp2[3] = {6,7,8};
   
   static MGROUP_ROTATE speedbrake2 (1,SBGrp2, 3, _V(0,-7.207546E-02,-0.1178391), _V(1,0,0), (float)(-45*RAD));
   
   
   
   
   anim_speedbrake = CreateAnimation (0);
   
   AddAnimationComponent  (anim_speedbrake, 0, 1, &speedbrake1);
   AddAnimationComponent  (anim_speedbrake, 0, 1, &speedbrake2);
   AddAnimationComponent  (anim_speedbrake, 0, 1, &speedbrake3);
   
   
   
  
   
   
   //Flaps animation
   
   //Left Flap
   static UINT FLGrp1[1] = {3};
   
   static MGROUP_ROTATE flapsl (0,FLGrp1, 1, _V(-2.575044,-0.1992677,-2.742415), _V(1,0,1), (float)(-27*RAD));
   
   
   //Right Flap
   static UINT FRGrp1[1] = {4};
   
   static MGROUP_ROTATE flapsr (0,FRGrp1, 1, _V(2.575044,-0.1992677,-2.742415), _V(1,0,-1), (float)(-27*RAD));
   
   //Flap Dial
   static UINT FLGrp3[1] = {58};
   
   static MGROUP_ROTATE flaps3 (1,FLGrp3, 1, _V(-0.2869548,-8.058967E-02,0), _V(0,0,1), (float)(145*RAD));
   
   //Flap Lever
   static UINT FLGrp4[2] = {60,61};
   
   static MGROUP_ROTATE flaps4 (1,FLGrp4, 2, _V(0,-7.094558E-02,-0.1197148), _V(1,0,0), (float)(-95*RAD));
   
   
   anim_flaps = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_flaps, 0, 1, &flapsl);
   AddAnimationComponent  (anim_flaps, 0, 1, &flapsr);
   AddAnimationComponent  (anim_flaps, 0, 1, &flaps3);
   AddAnimationComponent  (anim_flaps, 0, 1, &flaps4);
   
   
   // Gear Lever Animation
   
   //Gear Lever in Cockpit
   static UINT Glev0[2] = {69,70};
   
   static MGROUP_ROTATE gearlev (1,Glev0, 2,  _V(0,7.211046E-02,-0.1409099), _V(1,0,0), (float)(45*RAD));
   
   static UINT Glev2[3] = {71,72,73};
   
   static MGROUP_TRANSLATE gearlev2 (1,Glev2, 3, _V(0,0,-0.002));
   
   anim_gearlever = CreateAnimation (0);
   
   AddAnimationComponent  (anim_gearlever, 0, 1, &gearlev);
   AddAnimationComponent  (anim_gearlever, 0, 1, &gearlev2);
   
   
   // Throttle Animation
   
   static UINT Pow0[3] = {266,267,268};
   
   static MGROUP_TRANSLATE powg (1,Pow0, 3, _V(0,0,0.15));
   
   anim_power = CreateAnimation (0);
   
   AddAnimationComponent  (anim_power, 0, 1, &powg);
   
   
   // Altimeter Animations
   
   // Dial
   static UINT wWGrp0[1] = {10};
   
   static MGROUP_ROTATE wwheel0 (1,wWGrp0, 1, _V(0.2328434,0.3073601,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_wheel0 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel0, 0, 1, &wwheel0);
   
   // 1st dial from Left
   static UINT wWGrp1[10] = {209,210,211,212,213,214,215,216,217,218};
   
   static MGROUP_ROTATE wwheel1 (1,wWGrp1, 10, _V(0,0.319589,-0.129), _V(1,0,0), (float)(360*RAD));
   
   
   anim_wheel5 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel5, 0, 1, &wwheel1);
   
   // 2nd dial from Left
   static UINT wWGrp2[10] = {219,220,221,222,223,224,225,226,227,228};
   
   static MGROUP_ROTATE wwheel2 (1,wWGrp2, 10, _V(0,0.319589,-0.129), _V(1,0,0), (float)(360*RAD));
   
   
   anim_wheel4 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel4, 0, 1, &wwheel2);
   
   // 3rd dial from Left
   static UINT wWGrp3[10] = {179,180,181,182,183,184,185,186,187,188};
   
   static MGROUP_ROTATE wwheel3 (1,wWGrp3, 10, _V(0,0.319589,-0.1315), _V(1,0,0), (float)(360*RAD));
   
   
   anim_wheel3 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel3, 0, 1, &wwheel3);
   
   // 4th dial from Left
   static UINT wWGrp4[10] = {189,190,191,192,193,194,195,196,197,198};
   
   static MGROUP_ROTATE wwheel4 (1,wWGrp4, 10, _V(0,0.319589,-0.1315), _V(1,0,0), (float)(360*RAD));
   
   
   anim_wheel2 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel2, 0, 1, &wwheel4);
   
   // 5th dial from Left
   static UINT wWGrp5[10] = {199,200,201,202,203,204,205,206,207,208};
   
   static MGROUP_ROTATE wwheel5 (1,wWGrp5, 10, _V(0,0.319589,-0.1315), _V(1,0,0), (float)(360*RAD));
   
   
   anim_wheel1 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel1, 0, 1, &wwheel5);
   
   
   
   // AOA Animations
   
   // Dial1
   static UINT dGrp0[1] = {5};
   
   static MGROUP_ROTATE dial1 (1,dGrp0, 1, _V(0.5363805,7.121998E-02,0), _V(0,0,1), (float)(311*RAD));
   
   
   anim_dial1 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_dial1, 0, 1, &dial1);
   

    // Direction Dial Animations
   
   // Dial2
   static UINT d2Grp0[1] = {233};
   
   static MGROUP_ROTATE dial2 (1,d2Grp0, 1, _V(-0.57394,-4.508492E-02,0), _V(0,0,1), (float)(360*RAD));
   
   
   anim_dial2 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_dial2, 0, 1, &dial2);
   

   // Speed Dial Animations
   
   // Dial3
   static UINT d3Grp0[2] = {263,264};
   
   static MGROUP_ROTATE dial3 (1,d3Grp0, 2, _V(0.2749934,-9.301734E-02,0), _V(0,0,1), (float)(360*RAD));
   
   
   anim_dial3 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_dial3, 0, 1, &dial3);
   
   
   // Bank Guage Animations
   
   static UINT BAGrp0[2] = {11,271};
   
   static MGROUP_ROTATE bank (1,BAGrp0, 2, _V(3.878657E-04,-0.1893478,0), _V(0,0,1), (float)(-360*RAD));
   
   
   
   anim_bank = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_bank, 0, 1, &bank);
   
   
   
   
   // Clock
   
   // Second Hand
   static UINT dGrp7[1] = {256};
   
   static MGROUP_ROTATE dial7 (1,dGrp7, 1, _V(0.4212073,-6.707271E-02,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_clock_s = CreateAnimation (0);
   
   AddAnimationComponent  (anim_clock_s, 0, 1, &dial7);
   
   // Minute Hand
   static UINT dGrp8[1] = {251};
   
   static MGROUP_ROTATE dial8 (1,dGrp8, 1, _V(0.4212073,-6.707271E-02,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_clock_m = CreateAnimation (0);
   
   AddAnimationComponent  (anim_clock_m, 0, 1, &dial8);
   
   // Hour Hand
   static UINT dGrp9[1] = {74};
   
   static MGROUP_ROTATE dial9 (1,dGrp9, 1, _V(0.4212073,-6.707271E-02,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_clock_h = CreateAnimation (0);
   
   AddAnimationComponent  (anim_clock_h, 0, 1, &dial9);
   
   
   // Fuel Guage Animation
   
   static UINT Ful0[1] = {63};
   
   static MGROUP_ROTATE fulg (1,Ful0, 1, _V(0.5540832,-5.151631E-02,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_fuel = CreateAnimation (0);
   
   AddAnimationComponent  (anim_fuel, 0, 1, &fulg);

   

   
   
   
}
	

bool ShuttlePB::clbkLoadVC (int id)
{
	
	SetCameraOffset (_V(0,0.64,2.71));
	
    SetCameraDefaultDirection (_V(0,0,1));

    SetCameraRotationRange (RAD*60, RAD*60, RAD*40, RAD*40);
	
	static VCHUDSPEC hud_pilot  = {1, 331,{0,0.64,4.1},0.8};
	static VCMFDSPEC mfds_left = {1, 328};
    static VCMFDSPEC mfds_right = {1, 329};
    static VCMFDSPEC mfds_user1 = {1, 330};
   
	
    oapiVCRegisterHUD (&hud_pilot);
    oapiVCRegisterMFD (MFD_LEFT, &mfds_left);
    oapiVCRegisterMFD (MFD_RIGHT, &mfds_right);
    oapiVCRegisterMFD (MFD_USER1, &mfds_user1);
   
    
    
 return 1;

}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new ShuttlePB (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (ShuttlePB*)vessel;
}
