// ==============================================================
//                 ORBITER MODULE: ShuttlePB
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// ShuttlePB.cpp
// Control module for ShuttlePB vessel class
//
// Notes:
// This is an example for a "minimal" vessel implementation which
// only overloads the clbkSetClassCaps method to define vessel
// capabilities and otherwise uses the default VESSEL class
// behaviour.
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#define ENG 				10	
#define GEAR				11	
#define COVER               12
#define AFTER               13

#include <orbitersdk.h>
#include <cstdio>
#include <cmath>


#include <string>
#include "OrbiterSoundSDK35.h"

#include <stdio.h>
#include <stdlib.h>

using namespace std;


// ==============================================================
// Some vessel parameters
// ==============================================================
const double PB_FUELMASS = 20000.0;

const double PB_ISP = 50000.0;
const double PB_MAXMAINTH =0;

double main_level = 0;

double t0;

double tvec = 0;
double tvec2 = 0;

double mx1 = 0;
double iop = 0;
double st = 0;
double pld = 0;
double cen = 0;
double gear_proc = 0;
double gear_proc2 = 0;
double gear_procl = 0;
double power_proc = 0;
double fuel_proc = 0;
//double speedbrake = 0;
//double swingwing = 0;
//double swing = 0;
double stick3 = 0;
double stick4 = 0;

double first = 0;

double eng_turn = 0;

double after = 0;

double cov =0;

//double flaps = 0;
//double flaps2 = 0;
double exproc = 0;
double Lights = 0;
double switched = 0;
double aoa;
double dil = 0;
double eng_pitch = 22000;

double tpp = 0.5;


double start = 0;
double full = 0.001;
double i = 0;
double i1 = 0;
double i2 = 0;
double i3 = 0;
double i4 = 0;
double i5 = 0;
double i7 = 0;
double i9 = 0;
double i1a = 0;
double i2a = 0;
double i3a = 0;
double i4a = 0;
double i5a = 0;
double s1 = 0;
double s2 = 0;
double s3 = 0;
double s4 = 0;
double s5 = 0;
double ga = 0;
double diff = 0.1;
double k3;
int k4;
char k3buffer [33];
double amet = 0;
double dee = 0;
double i13 = 0;
double i14 = 0;
double i15 = 0;
double i16 = 0;
double i17 = 0;
double i18 = 0;
double i19 = 0;
double i20 = 0;
double i21 = 0;

double aoa2 = 0;

//double droptank = 0;

double i25 = 0;
double i26 = 0;

double Engine_Start = 0;
double prop_speed = 0;


double secondtime = 0;

double refuel = 0;

UINT animactive;

UINT animactivel;

UINT animactive2;

UINT animactive3;

UINT animactive4;

UINT animactive5;

UINT animactive7;

UINT animactive9;

UINT animactive10;

const VECTOR3 offset = {0,0.2,4.5};

THRUSTER_HANDLE thg_main;
THRUSTER_HANDLE thg_hover;

//THGROUP_HANDLE thg_main;

BEACONLIGHTSPEC beacon1;    

BEACONLIGHTSPEC beacon2;   

MESHHANDLE f150;
MESHHANDLE f151;
MESHHANDLE f152;





// 1. vertical lift component (wings and body)

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	static const double CL[nabsc] = {0.1, 0.17, 0.2, 0.2, 0.17, 0.1, 0, -0.11, -0.24, -0.38,  -0.5,  -0.5, -0.02, 0.6355,    0.63,   0.46, 0.28, 0.13, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,    0,0.002,0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = CM[idx] + (CM[idx+1]-CM[idx])*d;
	*cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.02 + oapiGetInducedDrag (*cl, 1.5, 0.6);
}


// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class ShuttlePB: public VESSEL2 {
public:
	ShuttlePB (OBJHANDLE hVessel, int flightmodel);
	//	: VESSEL2 (hVessel, flightmodel) {}
	void clbkSetClassCaps (FILEHANDLE cfg);
	
	~ShuttlePB();

	
	void DefineAnimations(void);
	void Steering();
	void Vectrl();

	//void Lighting();
	void clbkPostCreation (void);
    void clbkPostStep (double simt, double simdt, double mjd);
    int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
   
    UINT anim_wheel;
    UINT anim_gear;
    //UINT anim_gearlever;
    UINT anim_rudder;
    UINT anim_elevator;
    UINT anim_laileron;
    UINT anim_raileron;
    //UINT anim_speedbrake;
    UINT anim_engturn;
    
   // UINT anim_flaps;
    //UINT anim_hook;
    
    UINT anim_power;
   // UINT anim_dial44;
    
   // UINT anim_wheel0;
    //UINT anim_wheel1;
    //UINT anim_wheel2;
    //UINT anim_wheel3;
    //UINT anim_wheel4;
   // UINT anim_wheel5;
   //UINT anim_fuel;

    UINT anim_cover;
    
    UINT anim_turn;

   // UINT anim_dial1;
   // UINT anim_dial2;
   // UINT anim_dial3;
   // UINT anim_bank;
    //UINT anim_clock_s;
    //UINT anim_clock_m;
    //UINT anim_clock_h;
    UINT anim_prop;
    //UINT anim_props;
   // UINT anim_propb;
    // UINT anim_propbs;
    
   // UINT anim_stick0;
   // UINT anim_stick1;
    //UINT anim_stick2;
    
    UINT anim_engturnlb;
    UINT anim_engturnrb;

    
     MGROUP_TRANSFORM *wheel1;
     MGROUP_TRANSFORM *prop1;
     
    // MGROUP_TRANSFORM *stick0;
    // MGROUP_TRANSFORM *stick1;

    
    //bool clbkVCMouseEvent (int id, int event, VECTOR3 &p);

    bool clbkLoadVC (int id);
    
    int MySoundID;	


};

ShuttlePB::ShuttlePB (OBJHANDLE hVessel, int flightmodel)
		: VESSEL2 (hVessel, flightmodel)
{
	
	// preload meshes
	
    f150                 = oapiLoadMeshGlobal ("K33\\K-F-35B");
    f151                 = oapiLoadMeshGlobal ("K33\\K-F-35B-Cpit");
    f152                 = oapiLoadMeshGlobal ("K33\\K-F-35B-Cpit_n");
    

	
	DefineAnimations ();
	
}


void ShuttlePB::clbkPostCreation (void)
{

int MySoundID=ConnectToOrbiterSoundDLL3(GetHandle());

//ReplaceStockSound3(MySoundID,"Sound\\Vessel\\K-Test-Car\\K-Test-Car.wav",	REPLACE_MAIN_THRUST);

RequestLoadVesselWave3(MySoundID,ENG,"Sound\\K-F-35B\\K-F-35B-Eng.wav",BOTHVIEW_FADED_FAR);


RequestLoadVesselWave3(MySoundID,GEAR,"Sound\\K-F-35B\\K-F-35B-Gear.wav",BOTHVIEW_FADED_FAR);

RequestLoadVesselWave3(MySoundID,COVER,"Sound\\K-F-35B\\K-F-35B-Cover.wav",BOTHVIEW_FADED_FAR);

RequestLoadVesselWave3(MySoundID,AFTER,"Sound\\K-F-35B\\K-F-35B-After.wav",BOTHVIEW_FADED_FAR);


SoundOptionOnOff3(MySoundID,PLAYMAINTHRUST,FALSE);

SoundOptionOnOff3(MySoundID,PLAYHOVERTHRUST,FALSE);


SoundOptionOnOff3(MySoundID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);

SoundOptionOnOff3(MySoundID,PLAYCABINAIRCONDITIONING,FALSE);

SoundOptionOnOff3(MySoundID,PLAYCABINRANDOMAMBIANCE,FALSE);

SoundOptionOnOff3(MySoundID,PLAYRADIOATC,FALSE);

SoundOptionOnOff3(MySoundID,DISPLAYTIMER,FALSE);

}


// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------

void ShuttlePB::clbkSetClassCaps (FILEHANDLE cfg)
{
	// THRUSTER_HANDLE th_main;


	// physical specs
	SetSize (8);
	SetEmptyMass (12000.0);
	SetCW (0.3, 0.3, 0.6, 0.9);
	SetWingAspect (1.7);
	SetWingEffectiveness (3.1);
	SetCrossSections (_V(5,5,15));
	SetRotDrag (_V(3.5,3.5,3.5));
	if (GetFlightModel() >= 1) {
		SetPitchMomentScale (1e-4);
		SetBankMomentScale (1e-4);
	}
	SetPMI (_V(15.5,22.1,7.7));
	SetTrimScale (0.05);
	SetCameraOffset (_V(0,0.5,3.1));


	
	SetTouchdownPoints (_V(0,-2.413245,3.10011), _V(-1.285973,-2.4132455,-2.175774), _V(1.285973,-2.413245,-2.175774));

	// **************************** aerodynamics ***********************************
	
    CreateAirfoil (LIFT_VERTICAL,   _V(0,0,-0.5), VLiftCoeff, 20, 270, 2.266);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,-4), HLiftCoeff, 20,  50, 1.5);





	CreateControlSurface (AIRCTRL_ELEVATOR,     7, 0.7, _V(   0,0,-17), AIRCTRL_AXIS_XPOS,anim_elevator);
	CreateControlSurface (AIRCTRL_RUDDER,       2, 0.5, _V(   0,2,-15), AIRCTRL_AXIS_YPOS,anim_rudder);
	CreateControlSurface (AIRCTRL_AILERON,      4.8, 0.2, _V( 17.8,0,-17.8), AIRCTRL_AXIS_XPOS,anim_raileron);
	CreateControlSurface (AIRCTRL_AILERON,      4.8, 0.2, _V(-17.8,0,-17.8), AIRCTRL_AXIS_XNEG,anim_laileron);
	CreateControlSurface (AIRCTRL_ELEVATORTRIM, 5, 0.1, _V(   0,0,-15), AIRCTRL_AXIS_XPOS,anim_elevator);
    //CreateControlSurface (AIRCTRL_FLAP,         5, 5, _V( 0,0,-1), AIRCTRL_AXIS_XPOS,anim_flaps);
	
	//CreateVariableDragElement( &speedbrake, 5.0, _V(0,0,0) );

	//CreateVariableDragElement (&gear_proc2, 0.8, _V(0, -1, 0));
	
	//CreateVariableDragElement (&hook, 3.33, _V(0,1,0));


	
	// propellant resources
	PROPELLANT_HANDLE hpr = CreatePropellantResource (PB_FUELMASS);
	
	
	SURFHANDLE tex_main = oapiRegisterExhaustTexture ("K-F-35B-Exhaust");

	// ***************** thruster definitions *******************

	PARTICLESTREAMSPEC contrail_main = {
		0, 0.3, 165, 19, 0.019, 0.111, 1.11, 1.11, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 2,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	
	
	thg_main =  CreateThruster (_V(0,0,-0.01), _V(0,0,1), PB_MAXMAINTH, hpr, PB_ISP);
	thg_hover =  CreateThruster (_V(0,-0.1580385,-5.802197), _V(0,0,1), PB_MAXMAINTH, hpr, PB_ISP);
	
	
	
	CreateThrusterGroup (&thg_main, 1, THGROUP_MAIN);
	CreateThrusterGroup (&thg_hover, 1, THGROUP_HOVER);
	
	
	AddExhaust (thg_hover, 6, 0.65, _V(0,-0.1580385,-5.802197), _V(0,0,-1) , tex_main);
	
	AddExhaustStream (thg_hover, _V(0,-0.1580385,-5.802197), &contrail_main);



      // Strobe lights
	
	//Red strobe
	
	static VECTOR3 beacon1pos = {-5.473293,-0.1608606,-3.634886};
	static VECTOR3 beacon1col = {0.988, 0.133, 0.133};
	
		beacon1.shape = (BEACONSHAPE_STAR);
		beacon1.pos = &beacon1pos;
		beacon1.col = &beacon1col;
		beacon1.size = (0.2);
		beacon1.falloff = (0.6);
		beacon1.period = (1.13);
		beacon1.duration = (0.05);
		beacon1.tofs = (0.6);
		beacon1.active = false;
		
		AddBeacon (&beacon1);
		
		//Blue strobe
	
	static VECTOR3 beacon2pos = {5.473293,-0.1608606,-3.634886};
	static VECTOR3 beacon2col = {0.043, 0.161, 0.992};
	
		beacon2.shape = (BEACONSHAPE_STAR);
		beacon2.pos = &beacon2pos;
		beacon2.col = &beacon2col;
		beacon2.size = (0.2);
		beacon2.falloff = (0.6);
		beacon2.period = (1.13);
		beacon2.duration = (0.05);
		beacon2.tofs = (0.6);
		beacon2.active = false;
		
		AddBeacon (&beacon2);
		

    

    
    
    
	// visual specs
	
	AddMesh (f150);
	
	
	SetMeshVisibilityMode( AddMesh (f151,&offset), MESHVIS_VC );
	
	

    

}

// ==============================================================
// API callback interface
// ==============================================================

void ShuttlePB::clbkPostStep (double simt, double simdt, double mjd)

{
	
	
	if( oapiGetTimeAcceleration() > 10.0 ) oapiSetTimeAcceleration(10);
	
	double vsAirSpeed = GetAirspeed();
	
	
    //Thrust vectoring
	
	//SetThrusterDir (thg_main ,_V(0,exproc*3,1));
	
	SetThrusterDir (thg_main ,_V(0,exproc*2,1));
	
	if (!GroundContact() && exproc ==1) AddForce( _V( 0,0,-170000 ), _V( 0,0,0 ) );
	
	if (vsAirSpeed >= 33 && exproc ==1) AddForce( _V( 0,-5000,0 ), _V( 0,0,7 ) );

	if (vsAirSpeed >= 44 && exproc ==1) AddForce( _V( 0,-5000,0 ), _V( 0,0,7 ) );
	
    //speed and altitude control
	
	if ( eng_turn < 0.5 && GetAirspeed() >= 700) 
	{
		SetCW (999, 999, 999, 999);
		SetThrusterMax0 (thg_main ,0);
		
}

if ( eng_turn >= 0.5 && GetAirspeed() >= 155) 
	{
		SetCW (999, 999, 999, 999);
		SetThrusterMax0 (thg_main ,0);
		
}

if (GetAltitude() >= 15000) 
	{
		SetCW (999, 999, 999, 999);
		SetThrusterMax0 (thg_main ,0);
			
}

if (eng_turn < 0.5 && GetAirspeed() < 700 && GetAltitude() < 15000 ) 

	{
		
		SetCW (0.3, 0.3, 0.67, 0.92);
		
		if (after == 1)
		{
		SetThrusterMax0 (thg_main ,5.5e5);
		
		if (eng_turn < 0.06)
		{
		SetThrusterLevel (thg_hover,1);
		SetThrusterGroupLevel (thg_hover,1);
	}
		
	}
	
	if (after == 0)
		{
		SetThrusterMax0 (thg_main ,4.4e5);
		SetThrusterLevel (thg_hover,0);
		SetThrusterGroupLevel (thg_hover,0);
		
	}
	
	if (after == 1 && eng_turn < 0.06)
		{
		
		SetThrusterLevel (thg_main,1);
		SetThrusterGroupLevel (thg_main,1);
		
	}
	
	if (eng_turn >= 0.06)
		{
		
		SetThrusterLevel (thg_hover,0);
		SetThrusterGroupLevel (thg_hover,0);
		
	}
	
}


if (eng_turn >= 0.5 && GetAirspeed() < 155 && GetAltitude() < 15000)

		{
		SetCW (0.3, 0.3, 0.67, 0.92);
		SetThrusterMax0 (thg_main ,2.2e5);
}
	

	double vsAirSpeed2 = (vsAirSpeed/1000)*3600;
    
    //Ground Steering
    
    if (GroundContact () && vsAirSpeed > 0.6 && vsAirSpeed < 90 &&  gear_proc < 0.05) Steering();
    
    if (!GroundContact() ) Vectrl();

    //aoa = GetAOA();
    
    //sprintf(oapiDebugString(),"anim %2.2f", aoa );
    
  
    
    if (  gear_proc < 0.05 ) SetTouchdownPoints (_V(0,-2.413245,3.10011), _V(-1.285973,-2.4132455,-2.175774), _V(1.285973,-2.413245,-2.175774));

    


    if (  gear_proc > 0.95 ) SetTouchdownPoints (_V(0,-1.062473,3.10011), _V(-1.285973,-1.062473,-2.175774), _V(1.285973,-1.062473,-2.175774));

    
    //Anims 
    
    if (iop > 1)iop=0;
    
    if (vsAirSpeed > 0.6) {
    
	double aspeed = vsAirSpeed;
    if (iop > 1)iop=0;
    if (vsAirSpeed > 30) aspeed = 30; 
    iop = iop + (aspeed /300);
     
    if (vsAirSpeed ==0) iop=0;
    
    
    if ( GroundContact() && gear_proc < 0.05) SetAnimation (anim_wheel, iop);
    

   }
	
	 // Front Gear Turn Anims1
   if (!GroundContact() && gear_proc < 1.0 && vsAirSpeed > 0.6)
   {
	   tpp = 0.5;
	   SetAnimation( anim_turn, tpp );
}

   // Front Gear Turn Anims2
   if (GroundContact() && gear_proc < 0.05 )
   {
	   
   tpp = (GetControlSurfaceLevel (AIRCTRL_RUDDER) +1) *0.5;
   if (!GroundContact())tpp = 0.5;
   SetAnimation( anim_turn, tpp );
   
}

   
   // Engine Prop Animations
    
    if( GetThrusterLevel (thg_main) > 0)
    
    {
	    
	    Engine_Start = 1;
	    
	    prop_speed = prop_speed + (0.035 + (GetThrusterLevel (thg_main) * 0.1));
    
	    if (prop_speed > 1 ) prop_speed = 0;
	    
	    SetAnimation( anim_prop, prop_speed );
	    
    
}
    

if( GetThrusterLevel (thg_main) == 0 && vsAirSpeed <= 10)
    
    {
	    Engine_Start = 0;
	    
	    prop_speed = prop_speed;
	    
	     SetAnimation( anim_prop, prop_speed );
	     
    
    
}

if( GetThrusterLevel (thg_main) == 0 && vsAirSpeed > 10)
    
    {
	    
	    Engine_Start = 0;
	    
	    double aspeed2 = vsAirSpeed;
	    
	    if (vsAirSpeed > 44) aspeed2 = 44; 
	    
	    prop_speed = prop_speed + (0.01 + (aspeed2 * 0.001));
    
	    if (prop_speed > 1 ) prop_speed = 0;
	    
	    SetAnimation( anim_prop, prop_speed );
	    
    
}




   
// Gear animations


   if( animactive && !GroundContact()) {
		if( animactive == 1 ) {
			gear_proc += 0.00242 * oapiGetTimeAcceleration();
			if( gear_proc > 1.0 ) {
				gear_proc = 1.0;
				animactive = 0;
			}
		}
		else  {
			gear_proc -= 0.00242 * oapiGetTimeAcceleration();
			if( gear_proc < 0.0 ) {
				gear_proc = 0.0;
				animactive = 0;
			}
		}
		SetAnimation( anim_gear, gear_proc );
		gear_proc2 = -gear_proc + 1;
	}

	
	

	
	// Engine Turn animations


   if(first == 0 && GroundContact() && eng_turn < 1.0) {
	   
	   eng_turn += 0.0015 * oapiGetTimeAcceleration();
	   
	   SetAnimation( anim_engturn, eng_turn );
	   
	   exproc = eng_turn;
	   
	   if (eng_turn >= 1.0) first = 1;
	   
   }
	
   if( animactive10 ) {
		if( animactive10 == 1 ) {
			eng_turn += 0.0015 * oapiGetTimeAcceleration();
			if( eng_turn > 1.0 ) {
				eng_turn = 1.0;
				animactive10 = 0;
			}
		}
		else  {
			eng_turn -= 0.0015 * oapiGetTimeAcceleration();
					
			if( eng_turn < 0.0 ) {
				eng_turn = 0.0;
				animactive10 = 0;
			}
		}
		SetAnimation( anim_engturn, eng_turn );
		
		exproc = eng_turn;
	}
	
	
	exproc = eng_turn;
	
	
	if (eng_turn > 0.06)
	{
		cov = cov + 0.007;
		
		if (cov > 1) cov =1;
		
	SetAnimation( anim_cover, cov);
}

if (eng_turn < 0.06)
	{
		cov = cov - 0.007;
		
		if (cov < 0) cov =0;
		
	SetAnimation( anim_cover, cov);
}
   
 

// Engine sound
	
	
	
	
	if (GetThrusterLevel (thg_main) > 0) 
	{
		
	double pit = ((GetThrusterLevel (thg_main)*6000));
	
	eng_pitch = 14000 + pit;
	
	if (eng_pitch < 11500 ) eng_pitch = 11500;
	
	if (eng_pitch > 22500 ) eng_pitch = 22500;
	
	//sprintf(oapiDebugString(),"anim %7.7f", eng_pitch );
		
	if ( oapiCameraInternal ())
	{
	PlayVesselWave3(MySoundID,ENG,LOOP,220,eng_pitch - 800);
}

else

	PlayVesselWave3(MySoundID,ENG,LOOP,255,eng_pitch);
	
}


    if (GetThrusterLevel (thg_main) == 0) 
	{
	
	StopVesselWave3(MySoundID,ENG);
	
}
   

    if (after == 1 && eng_turn < 0.06) 
	{
	
	if ( oapiCameraInternal ())
	{
	PlayVesselWave3(MySoundID,AFTER,LOOP,210,22500);
}

else

	PlayVesselWave3(MySoundID,AFTER,LOOP,245,22500);
	

}

    if (after == 0) 
	{
	
	StopVesselWave3(MySoundID,AFTER);
}

 if (eng_turn >= 0.06) 
	{
	
	StopVesselWave3(MySoundID,AFTER);
}


 if (Lights == 1 && switched == 1)
 {
	 beacon1.active = true;
	 beacon2.active = true;

	 switched = 0;
	
	 SetMeshVisibilityMode( InsertMesh (f152,1,&offset), MESHVIS_VC );	
	 
}

if (Lights == 0 && switched == 1)
{
	beacon1.active = false;
	beacon2.active = false;

    switched = 0;
    
    SetMeshVisibilityMode( InsertMesh (f151,1,&offset), MESHVIS_VC );
 	
}



   // Refuel
   
   if (refuel == 1)
   {
	   
	  SetFuelMass (20000.0);

	   refuel = 0;
   }
 

   // Head Gyroscopic and G-Force movement
   
    if (vsAirSpeed > 0.65) {
		aoa2 = (GetAOA()* 0.5);
		
		if (aoa2 > 0.1333) aoa2 = 0.1333;
		
		if (aoa2 < -0.1333) aoa2 = -0.1333;
		
		//SetCameraOffset (_V(0,0.5,3.1));
		SetCameraOffset (_V(0,0.5 + aoa2,3.1));
		
}
   
//sprintf(oapiDebugString(),"aoa %19.19f", aoa2 );

if (vsAirSpeed <= 0.65) {
	
	aoa2 = 0;
			
	
		//SetCameraOffset (_V(0,0.5,3.1));
		SetCameraOffset (_V(0,0.5 + aoa2,3.1));
	}


	

	
	
	
	
}

//Ground Steering


void ShuttlePB::Steering()
 {

	double pos = GetControlSurfaceLevel(AIRCTRL_RUDDER);
	double lev = 3e6 * pos ;

	if( oapiGetTimeAcceleration() > 10.0 )
		return;

	AddForce( _V( lev,0,0 ), _V( 0,0,3.33 ) );
	AddForce( _V( -1.0*lev,0,0 ), _V( 0,0,-3.33 ) );
	
}


void ShuttlePB::Vectrl()
 {
	 
	 // Rear Rotaion Thruster
	double pos2 = GetControlSurfaceLevel(AIRCTRL_RUDDER);
	double lev2 = 1e6 * pos2 * GetAirspeed() / 10.0;

	if( oapiGetTimeAcceleration() > 10.0 )
		return;

		
    if (GetAirspeed() > 155) return; 
    
    
	if (GetAirspeed() > 5 && GetAirspeed() < 33) 
	{
	AddForce( _V( lev2,0,0 ), _V( 0,0,0.05 ) );
	AddForce( _V( -1.0*lev2,0,0 ), _V( 0,0,-0.05 ) );
}

if (GetAirspeed() >= 33) 
	{
	AddForce( _V( lev2,0,0 ), _V( 0,0,0.01 ) );
	AddForce( _V( -1.0*lev2,0,0 ), _V( 0,0,-0.01 ) );
}



if (GetAirspeed() <= 5) 
	{
	AddForce( _V( lev2,0,0 ), _V( 0,0,0.1 ) );
	AddForce( _V( -1.0*lev2,0,0 ), _V( 0,0,-0.1 ) );
}

	
	 // Elevation Thruster
	 double pos3 = GetControlSurfaceLevel(AIRCTRL_AILERON);
	double lev3 = 1e6 * pos3 * GetAirspeed() / 10.0;

	if( oapiGetTimeAcceleration() > 10.0 )
		return;

    if (GetAirspeed() > 155) return; 
	
    	if (GetAirspeed() > 5 && GetAirspeed() < 33) 
	{
	AddForce( _V( 0,lev3,0 ), _V( -0.05,0,0 ) );
	AddForce( _V( 0,-1.0*lev3,0 ), _V( 0.05,0,0 ) );
}

if (GetAirspeed() >= 33) 
	{
	AddForce( _V( 0,lev3,0 ), _V( -0.01,0,0 ) );
	AddForce( _V( 0,-1.0*lev3,0 ), _V( 0.01,0,0 ) );
}

	 // Bank Thruster
	 double pos = GetControlSurfaceLevel(AIRCTRL_ELEVATOR);
	double lev = 1e6 * pos * GetAirspeed() / 10.0;

	if( oapiGetTimeAcceleration() > 10.0 )
		return;

    if (GetAirspeed() > 155) return; 
	
    if (GetAirspeed() > 5 && GetAirspeed() < 33) 
	{	
	AddForce( _V( 0,0,lev ), _V( 0,-0.05,0 ) );
	AddForce( _V( 0,0,-1.0*lev ), _V( 0,0.05,0 ) );
}

if (GetAirspeed() >= 33) 
	{	
	AddForce( _V( 0,0,lev ), _V( 0,-0.01,0 ) );
	AddForce( _V( 0,0,-1.0*lev ), _V( 0,0.01,0 ) );
}
	
}



// --------------------------------------------------------------
// Keyboard interface handler (buffered key events)
// --------------------------------------------------------------
int ShuttlePB::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate)
{
	if (!down) return 0; // only process keydown events

	// if (KEYMOD_SHIFT (kstate)) {

		switch (key) {
		case OAPI_KEY_G: // Gear
		
		if ( GroundContact() ) return 1;
		
		if( gear_proc > 0.5 )
					animactive = 2;
				else
					animactive = 1;

		if ( !GroundContact()) PlayVesselWave3(MySoundID,GEAR,NOLOOP,255);
		
		
		if( gear_procl > 0.5 )
					animactivel = 2;
				else
					animactivel = 1;

			
			return 1;
				
			case OAPI_KEY_0: // Engine Turn Up and Down
		
		if( eng_turn > 0.5 )
					animactive10 = 2;
				else
					animactive10 = 1;
					
					first = 1;

		PlayVesselWave3(MySoundID,COVER,NOLOOP,255);
		
			
			return 1;

			case OAPI_KEY_9: // Engine Turn Up 10 deg
	
		eng_turn += 0.1;
		
			if( eng_turn > 1.0 ) {
				eng_turn = 1.0;
				}
				
	    SetAnimation( anim_engturn, eng_turn );
	    
		exproc = eng_turn;
		
		first = 1;

		PlayVesselWave3(MySoundID,COVER,NOLOOP,255);
		
			
			return 1;
			
			case OAPI_KEY_8: // Engine Turn Down 10 deg
		
		eng_turn -= 0.1;
		
			if( eng_turn < 0.0 ) {
				eng_turn = 0.0;
				}
				
	    SetAnimation( anim_engturn, eng_turn );
	    
		exproc = eng_turn;
		
		first = 1;

		PlayVesselWave3(MySoundID,COVER,NOLOOP,255);
		
			
			return 1;
			
			case OAPI_KEY_7: // Afterburners
		
		
		
			if( after == 0)
			{
				after = 1;
				
				return 1;
			}
			
        if( after == 1)
			{
				after = 0;
				
				return 1;
			}
			
			
		
			
			return 1;
			
			
			
			case OAPI_KEY_J: // Refuel
			
			
		if (refuel == 0 && GetAirspeed() == 0)
		{
			refuel = 1;
		}
		
			
			return 1;
		
		

			case OAPI_KEY_K: // Lights
			
			if (GetControlSurfaceLevel (AIRCTRL_RUDDER) > 0) return 1;
			
			if (GetControlSurfaceLevel (AIRCTRL_RUDDER) < 0) return 1;
		
			//if (flaps > 0.01) return 1;	
			
			
		if( Lights == 0 )
		{
			Lights = 1;
			switched =1;
					
			return 1;
}

		if( Lights == 1 )
		{
			Lights = 0;
			switched = 1;
					
			return 1;
}
	
    } 
	return 0;
}


// --------------------------------------------------------------
// Destructor
// --------------------------------------------------------------
ShuttlePB::~ShuttlePB ()
{
	delete wheel1;
	delete prop1;
	
	

}



void ShuttlePB::DefineAnimations(void)
{
	
	ANIMATIONCOMPONENT_HANDLE parent1;
	
    ANIMATIONCOMPONENT_HANDLE parent3;
    
	
   // Front Wheel
   static UINT WGrp1[1] = {125};
   
   wheel1 = new MGROUP_ROTATE  (0,WGrp1, 1, _V(0,-2.124649,3.099004), _V(1,0,0), (float)(360*RAD));
   
   
    //Turn Front Gear
   static UINT Trn1[4] = {6,7,8,9};
   
   static MGROUP_ROTATE turn1 (0,Trn1, 4, _V(0,-0.8829094,3.099459), _V(0,1,0), (float)(70*RAD));
   

   
   //Rear left Wheel
   static UINT WGrp3[1] = {126};
   
   static MGROUP_ROTATE wheel2 (0,WGrp3, 1, _V(0,-2.019984,-2.173306), _V(1,0,0), (float)(360*RAD));
   
   //Rear Right wheel
   static UINT WGrp4[1] = {127};
   
   static MGROUP_ROTATE wheel3 (0,WGrp4, 1, _V(0,-2.019984,-2.173306), _V(1,0,0), (float)(360*RAD));
   
   anim_turn = CreateAnimation (0.5);
   anim_wheel = CreateAnimation (0);
   
   parent1 = AddAnimationComponent  (anim_turn, 0, 1, &turn1);
   AddAnimationComponent  (anim_wheel, 0, 1, wheel1, parent1);
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel2);
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel3);
   
   
   
   
   // Gear anims
  
   
   //Front Gear
   static UINT GGrp1[9] = {6,7,8,9,10,11,12,13,125};
   
   static MGROUP_ROTATE gear1 (0,GGrp1, 9, _V(0,-0.9227885,3.099234), _V(1,0,0), (float)(-115*RAD));
   
   
   //Rear Gear Left
   static UINT GGrp2[4] = {14,15,16,126};
   
   static MGROUP_ROTATE gear2 (0,GGrp2, 4, _V(-1.15901,-0.4520788,-2.172986), _V(1,0,0), (float)(-99*RAD));
   
   //Rear Gear Right
   static UINT GGrp3[4] = {18,20,21,127};
   
   static MGROUP_ROTATE gear3 (0,GGrp3, 4, _V(1.15901,-0.4520788,-2.172986), _V(1,0,0), (float)(-99*RAD));
   
    
    //Front Gear Cover
   static UINT GGrp5[1] = {5};
   
   static MGROUP_ROTATE gear5 (0,GGrp5, 1, _V(0.2282761,-0.870366,3.679197), _V(0,0,1), (float)(-93*RAD));
   
     
   //Rear Cover Left
   static UINT GGrp12[2] = {173,174};
   
   static MGROUP_ROTATE gear12 (0,GGrp12, 2, _V(-1.199155,-0.3544314,-1.368714), _V(0,0,1), (float)(99*RAD));
   
   //Rear Cover Right
   static UINT GGrp13[2] = {175,176};
   
   static MGROUP_ROTATE gear13 (0,GGrp13, 2, _V(1.199155,-0.3544314,-1.368714), _V(0,0,1), (float)(-99*RAD));
   
  
   
    //Rear Gear Left2
   static UINT GGrp20[5] = {14,15,16,17,126};
   
   static MGROUP_ROTATE gear20 (0,GGrp20, 5, _V(-1.15901,-0.4520788,-2.172986), _V(0,0,1), (float)(15*RAD));
   
   //Rear Gear Right2
   static UINT GGrp21[5] = {18,19,20,21,127};
   
   static MGROUP_ROTATE gear21 (0,GGrp21, 5, _V(1.15901,-0.4520788,-2.172986), _V(0,0,1), (float)(-15*RAD));
   
     //Rear Gear Left3
   static UINT GGrp23[1] = {17};
   
   static MGROUP_ROTATE gear23 (0,GGrp23, 1, _V(-1.15901,-0.4969712,-1.226534), _V(1,0,0), (float)(50*RAD));
   
   //Rear Gear Right3
   static UINT GGrp24[1] = {19};
   
   static MGROUP_ROTATE gear24 (0,GGrp24, 1, _V(1.15901,-0.4969712,-1.226534), _V(1,0,0), (float)(50*RAD));
   
   
   anim_gear = CreateAnimation (0);
   
   AddAnimationComponent  (anim_gear, 0, 0.5, &gear1);
   AddAnimationComponent  (anim_gear, 0.2, 0.5, &gear2);
   AddAnimationComponent  (anim_gear, 0.2, 0.5, &gear3);
   
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear5);
   
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear12);
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear13);
   
   
   AddAnimationComponent  (anim_gear, 0, 0.2, &gear20);
   AddAnimationComponent  (anim_gear, 0, 0.2, &gear21);
   
   AddAnimationComponent  (anim_gear, 0.2, 0.5, &gear23);
   AddAnimationComponent  (anim_gear, 0.2, 0.5, &gear24);
   
   
   //Rudder Left
   static UINT RGrp1[2] = {118,119};
   
   static MGROUP_ROTATE rudderl (0,RGrp1, 2, _V(-1.514455,0.487969,-6.5), _V(-0.4,1,-0.3), (float)(-20*RAD));
   
   //Rudder Right
   static UINT RGrp2[2] = {121,122};
   
   static MGROUP_ROTATE rudderr (0,RGrp2, 2, _V(1.514455,0.487969,-6.5), _V(0.4,1,-0.3), (float)(-20*RAD));
   
//Cpit Rudder Pedals Pilot Left
   static UINT RGrp3[2] = {1,3};
   
   static MGROUP_ROTATE ruddercl (1,RGrp3, 2, _V(0,6,0), _V(1,0,0), (float)(1.33*RAD));
   
   //Cpit Rudder Pedals Pilot Right
   static UINT RGrp4[2] = {2,4};
   
   static MGROUP_ROTATE ruddercr (1,RGrp4, 2, _V(0,6,0), _V(1,0,0), (float)(-1.33*RAD));

   
   
   
   anim_rudder = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderl);
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderr);
   AddAnimationComponent  (anim_rudder, 0, 1, &ruddercl);
   AddAnimationComponent  (anim_rudder, 0, 1, &ruddercr);
   
   
   //Elevator
   static UINT EGrp1[2] = {133,145};
   
   static MGROUP_ROTATE elevator (0,EGrp1, 2, _V(0,-6.493155E-02,-6.711857), _V(1,0,0), (float)(27*RAD));
   
   
   anim_elevator = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_elevator, 0, 1, &elevator);
   
   
   
   
   
   //Left Aleron
   static UINT LAGrp1[1] = {140};
   
   static MGROUP_ROTATE laileron (0,LAGrp1, 1, _V(-3.166646,-0.104697,-4.633918), _V(1,0,-0.1), (float)(-33*RAD));
   
   
   anim_laileron = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_laileron, 0, 1, &laileron);
   
   
   //Right Aleron
   static UINT RAGrp1[1] = {158};
   
   static MGROUP_ROTATE raileron (0,RAGrp1, 1, _V(3.166646,-0.104697,-4.633918), _V(1,0,0.1), (float)(33*RAD));
   
   
   anim_raileron = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_raileron, 0, 1, &raileron);
   
  
    
  
   //Prop
   static UINT PropGrp1[17] = {177,178,179,180,181,182,183,184,185,186,187,188,189,190,191,192,193};
   
  static MGROUP_ROTATE prop1 (0,PropGrp1, 17, _V(0,0,1.563214), _V(0,1,0), (float)(-360*RAD));
   
  

   
   
   anim_prop = CreateAnimation (0);
   
   AddAnimationComponent  (anim_prop, 0, 1, &prop1);
   
  
  // Engine-Rotate anims
  
   static UINT ETrn0[2] = {32,130};
   
   static MGROUP_ROTATE engineturn0 (0,ETrn0, 2, _V(0,0.6463552,-4.58964), _V(1,0,0), (float)(-33*RAD));
   
   static UINT ETrn1[2] = {31,85};
   
   static MGROUP_ROTATE engineturn1 (0,ETrn1, 2, _V(0,0.5,-5.1), _V(1,0,0), (float)(-70*RAD));


   anim_engturn = CreateAnimation (0);

   AddAnimationComponent  (anim_engturn, 0, 1, &engineturn0);
   AddAnimationComponent  (anim_engturn, 0, 1, &engineturn1);
  
   
   
   
   // Hover Cover Anims
   
   //Rear Cover Left
   static UINT CovGrp0[1] = {164};
   
   static MGROUP_ROTATE cov0 (0,CovGrp0, 1, _V(0,0.847259,0), _V(0,0,1), (float)(80*RAD));
   
    //Rear Cover Right
   static UINT CovGrp1[1] = {165};
   
   static MGROUP_ROTATE cov1 (0,CovGrp1, 1, _V(0,0.847259,0), _V(0,0,1), (float)(-80*RAD));
   
   //Frontr Cover Left1
   static UINT CovGrp2[1] = {166};
   
   static MGROUP_ROTATE cov2 (0,CovGrp2, 1, _V(-0.5228063,0.8016219,0), _V(0,0,1), (float)(95*RAD));
   
    //Front Cover Right1
   static UINT CovGrp3[1] = {168};
   
   static MGROUP_ROTATE cov3 (0,CovGrp3, 1, _V(0.5228063,0.8016219,0), _V(0,0,1), (float)(-95*RAD));
   
   //Frontr Cover Left2
   static UINT CovGrp4[2] = {96,167};
   
   static MGROUP_ROTATE cov4 (0,CovGrp4, 2, _V(-0.5228063,0.8016219,0), _V(0,0,1), (float)(95*RAD));
   
    //Front Cover Right3
   static UINT CovGrp5[2] = {169,170};
   
   static MGROUP_ROTATE cov5 (0,CovGrp5, 2, _V(0.5228063,0.8016219,0), _V(0,0,1), (float)(-95*RAD));
   
   //Bottom Cover Left
   static UINT CovGrp6[1] = {171};
   
   static MGROUP_ROTATE cov6 (0,CovGrp6, 1, _V(-0.525374,-0.894828,0), _V(0,0,1), (float)(-90*RAD));
   
    //Bottom Cover Right
   static UINT CovGrp7[1] = {172};
   
   static MGROUP_ROTATE cov7 (0,CovGrp7, 1, _V(0.525374,-0.894828,0), _V(0,0,1), (float)(90*RAD));
   
   //Frontr Cover Left3
   static UINT CovGrp8[1] = {166};
   
   static MGROUP_ROTATE cov8 (0,CovGrp8, 1, _V(-0.595,1.025,0), _V(0,0,1), (float)(-140*RAD));
   
    //Front Cover Right3
   static UINT CovGrp9[1] = {168};
   
   static MGROUP_ROTATE cov9 (0,CovGrp9, 1, _V(0.595,1.025,0), _V(0,0,1), (float)(140*RAD));
   
   anim_cover = CreateAnimation (0);

   AddAnimationComponent  (anim_cover, 0, 1, &cov0);
   AddAnimationComponent  (anim_cover, 0, 1, &cov1);
   AddAnimationComponent  (anim_cover, 0, 0.6, &cov2);
   AddAnimationComponent  (anim_cover, 0, 0.6, &cov3);
   AddAnimationComponent  (anim_cover, 0, 0.6, &cov4);
   AddAnimationComponent  (anim_cover, 0, 0.6, &cov5);
   AddAnimationComponent  (anim_cover, 0, 1, &cov6);
   AddAnimationComponent  (anim_cover, 0, 1, &cov7);
   AddAnimationComponent  (anim_cover, 0.4, 1, &cov8);
   AddAnimationComponent  (anim_cover, 0.4, 1, &cov9);
  
   
}
	

bool ShuttlePB::clbkLoadVC (int id)
{
	
	SetCameraOffset (_V(0,0.5,3.1));
	
    SetCameraDefaultDirection (_V(0,0,1));

    SetCameraRotationRange (RAD*50, RAD*50, RAD*30, RAD*30);
	
	
	static VCMFDSPEC mfds_left = {1, 28};
    static VCMFDSPEC mfds_right = {1, 29};
    
   
	
    
    oapiVCRegisterMFD (MFD_LEFT, &mfds_left);
    oapiVCRegisterMFD (MFD_RIGHT, &mfds_right);
    

   
    
    
 return 1;

}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new ShuttlePB (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (ShuttlePB*)vessel;
}
