// ==============================================================
//                 ORBITER MODULE: ShuttlePB
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// ShuttlePB.cpp
// Control module for ShuttlePB vessel class
//
// Notes:
// This is an example for a "minimal" vessel implementation which
// only overloads the clbkSetClassCaps method to define vessel
// capabilities and otherwise uses the default VESSEL class
// behaviour.
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#define ENG 				1	
#define GEAR				2	
#define BRAKE               3
#define CANOPY              4
#define SWING               5

#include <orbitersdk.h>
#include <cstdio>
#include <cmath>


#include <string>
#include "OrbiterSoundSDK35.h"

#include <stdio.h>
#include <stdlib.h>

using namespace std;


// ==============================================================
// Some vessel parameters
// ==============================================================
const double PB_FUELMASS = 20000.0;
const double PB_ISP = 50000.0;
const double PB_MAXMAINTH =0;

double iop = 0;
double st = 0;
double pld = 0;
double cen = 0;
double gear_proc = 0;
double gear_proc2 = 0;
double gear_procl = 0;
double power_proc = 0;
double fuel_proc = 0;
double speedbrake = 0;
double swingwing = 0;
double swing = 0;
double swing3 = 0;
double hook = 0;
double canopy = 0;
double flaps = 0;
double flaps2 = 0;
double exproc = 0;
double Lights = 0;
double switched = 0;
double aoa;
double dil = 0;
double eng_pitch = 22000;

double start = 0;
double full = 0.001;
double i = 0;
double i1 = 0;
double i2 = 0;
double i3 = 0;
double i4 = 0;
double i5 = 0;
double i7 = 0;
double i9 = 0;
double i1a = 0;
double i2a = 0;
double i3a = 0;
double i4a = 0;
double i5a = 0;
double s1 = 0;
double s2 = 0;
double s3 = 0;
double s4 = 0;
double s5 = 0;
double ga = 0;
double diff = 0.1;
double k3;
int k4;
char k3buffer [33];
double amet = 0;
double dee = 0;
double i13 = 0;
double i14 = 0;
double i15 = 0;
double i16 = 0;
double i17 = 0;
double i18 = 0;
double i19 = 0;
double secondtime = 0;

UINT animactive;

UINT animactivel;

UINT animactive2;

UINT animactive3;

UINT animactive4;

UINT animactive5;

UINT animactive7;

UINT animactive9;

const VECTOR3 offset = {0,0.2,4.9};

const VECTOR3 offsetalt = {-0.12,0.575,4.9};

const VECTOR3 offsetaoa = {-0.14,0.504,4.9};

const VECTOR3 offsetdirect = {0.14,0.5,4.9};

const VECTOR3 offsetspeed = {0.12,0.575,4.9};

const VECTOR3 offsetbank = {-0.22,0.54,4.9};

const VECTOR3 offsetclock = {0.22,0.54,4.9};

THRUSTER_HANDLE th_main;

BEACONLIGHTSPEC beacon1;    

BEACONLIGHTSPEC beacon2;   

MESHHANDLE f150;
MESHHANDLE f151;
MESHHANDLE f152;
MESHHANDLE f153;
MESHHANDLE f154;
MESHHANDLE f155;
MESHHANDLE f156;
MESHHANDLE f157;
MESHHANDLE f158;
MESHHANDLE f159;
MESHHANDLE f160;
MESHHANDLE f161;
MESHHANDLE f162;
MESHHANDLE f163;
MESHHANDLE f164;


// 1. vertical lift component (wings and body)

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	static const double CL[nabsc] = {0.1, 0.17, 0.2, 0.2, 0.17, 0.1, 0, -0.11, -0.24, -0.38,  -0.5,  -0.5, -0.02, 0.6355,    0.63,   0.46, 0.28, 0.13, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,    0,0.002,0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = CM[idx] + (CM[idx+1]-CM[idx])*d;
	*cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.02 + oapiGetInducedDrag (*cl, 1.5, 0.6);
}


// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class ShuttlePB: public VESSEL2 {
public:
	ShuttlePB (OBJHANDLE hVessel, int flightmodel);
	//	: VESSEL2 (hVessel, flightmodel) {}
	void clbkSetClassCaps (FILEHANDLE cfg);
	void DefineAnimations(void);
	void Steering();
	void Lighting();
	void clbkPostCreation (void);
    void clbkPostStep (double simt, double simdt, double mjd);
    int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
   
    UINT anim_wheel;
    UINT anim_gear;
    UINT anim_gearlever;
    UINT anim_rudder;
    UINT anim_elevator;
    UINT anim_laileron;
    UINT anim_raileron;
    UINT anim_speedbrake;
    UINT anim_swingwing;
    UINT anim_flaps;
    UINT anim_hook;
    UINT anim_canopy;
    UINT anim_power;
    UINT anim_fuel;
    
    UINT anim_wheel0;
    UINT anim_wheel1;
    UINT anim_wheel2;
    UINT anim_wheel3;
    UINT anim_wheel4;
    UINT anim_wheel5;
    
    UINT anim_dial1;
    UINT anim_dial2;
    UINT anim_dial3;
    UINT anim_bank;
    UINT anim_clock_s;
    UINT anim_clock_m;
    UINT anim_clock_h;

    
    bool clbkLoadVC (int id);
    
    int MySoundID;	


};

ShuttlePB::ShuttlePB (OBJHANDLE hVessel, int flightmodel)
		: VESSEL2 (hVessel, flightmodel)
{
	
	// preload meshes
	
    f150                 = oapiLoadMeshGlobal ("K33\\K-F14A");
    f151                 = oapiLoadMeshGlobal ("K33\\K-F14A-Cpit");
    f152                 = oapiLoadMeshGlobal ("K33\\K-F14A-Cpit-Alt");
    f153                 = oapiLoadMeshGlobal ("K33\\K-F14A-Cpit-AOA");
    f154                 = oapiLoadMeshGlobal ("K33\\K-F14A-Cpit-Direct");
    f155                 = oapiLoadMeshGlobal ("K33\\K-F14A-Cpit_n");
    f156                 = oapiLoadMeshGlobal ("K33\\K-F14A-Cpit-Alt_n");
    f157                 = oapiLoadMeshGlobal ("K33\\K-F14A-Cpit-AOA_n");
    f158                 = oapiLoadMeshGlobal ("K33\\K-F14A-Cpit-Direct_n");
    f159                 = oapiLoadMeshGlobal ("K33\\K-F14A-Cpit-Speed");
    f160                 = oapiLoadMeshGlobal ("K33\\K-F14A-Cpit-Speed_n");
	f161                 = oapiLoadMeshGlobal ("K33\\K-F14A-Cpit-Bank");
    f162                 = oapiLoadMeshGlobal ("K33\\K-F14A-Cpit-Bank_n");
    f163                 = oapiLoadMeshGlobal ("K33\\K-F14A-Cpit-Clock");
    f164                 = oapiLoadMeshGlobal ("K33\\K-F14A-Cpit-Clock_n");
	
	DefineAnimations ();
}


void ShuttlePB::clbkPostCreation (void)
{

int MySoundID=ConnectToOrbiterSoundDLL3(GetHandle());

//ReplaceStockSound3(MySoundID,"Sound\\Vessel\\K-Test-Car\\K-Test-Car.wav",	REPLACE_MAIN_THRUST);

RequestLoadVesselWave3(MySoundID,ENG,"Sound\\Vessel\\K-F14A\\K-F14A-Eng.wav",BOTHVIEW_FADED_FAR);

RequestLoadVesselWave3(MySoundID,BRAKE,"Sound\\Vessel\\K-F14A\\K-F14A-Brake.wav",BOTHVIEW_FADED_FAR);

RequestLoadVesselWave3(MySoundID,GEAR,"Sound\\Vessel\\K-F14A\\K-F14A-Gear.wav",BOTHVIEW_FADED_FAR);

RequestLoadVesselWave3(MySoundID,CANOPY,"Sound\\Vessel\\K-F14A\\K-F14A-Canopy.wav",BOTHVIEW_FADED_FAR);

RequestLoadVesselWave3(MySoundID,SWING,"Sound\\Vessel\\K-F14A\\K-F14A-Swing.wav",BOTHVIEW_FADED_FAR);


SoundOptionOnOff3(MySoundID,PLAYMAINTHRUST,FALSE);

SoundOptionOnOff3(MySoundID,PLAYHOVERTHRUST,FALSE);


SoundOptionOnOff3(MySoundID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);

SoundOptionOnOff3(MySoundID,PLAYCABINAIRCONDITIONING,FALSE);

SoundOptionOnOff3(MySoundID,PLAYCABINRANDOMAMBIANCE,FALSE);

SoundOptionOnOff3(MySoundID,PLAYRADIOATC,FALSE);

SoundOptionOnOff3(MySoundID,DISPLAYTIMER,FALSE);

}


// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------

void ShuttlePB::clbkSetClassCaps (FILEHANDLE cfg)
{
	// THRUSTER_HANDLE th_main;


	// physical specs
	SetSize (8);
	SetEmptyMass (12000.0);
	SetCW (0.3, 0.3, 0.6, 0.9);
	SetWingAspect (1.7);
	SetWingEffectiveness (3.1);
	SetCrossSections (_V(5,5,15));
	SetRotDrag (_V(3.5,3.5,3.5));
	if (GetFlightModel() >= 1) {
		SetPitchMomentScale (1e-4);
		SetBankMomentScale (1e-4);
	}
	SetPMI (_V(15.5,22.1,7.7));
	SetTrimScale (0.05);
	SetCameraOffset (_V(0,0.7303072,4.200835));
	
	SetTouchdownPoints (_V(0,-2.2,9), _V(-4,-2.1,-5), _V(4,-2.1,-5));

	// **************************** aerodynamics ***********************************

	CreateAirfoil (LIFT_VERTICAL,   _V(0,0,-0.5), VLiftCoeff, 20, 270, 2.266);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,-4), HLiftCoeff, 20,  50, 1.5);





	CreateControlSurface (AIRCTRL_ELEVATOR,     7, 0.7, _V(   0,0,-17), AIRCTRL_AXIS_XPOS,anim_elevator);
	CreateControlSurface (AIRCTRL_RUDDER,       2, 0.5, _V(   0,2,-15), AIRCTRL_AXIS_YPOS,anim_rudder);
	CreateControlSurface (AIRCTRL_AILERON,      4.8, 0.2, _V( 17.8,0,-17.8), AIRCTRL_AXIS_XPOS,anim_raileron);
	CreateControlSurface (AIRCTRL_AILERON,      4.8, 0.2, _V(-17.8,0,-17.8), AIRCTRL_AXIS_XNEG,anim_laileron);
	CreateControlSurface (AIRCTRL_ELEVATORTRIM, 5, 0.1, _V(   0,0,-15), AIRCTRL_AXIS_XPOS,anim_elevator);
    CreateControlSurface (AIRCTRL_FLAP,         5, 5, _V( 0,0,-1), AIRCTRL_AXIS_XPOS,anim_flaps);
	
	CreateVariableDragElement( &speedbrake, 8.0, _V(0,0,0) );

	CreateVariableDragElement (&gear_proc2, 0.8, _V(0, -1, 0));
	
	CreateVariableDragElement (&swing3, 0.8, _V(0,0,0));
	
	CreateVariableDragElement (&canopy, 3.33, _V(0,1,0));

	
	// propellant resources
	PROPELLANT_HANDLE hpr = CreatePropellantResource (PB_FUELMASS);
	
	
	SURFHANDLE tex_main = oapiRegisterExhaustTexture ("K-F14A-Exhaust");



	// ***************** thruster definitions *******************

	PARTICLESTREAMSPEC contrail_main = {
		0, 0.34, 165, 19, 0.019, 0.111, 1.11, 1.11, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 2,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};
	
	th_main = CreateThruster (_V(0,0,-5), _V(0,0,1), PB_MAXMAINTH, hpr, PB_ISP);
	
	
	
	CreateThrusterGroup (&th_main, 1, THGROUP_MAIN);
	
	
	
	AddExhaust (th_main, 7, 0.55, _V(-1.1132,-0.54,-6), _V(0,0,-1) , tex_main);
	AddExhaust (th_main, 7, 0.55, _V(1.1132,-0.54,-6), _V(0,0,-1) , tex_main);

    
    AddExhaustStream (th_main, _V(-1.1132,-0.54,-7.76126), &contrail_main);
    AddExhaustStream (th_main, _V(1.1132,-0.54,-7.76126), &contrail_main);
    
    
     // Strobe lights
	
	//Red Right Tail strobe
	
	static VECTOR3 beacon1pos = {1.43,2.11,-8.31};
	static VECTOR3 beacon1col = {0.988, 0.133, 0.133};
	
		beacon1.shape = (BEACONSHAPE_STAR);
		beacon1.pos = &beacon1pos;
		beacon1.col = &beacon1col;
		beacon1.size = (0.12);
		beacon1.falloff = (0.6);
		beacon1.period = (1.13);
		beacon1.duration = (0.05);
		beacon1.tofs = (0.6);
		beacon1.active = false;
		
		AddBeacon (&beacon1);
		
		//Red Left Tail strobe
	
	static VECTOR3 beacon2pos = {-1.43,2.11,-8.31};
	static VECTOR3 beacon2col = {0.988, 0.133, 0.133};
	
		beacon2.shape = (BEACONSHAPE_STAR);
		beacon2.pos = &beacon2pos;
		beacon2.col = &beacon2col;
		beacon2.size = (0.12);
		beacon2.falloff = (0.6);
		beacon2.period = (1.13);
		beacon2.duration = (0.05);
		beacon2.tofs = (0.6);
		beacon2.active = false;
		
		AddBeacon (&beacon2);
		

    
    
    
    
	// visual specs
	
	AddMesh (f150);
	
    SetMeshVisibilityMode( AddMesh (f151,&offset), MESHVIS_VC );
    SetMeshVisibilityMode( AddMesh (f152,&offsetalt), MESHVIS_VC );
    SetMeshVisibilityMode( AddMesh (f153,&offsetaoa), MESHVIS_VC );
    SetMeshVisibilityMode( AddMesh (f154,&offsetdirect), MESHVIS_VC );
    SetMeshVisibilityMode( AddMesh (f159,&offsetspeed), MESHVIS_VC );
    SetMeshVisibilityMode( AddMesh (f161,&offsetbank), MESHVIS_VC );
    SetMeshVisibilityMode( AddMesh (f163,&offsetclock), MESHVIS_VC );
    

}

// ==============================================================
// API callback interface
// ==============================================================

void ShuttlePB::clbkPostStep (double simt, double simdt, double mjd)

{
	
	if( oapiGetTimeAcceleration() > 10.0 ) oapiSetTimeAcceleration(10);
	
	double vsAirSpeed = GetAirspeed();
	
	
	
	
    //speed and altitude control
	
	if (GetAirspeed() >= 700) 
	{
		SetCW (999, 999, 999, 999);
		SetThrusterMax0 (th_main ,0);

	
}

if (GetAltitude() >= 17000) 
	{
		SetCW (999, 999, 999, 999);
		SetThrusterMax0 (th_main ,0);

	
}



if (GetAirspeed() < 700 && GetAltitude() < 17000 ) 
	{
		SetCW (0.3, 0.3, 0.67, 0.92);
		SetThrusterMax0 (th_main ,4.4e5);

	
}

	double vsAirSpeed2 = (vsAirSpeed/1000)*3600;
    
    //Ground Steering
    
    if (GroundContact () ) Steering();
    
    
    //aoa = GetAOA();
    
    //sprintf(oapiDebugString(),"anim %2.2f", aoa );
    
  

    //Anims 
    
    if (iop > 1)iop=0;
    
    if (vsAirSpeed > 0) {
    
	double aspeed = vsAirSpeed;
    if (iop > 1)iop=0;
    if (vsAirSpeed > 30) aspeed = 30; 
    iop = iop + (aspeed /300);
     
    if (vsAirSpeed ==0) iop=0;
    
    
    if ( GroundContact() && gear_proc < 0.05) SetAnimation (anim_wheel, iop);
    

   }
	
	
// Gear animations


   if( animactive && !GroundContact()) {
		if( animactive == 1 ) {
			gear_proc += 0.00242;
			if( gear_proc > 1.0 ) {
				gear_proc = 1.0;
				animactive = 0;
			}
		}
		else  {
			gear_proc -= 0.00242;
			if( gear_proc < 0.0 ) {
				gear_proc = 0.0;
				animactive = 0;
			}
		}
		SetAnimation( anim_gear, gear_proc );
		gear_proc2 = -gear_proc + 1;
	}

	
	// Gear Lever animations


   if( animactivel && !GroundContact()) {
		if( animactivel == 1 ) {
			gear_procl += 0.00242;
			if( gear_procl > 1.0 ) {
				gear_procl = 1.0;
				animactivel = 0;
			}
		}
		else  {
			gear_procl -= 0.00242;
			if( gear_procl < 0.0 ) {
				gear_procl = 0.0;
				animactivel = 0;
			}
		}
		SetAnimation( anim_gearlever, gear_procl );
	}

	
	// Power Guage animation

        power_proc = GetThrusterLevel(th_main);
	
		SetAnimation( anim_power, power_proc );
		
	
    // Fuel Guage animation

        fuel_proc = (-GetFuelMass() + 20000)/20000; 
        
        if (fuel_proc < 0)fuel_proc = 0;
        
        if (fuel_proc > 1)fuel_proc = 1;
	
		SetAnimation( anim_fuel, fuel_proc );
	
	

   // Speedbrake animations

   if( animactive2 ) {
		if( animactive2 == 1 ) {
			speedbrake += 0.006;
			if( speedbrake > 1.0 ) {
				speedbrake = 1.0;
				animactive2 = 0;
			}
		}
		else  {
			speedbrake -= 0.006;
			if( speedbrake < 0.0 ) {
				speedbrake = 0.0;
				animactive2 = 0;
			}
		}
		SetAnimation( anim_speedbrake, speedbrake );
	}

	
	
	// Hook Animations


   if( animactive9 ) {
		if( animactive9 == 1 ) {
			hook += 0.005;
			if( hook > 1.0 ) {
				hook = 1.0;
				animactive9 = 0;
			}
		}
		else  {
			hook -= 0.005;
			if( hook < 0.0 ) {
				hook = 0.0;
				animactive9 = 0;
			}
		}
		SetAnimation( anim_hook, hook );
	}
	
   
	// Canopy Animations


   if( animactive5 ) {
		if( animactive5 == 1 ) {
			canopy += 0.003;
			if( canopy > 1.0 ) {
				canopy = 1.0;
				animactive5 = 0;
			}
		}
		else  {
			canopy -= 0.003;
			if( canopy < 0.0 ) {
				canopy = 0.0;
				animactive5 = 0;
			}
		}
		SetAnimation( anim_canopy, canopy );
	}
	
	
	
	
   // Flaps


   if( animactive4 ) {
		if( animactive4 == 1 ) {
			flaps += 0.006;
			if( flaps > 1.0 ) {
				flaps = 1.0;
				animactive4 = 0;
			}
		}
		else  {
			flaps -= 0.006;
			if( flaps < 0.0 ) {
				flaps = 0.0;
				animactive4 = 0;
			}
		}
		SetAnimation( anim_flaps, flaps );
	}

   SetControlSurfaceLevel(AIRCTRL_FLAP,flaps);

  
   
     // Swingwing animations


   if( animactive7 ) {
		if( animactive7 == 1 ) {
			swingwing += 0.002;
			if( swingwing > 1.0 ) {
				swingwing = 1.0;
				animactive7 = 0;
			}
		}
		else  {
			swingwing -= 0.002;
			if( swingwing < 0.0 ) {
				swingwing = 0.0;
				animactive7 = 0;
			}
		}
		SetAnimation( anim_swingwing, swingwing );
		swing3 = -swingwing + 1;
	}

// Engine sound
	
	
	
	
	if (GetThrusterLevel (th_main) > 0) 
	{
		
	double pit = ((GetThrusterLevel (th_main)*6000));
	
	eng_pitch = 13000 + pit;
	
	if (eng_pitch < 11500 ) eng_pitch = 11500;
	
	if (eng_pitch > 22500 ) eng_pitch = 22500;
	
	//sprintf(oapiDebugString(),"anim %7.7f", eng_pitch );
		
	PlayVesselWave3(MySoundID,ENG,LOOP,255,eng_pitch);
}


    if (GetThrusterLevel (th_main) == 0) 
	{
	
	StopVesselWave3(MySoundID,ENG);
}
   
 if (Lights == 1 && switched == 1)
 {
	 beacon1.active = true;
	 beacon2.active = true;

	 switched = 0;
	 Lighting();
}

if (Lights == 0 && switched == 1)
{
	beacon1.active = false;
	beacon2.active = false;

    switched = 0;
    Lighting();
}



// Altimeter animation
   
   
    //Anims 
    
     k3 = GetAltitude();
    
     k4 = int (k3);
      
     gcvt (k3,6,k3buffer);
     
     

   
     string stringOne(k3buffer);
     
     if (k3 >= 0 )
     {

     string is1 = stringOne.substr(0 + s2 + s3 + s4 + s5, 1) ;
       
     char const *cString1 = is1.c_str();
     
     i1a = atol(cString1);
     
     i1 = i1a * 0.1;
     
 }
 
 if (k3 >= 10 ) 
 {
	 
     string is2 = stringOne.substr(0 + s3 + s4 + s5 , 1) ;
       
     char const *cString2 = is2.c_str();
     
     i2a = atol(cString2);
     
     i2 = i2a * 0.1;
      s2 = 1;
 }
 
 if (k3 >= 100 )
 {
     string is3 = stringOne.substr(0 + s4 + s5, 1) ;
       
     char const *cString3 = is3.c_str();
     
     i3a = atol(cString3);
     
     i3 = i3a * 0.1;
     
     s3 = 1;
 }
 
 if (k3 >= 1000)
 {
     string is4 = stringOne.substr(0 + s5, 1) ;
       
     char const *cString4 = is4.c_str();
     
     i4a = atol(cString4);
     
     i4 = i4a * 0.1;
     
     s4 = 1;
 }
 
 if (k3 >= 10000)
 {
     string is5 = stringOne.substr(0, 1) ;
       
     char const *cString5 = is5.c_str();
     
     i5a = atol(cString5);
     
     i5 = i5a * 0.1;
     
     s5 = 1;
}

      if(k3<10)
      {
	      s2=0;
	      i2=0;
	      s3=0;
	      i3=0;
	      s4=0;
	      i4=0;
	      s5=0;
	      i5=0;
    }
         
    if(k3<100)
      {
	     
	      s3=0;
	      i3=0;
	      s4=0;
	      i4=0;
	      s5=0;
	      i5=0;
    }
    
    if(k3<1000)
      {
	      
	      s4=0;
	      i4=0;
	      s5=0;
	      i5=0;
    }
   
    if(k3<10000)
      {
	     
	      s5=0;
	      i5=0;
	      
    }
    
    
   i=(k3 * 0.01)- dee;
   
   if (i>=1)
   {
	   dee = dee + 1;
   }
   
   if (i <= 0)
   {
	   dee = dee - 1;
	   
	   if (dee < 0)dee = 0;
   }
   
    
	//sprintf(oapiDebugString(),"force %2.2f", i1 );
	
	
	SetAnimation (anim_wheel0, i);
	SetAnimation (anim_wheel1, i1);
	SetAnimation (anim_wheel2, i2);
	SetAnimation (anim_wheel3, i3);
	SetAnimation (anim_wheel4, i4);
	SetAnimation (anim_wheel5, i5);
	
	


// AOA Dial Animation

    aoa = GetAOA();
    
    
   i7 = aoa * 6;

   SetAnimation (anim_dial1, i7);
   
   
   // Direction Dial Animation

   oapiGetFocusHeading(&dil);
    
   i9 = (dil / 6.25) + 0.001;
   
   //sprintf(oapiDebugString(),"hed %2.2f", dil );

   SetAnimation (anim_dial2, i9);
   
   
   // Speed Dial Animation
    
   i13 = GetAirspeed();
   
   if (i13 > 600) i13 = 600;
   
   if (i13 < 0) i13 = 0;
   
   i14 = (-i13 + 600)/600;
   
   if (i14 < 0) i14 = 0;
   
   if (i14 > 1) i14 = 1;
   
   SetAnimation (anim_dial3, i14);
   
    // Bank Guage Animation
    
   i15 = GetBank();
   
   i16 = (i15 + 3) / 6;
   
   if (i16 < 0) i16 = 0;
   
   if (i16 > 1) i16 = 1;
   
   SetAnimation (anim_bank, i16);
   
    //sprintf(oapiDebugString(),"hed %5.5f", i15 );
    
    
     // Clock Dial Animation
     
     //Second Hand
     
    
   i17 = i17 + ( 0.0002 * oapiGetTimeAcceleration() );
   
   if (i17 < 0) i17 = 0;
   
   if (i17 >= 1) 
   {
	   i17 = 0;
	   i18 = i18 + 0.016666666;
}
   SetAnimation (anim_clock_s, i17);
   
  // sprintf(oapiDebugString(),"time %19.19f", i17 );
   
    //Minute Hand
    
   if (i18 < 0) i18 = 0;
   
   if (i18 >= 1) 
   {
	   i18 = 0;
	   i19 = i19 + 0.016666666;
   }
   
   
   SetAnimation (anim_clock_m, i18);
   
    //Hour Hand
   
   if (i19 < 0) i19 = 0;
   
   if (i19 >= 1) i19 = 0;
   
   SetAnimation (anim_clock_h, i19);

}

//Ground Steering


void ShuttlePB::Steering()
 {

	double pos = GetControlSurfaceLevel(AIRCTRL_RUDDER);
	double lev = 1e6 * pos * GetAirspeed() / 10.0;

	if( oapiGetTimeAcceleration() > 10.0 )
		return;

	AddForce( _V( lev,0,0 ), _V( 0,0,0.33 ) );
	AddForce( _V( -1.0*lev,0,0 ), _V( 0,0,-0.33 ) );
	
}

// Lighting


void ShuttlePB::Lighting()
 {
    if(Lights == 1)
    {
	
	SetMeshVisibilityMode( InsertMesh (f155,1,&offset), MESHVIS_VC );
	SetMeshVisibilityMode( InsertMesh (f156,2,&offsetalt), MESHVIS_VC );
	SetMeshVisibilityMode( InsertMesh (f157,3,&offsetaoa), MESHVIS_VC );
	SetMeshVisibilityMode( InsertMesh (f158,4,&offsetdirect), MESHVIS_VC );
	SetMeshVisibilityMode( InsertMesh (f160,5,&offsetspeed), MESHVIS_VC );
	SetMeshVisibilityMode( InsertMesh (f162,6,&offsetbank), MESHVIS_VC );
	SetMeshVisibilityMode( InsertMesh (f164,7,&offsetclock), MESHVIS_VC );
}

    if(Lights == 0)
    {
	
	SetMeshVisibilityMode( InsertMesh (f151,1,&offset), MESHVIS_VC );
	SetMeshVisibilityMode( InsertMesh (f152,2,&offsetalt), MESHVIS_VC );
	SetMeshVisibilityMode( InsertMesh (f153,3,&offsetaoa), MESHVIS_VC );
	SetMeshVisibilityMode( InsertMesh (f154,4,&offsetdirect), MESHVIS_VC );
	SetMeshVisibilityMode( InsertMesh (f159,5,&offsetspeed), MESHVIS_VC );
	SetMeshVisibilityMode( InsertMesh (f161,6,&offsetbank), MESHVIS_VC );
	SetMeshVisibilityMode( InsertMesh (f163,7,&offsetclock), MESHVIS_VC );
}

}


// --------------------------------------------------------------
// Keyboard interface handler (buffered key events)
// --------------------------------------------------------------
int ShuttlePB::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate)
{
	if (!down) return 0; // only process keydown events

	// if (KEYMOD_SHIFT (kstate)) {

		switch (key) {
		case OAPI_KEY_G: // Gear
		
		if( gear_proc > 0.5 )
					animactive = 2;
				else
					animactive = 1;

		if ( !GroundContact()) PlayVesselWave3(MySoundID,GEAR,NOLOOP,255);
		
		
		if( gear_procl > 0.5 )
					animactivel = 2;
				else
					animactivel = 1;

			
			return 1;
			
			
			case OAPI_KEY_B: // Speedbrake
		
		if( speedbrake > 0.5 )
					animactive2 = 2;
				else
					animactive2 = 1;

		PlayVesselWave3(MySoundID,BRAKE,NOLOOP,255);
		
			
			return 1;
			
			case OAPI_KEY_9: // Hook
		
		if( hook > 0.5 )
					animactive9 = 2;
				else
					animactive9 = 1;

		PlayVesselWave3(MySoundID,BRAKE,NOLOOP,255);
		
			
			return 1;
			
			case OAPI_KEY_8: // Canopy
		
		if( canopy > 0.5 )
					animactive5 = 2;
				else
					animactive5 = 1;

		PlayVesselWave3(MySoundID,CANOPY,NOLOOP,255);
		
			
			return 1;
			
			case OAPI_KEY_F: // Flaps
			
			if (swingwing == 0)
			{
		
		if( flaps > 0.5 )
					animactive4 = 2;
				else
					animactive4 = 1;

		PlayVesselWave3(MySoundID,BRAKE,NOLOOP,255);
		
			
			return 1;
		}
		else
		
		return 1;
		
		case OAPI_KEY_0: // Swingwing
		
		if (flaps == 0)
		{
		
		if( swingwing > 0.5 )
					animactive7 = 2;
				else
					animactive7 = 1;

		PlayVesselWave3(MySoundID,SWING,NOLOOP,255);
		
		
        return 1;
    }
    else
    
    return 1;
    

			case OAPI_KEY_K: // Lights
		
		if( Lights == 0 )
		{
			Lights = 1;
			switched =1;
					
			return 1;
}

		if( Lights == 1 )
		{
			Lights = 0;
			switched = 1;
					
			return 1;
}
	
    } 
	return 0;
}


void ShuttlePB::DefineAnimations(void)
{
   // Front Wheel
   static UINT WGrp1[2] = {76,77};
   
   static MGROUP_ROTATE wheel1 (0,WGrp1, 2, _V(0,-1.913335,3.658915), _V(1,0,0), (float)(360*RAD));
   
   //Rear left Wheel
   static UINT WGrp3[1] = {124};
   
   static MGROUP_ROTATE wheel2 (0,WGrp3, 1, _V(0,-1.699602,-2.574417), _V(1,0,0), (float)(360*RAD));
   
   //Rear Right wheel
   static UINT WGrp4[1] = {125};
   
   static MGROUP_ROTATE wheel3 (0,WGrp4, 1, _V(0,-1.699602,-2.574417), _V(1,0,0), (float)(360*RAD));
   
   anim_wheel = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel1);
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel2);
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel3);
   
   
   
   
   // Gear anims
  
   
   //Front Gear
   static UINT GGrp1[7] = {32,33,34,35,36,76,77};
   
   static MGROUP_ROTATE gear1 (0,GGrp1, 7, _V(0,-0.8,3.660742), _V(1,0,0), (float)(-111*RAD));
   
   //Front Gear2
   static UINT GGrp6[2] = {37,38};
   
   static MGROUP_ROTATE gear6 (0,GGrp6, 2, _V(0,-0.67,2.628464), _V(1,0,0), (float)(-50*RAD));
   
   //Rear Gear Left
   static UINT GGrp2[5] = {124,195,196,197,198};
   
   static MGROUP_ROTATE gear2 (0,GGrp2, 5, _V(-1.795179,-0.4019121,-2.574151), _V(0,1,0), (float)(90*RAD));
   
   //Rear Gear Right
   static UINT GGrp3[5] = {125,199,200,201,202};
   
   static MGROUP_ROTATE gear3 (0,GGrp3, 5, _V(1.795179,-0.4019121,-2.574151), _V(0,1,0), (float)(-90*RAD));
   
   //Front Gear Cover Left
   static UINT GGrp4[2] = {28,30};
   
   static MGROUP_ROTATE gear4 (0,GGrp4, 2, _V(-0.2150556,-0.8087077,4.683463), _V(0,0,1), (float)(90*RAD));
   
   //Front Gear Cover Right
   static UINT GGrp5[2] = {29,31};
   
   static MGROUP_ROTATE gear5 (0,GGrp5, 2, _V(0.2150556,-0.8087077,4.683463), _V(0,0,1), (float)(-90*RAD));
   
   //Rear Gear Left2
   static UINT GGrp8[5] = {124,195,196,197,198};
   
   static MGROUP_ROTATE gear8 (0,GGrp8, 5, _V(-1.795179,0,-2.574151), _V(1,0,0), (float)(-90*RAD));
   
   //Rear Gear Right2
   static UINT GGrp9[5] = {125,199,200,201,202};
   
   static MGROUP_ROTATE gear9 (0,GGrp9, 5, _V(1.795179,0,-2.574151), _V(1,0,0), (float)(-90*RAD));
   
   
   anim_gear = CreateAnimation (0);
   
   AddAnimationComponent  (anim_gear, 0, 0.5, &gear1);
   AddAnimationComponent  (anim_gear, 0, 0.25, &gear2);
   AddAnimationComponent  (anim_gear, 0, 0.25, &gear3);
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear4);
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear5);
   AddAnimationComponent  (anim_gear, 0, 0.5, &gear6);
   AddAnimationComponent  (anim_gear, 0.25, 0.8, &gear8);
   AddAnimationComponent  (anim_gear, 0.25, 0.8, &gear9);
   
   
   
   //Rudder left
   static UINT RGrp1[2] = {127,129};
   
   static MGROUP_ROTATE rudderl (0,RGrp1, 2, _V(-1.332634,0.9283338,-7.474399), _V(0,1,-0.5), (float)(-22*RAD));
   
   //Rudder right
   static UINT RGrp2[2] = {130,194};
   
   static MGROUP_ROTATE rudderr (0,RGrp2, 2, _V(1.332634,0.9283338,-7.474399), _V(0,1,-0.5), (float)(-22*RAD));
   
   anim_rudder = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderl);
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderr);
   
   
   //Elevator
   static UINT EGrp1[2] = {188,192};
   
   static MGROUP_ROTATE elevator (0,EGrp1, 2, _V(0,-0.4484807,-6.004059), _V(1,0,0), (float)(18*RAD));
   
   
   anim_elevator = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_elevator, 0, 1, &elevator);
   
   
   
   
   //Left Aleron
   static UINT LAGrp1[1] = {188};
   
   static MGROUP_ROTATE laileron (0,LAGrp1, 1, _V(-1.885501,-0.4484807,-6.004059), _V(1,0,0), (float)(-18*RAD));
   
   
   anim_laileron = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_laileron, 0, 1, &laileron);
   
   
   //Right Aleron
   static UINT RAGrp1[1] = {192};
   
   static MGROUP_ROTATE raileron (0,RAGrp1, 1, _V(1.885501,-0.4484807,-6.004059), _V(1,0,0), (float)(18*RAD));
   
   
   anim_raileron = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_raileron, 0, 1, &raileron);
   
   
   //Speedbrake top
   static UINT SBGrp1[5] = {5,6,7,8,9};
   
   static MGROUP_ROTATE speedbrake (0,SBGrp1, 5, _V(0,-0.2038314,-6.395447), _V(1,0,0), (float)(30*RAD));
   
   //Speedbrake bottom
   static UINT SBGrp2[2] = {10,11};
   
   static MGROUP_ROTATE speedbrake2 (0,SBGrp2, 2, _V(0,-0.6887364,-6.045441), _V(1,0,0), (float)(-30*RAD));
   
   
   anim_speedbrake = CreateAnimation (0);
   
   AddAnimationComponent  (anim_speedbrake, 0, 1, &speedbrake);
   AddAnimationComponent  (anim_speedbrake, 0, 1, &speedbrake2);
   
  
   //Hook
   static UINT HGrp1[1] = {39};
   
   static MGROUP_ROTATE hook (0,HGrp1, 1, _V(0,-0.6887364,-6.045441), _V(1,0,0), (float)(-40*RAD));
   
   
   anim_hook = CreateAnimation (0);
   
   AddAnimationComponent  (anim_hook, 0, 1, &hook);
   
   //Canopy
   static UINT CGrp1[9] = {16,17,18,19,20,21,22,23,219};
   
   static MGROUP_ROTATE can1 (0,CGrp1, 9, _V(0,0.7447336,1.477891), _V(1,0,0), (float)(-30*RAD));
   
   
   anim_canopy = CreateAnimation (0);
   
   AddAnimationComponent  (anim_canopy, 0, 1, &can1);
   
   //Flaps animation
   
   //Left Flap
   static UINT FLGrp1[4] = {179,180,181,182};
   
   static MGROUP_ROTATE flapsl (0,FLGrp1, 4, _V(-4.942456,0.1215288,-3.412283), _V(1,0,0.17), (float)(-45*RAD));
   
   
   //Right Flap
   static UINT FRGrp1[4] = {184,185,186,187};
   
   static MGROUP_ROTATE flapsr (0,FRGrp1, 4, _V(4.942456,0.1215288,-3.412283), _V(1,0,-0.17), (float)(-45*RAD));
   
   anim_flaps = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_flaps, 0, 1, &flapsl);
   AddAnimationComponent  (anim_flaps, 0, 1, &flapsr);
   
   
   //Swingwing Left
   static UINT SWGrp1[6] = {177,178,179,180,181,182};
   
   static MGROUP_ROTATE swing (0,SWGrp1, 6, _V(-2.789116,0.1254393,-0.9770693), _V(0,1,0), (float)(-47*RAD));
   
   //Swingwing Right
   static UINT SWGrp2[6] = {183,184,185,186,187,191};
   
   static MGROUP_ROTATE swing2 (0,SWGrp2, 6, _V(2.789116,0.1254393,-0.9770693), _V(0,1,0), (float)(47*RAD));
   
   
   //Swingwing Left2
   static UINT SWGrp3[6] = {177,178,179,180,181,182};
   
   static MGROUP_ROTATE swing3(0,SWGrp3, 6, _V(-2.789116,0.1254393,-0.9770693), _V(1,0,0), (float)(-2.6*RAD));
   
   //Swingwing Right2
   static UINT SWGrp4[6] = {183,184,185,186,187,191};
   
   static MGROUP_ROTATE swing4 (0,SWGrp4, 6, _V(2.789116,0.1254393,-0.9770693), _V(1,0,0), (float)(-2.6*RAD));
   
   
   
   anim_swingwing = CreateAnimation (0);
   
   AddAnimationComponent  (anim_swingwing, 0, 1, &swing);
   AddAnimationComponent  (anim_swingwing, 0, 1, &swing2);
   AddAnimationComponent  (anim_swingwing, 0.35, 1, &swing3);
   AddAnimationComponent  (anim_swingwing, 0.35, 1, &swing4);
   
   
   
   // Altimeter Animations
   
   // Dial
   static UINT wWGrp0[1] = {1};
   
   static MGROUP_ROTATE wwheel0 (2,wWGrp0, 1, _V(0,0,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_wheel0 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel0, 0, 1, &wwheel0);
   
   // 1st dial from Left
   static UINT wWGrp1[10] = {23,24,25,26,27,28,29,30,31,32};
   
   static MGROUP_ROTATE wwheel1 (2,wWGrp1, 10, _V(0,7.199557e-03,0), _V(1,0,0), (float)(360*RAD));
   
   
   anim_wheel1 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel1, 0, 1, &wwheel1);
   
   // 2nd dial from Left
   static UINT wWGrp2[10] = {13,14,15,16,17,18,19,20,21,22};
   
   static MGROUP_ROTATE wwheel2 (2,wWGrp2, 10, _V(0,7.199557e-03,0), _V(1,0,0), (float)(360*RAD));
   
   
   anim_wheel2 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel2, 0, 1, &wwheel2);
   
   // 3rd dial from Left
   static UINT wWGrp3[10] = {3,4,5,6,7,8,9,10,11,12};
   
   static MGROUP_ROTATE wwheel3 (2,wWGrp3, 10, _V(0,7.199557e-03,0), _V(1,0,0), (float)(360*RAD));
   
   
   anim_wheel3 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel3, 0, 1, &wwheel3);
   
   // 4th dial from Left
   static UINT wWGrp4[10] = {43,44,45,46,47,48,49,50,51,52};
   
   static MGROUP_ROTATE wwheel4 (2,wWGrp4, 10, _V(0,7.199557e-03,0.785718e-03), _V(1,0,0), (float)(360*RAD));
   
   
   anim_wheel4 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel4, 0, 1, &wwheel4);
   
   // 5th dial from Left
   static UINT wWGrp5[10] = {33,34,35,36,37,38,39,40,41,42};
   
   static MGROUP_ROTATE wwheel5 (2,wWGrp5, 10, _V(0,7.199557e-03,0.785718e-03), _V(1,0,0), (float)(360*RAD));
   
   
   anim_wheel5 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel5, 0, 1, &wwheel5);
   
   
   
   // AOA Animations
   
   // Dial1
   static UINT dGrp0[1] = {1};
   
   static MGROUP_ROTATE dial1 (3,dGrp0, 1, _V(0,0,0), _V(0,0,1), (float)(311*RAD));
   
   
   anim_dial1 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_dial1, 0, 1, &dial1);
   

    // Direction Dial Animations
   
   // Dial2
   static UINT d2Grp0[1] = {0};
   
   static MGROUP_ROTATE dial2 (4,d2Grp0, 1, _V(0,0,0), _V(0,0,1), (float)(360*RAD));
   
   
   anim_dial2 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_dial2, 0, 1, &dial2);
   
   
   // Gear Lever Animation
   
   //Gear Lever in Cockpit
   static UINT Glev0[2] = {53,54};
   
   static MGROUP_ROTATE gearlev (1,Glev0, 2, _V(0,2.645784E-02,-3.477395E-03), _V(1,0,0), (float)(41*RAD));
   
   anim_gearlever = CreateAnimation (0);
   
   AddAnimationComponent  (anim_gearlever, 0, 1, &gearlev);
   
   
   // Power Guage Animation
   
   static UINT Pow0[1] = {70};
   
   static MGROUP_TRANSLATE powg (1,Pow0, 1, _V(0,0.05,0));
   
   anim_power = CreateAnimation (0);
   
   AddAnimationComponent  (anim_power, 0, 1, &powg);
   
   // Fuel Guage Animation
   
   static UINT Ful0[1] = {71};
   
   static MGROUP_TRANSLATE fulg (1,Ful0, 1, _V(0,-0.09,0));
   
   anim_fuel = CreateAnimation (0);
   
   AddAnimationComponent  (anim_fuel, 0, 1, &fulg);
   
   // Speed Dial Animations
   
   // Dial3
   static UINT d3Grp0[2] = {0,1};
   
   static MGROUP_ROTATE dial3 (5,d3Grp0, 2, _V(0,0,0), _V(0,0,1), (float)(360*RAD));
   
   
   anim_dial3 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_dial3, 0, 1, &dial3);
   
   
   // Bank Guage Animations
   
   static UINT BAGrp0[2] = {0,2};
   
   static MGROUP_ROTATE bank (6,BAGrp0, 2, _V(0,0,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_bank = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_bank, 0, 1, &bank);
   
   // Clock
   
   // Second Hand
   static UINT dGrp7[1] = {4};
   
   static MGROUP_ROTATE dial7 (7,dGrp7, 1, _V(0,0,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_clock_s = CreateAnimation (0);
   
   AddAnimationComponent  (anim_clock_s, 0, 1, &dial7);
   
   // Minute Hand
   static UINT dGrp8[1] = {3};
   
   static MGROUP_ROTATE dial8 (7,dGrp8, 1, _V(0,0,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_clock_m = CreateAnimation (0);
   
   AddAnimationComponent  (anim_clock_m, 0, 1, &dial8);
   
   // Hour Hand
   static UINT dGrp9[1] = {1};
   
   static MGROUP_ROTATE dial9 (7,dGrp9, 1, _V(0,0,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_clock_h = CreateAnimation (0);
   
   AddAnimationComponent  (anim_clock_h, 0, 1, &dial9);
   
   
}
	

bool ShuttlePB::clbkLoadVC (int id)
{
	
	SetCameraOffset (_V(0,0.7303072,4.200835));
	
    SetCameraDefaultDirection (_V(0,0,1));

    SetCameraRotationRange (RAD*88, RAD*88, RAD*48, RAD*48);
	
	static VCHUDSPEC hud_pilot  = {1, 78,{0,0.7,4.7},0.5};
	
    oapiVCRegisterHUD (&hud_pilot);

		
	
	
 return true;

}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new ShuttlePB (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (ShuttlePB*)vessel;
}
