// ==============================================================
//                 ORBITER MODULE: ShuttlePB
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// ShuttlePB.cpp
// Control module for ShuttlePB vessel class
//
// Notes:
// This is an example for a "minimal" vessel implementation which
// only overloads the clbkSetClassCaps method to define vessel
// capabilities and otherwise uses the default VESSEL class
// behaviour.
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#define ENG 				1	
#define GEAR				2	
#define BRAKE               3

#include <orbitersdk.h>
#include <cstdio>
#include <cmath>


//#include <string>
#include "OrbiterSoundSDK35.h"

//#include <stdio.h>
//#include <stdlib.h>

//using namespace std;


// ==============================================================
// Some vessel parameters
// ==============================================================
const double PB_FUELMASS = 20000.0;
const double PB_ISP = 50000.0;
const double PB_MAXMAINTH =0;

#define AID_ENGINEMAIN_TOP           4
#define AID_ENGINEMAIN_BOT           5
#define AID_GEAR                     6

// power buttons
#define AID_MFD1_PWR      7
#define AID_MFD2_PWR      8
#define AID_MFD3_PWR      9

// power buttons2
#define AID_MFD4_PWR      10
#define AID_MFD5_PWR      11
#define AID_MFD6_PWR      12

// Mode Select buttons
#define AID_MFD1_SEL      13
#define AID_MFD2_SEL      14
#define AID_MFD3_SEL      15

// Mode Select buttons2
#define AID_MFD4_SEL      16
#define AID_MFD5_SEL      17
#define AID_MFD6_SEL      18

// Menu buttons
#define AID_MFD1_MEN      19
#define AID_MFD2_MEN      20
#define AID_MFD3_MEN      21

// Menu buttons2
#define AID_MFD4_MEN      22
#define AID_MFD5_MEN      23
#define AID_MFD6_MEN      24

// MFD_LEFT buttons
#define AID_MFD1_BT1      25
#define AID_MFD1_BT2      26
#define AID_MFD1_BT3      27
#define AID_MFD1_BT4      28
#define AID_MFD1_BT5      29
#define AID_MFD1_BT6      30
#define AID_MFD1_BT7      31
#define AID_MFD1_BT8      32
#define AID_MFD1_BT9      33
#define AID_MFD1_BT10     34
#define AID_MFD1_BT11     35
#define AID_MFD1_BT12     36

// MFD_RIGHT buttons
#define AID_MFD2_BT1      37
#define AID_MFD2_BT2      38
#define AID_MFD2_BT3      39
#define AID_MFD2_BT4      40
#define AID_MFD2_BT5      41
#define AID_MFD2_BT6      42
#define AID_MFD2_BT7      43
#define AID_MFD2_BT8      44
#define AID_MFD2_BT9      45
#define AID_MFD2_BT10     46
#define AID_MFD2_BT11     47
#define AID_MFD2_BT12     48

// MFD_USER1 buttons
#define AID_MFD3_BT1      49
#define AID_MFD3_BT2      50
#define AID_MFD3_BT3      51
#define AID_MFD3_BT4      52
#define AID_MFD3_BT5      53
#define AID_MFD3_BT6      54
#define AID_MFD3_BT7      55
#define AID_MFD3_BT8      56
#define AID_MFD3_BT9      57
#define AID_MFD3_BT10     58
#define AID_MFD3_BT11     59
#define AID_MFD3_BT12     60

// MFD_USER2 buttons
#define AID_MFD4_BT1      61
#define AID_MFD4_BT2      62
#define AID_MFD4_BT3      63
#define AID_MFD4_BT4      64
#define AID_MFD4_BT5      65
#define AID_MFD4_BT6      67
#define AID_MFD4_BT7      68
#define AID_MFD4_BT8      69
#define AID_MFD4_BT9      70
#define AID_MFD4_BT10     71
#define AID_MFD4_BT11     72
#define AID_MFD4_BT12     73

double t0;

double tvec = 0;
double tvec2 = 0;

double mx1 = 0;
double iop = 0;
double st = 0;
double pld = 0;
double cen = 0;
double gear_proc = 0;
double gear_proc2 = 0;
double gear_procl = 0;
double power_proc = 0;
double fuel_proc = 0;
double speedbrake = 0;
double swingwing = 0;
double swing = 0;
double swing3 = 0;
double hook = 0;
double canopy = 0;
double flaps = 0;
double flaps2 = 0;
double exproc = 0;
double Lights = 0;
double switched = 0;
double aoa;
double dil = 0;
double eng_pitch = 22000;

double start = 0;
double full = 0.001;
double i = 0;
double i1 = 0;
double i2 = 0;
double i3 = 0;
double i4 = 0;
double i5 = 0;
double i7 = 0;
double i9 = 0;
double i1a = 0;
double i2a = 0;
double i3a = 0;
double i4a = 0;
double i5a = 0;
double s1 = 0;
double s2 = 0;
double s3 = 0;
double s4 = 0;
double s5 = 0;
double ga = 0;
double diff = 0.1;
double k3;
int k4;
char k3buffer [33];
double amet = 0;
double dee = 0;
double i13 = 0;
double i14 = 0;
double i15 = 0;
double i16 = 0;
double i17 = 0;
double i18 = 0;
double i19 = 0;
double i20 = 0;
double i21 = 0;
double droptank = 0;

double secondtime = 0;

double refuel = 0;

UINT animactive;

UINT animactivel;

UINT animactive2;

UINT animactive3;

UINT animactive4;

UINT animactive5;

UINT animactive7;

UINT animactive9;

const VECTOR3 offset = {0,0.2,4.5};


THRUSTER_HANDLE th_main;

BEACONLIGHTSPEC beacon1;    

BEACONLIGHTSPEC beacon2;   

MESHHANDLE f150;
MESHHANDLE f151;
MESHHANDLE f152;




// 1. vertical lift component (wings and body)

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	static const double CL[nabsc] = {0.1, 0.17, 0.2, 0.2, 0.17, 0.1, 0, -0.11, -0.24, -0.38,  -0.5,  -0.5, -0.02, 0.6355,    0.63,   0.46, 0.28, 0.13, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,    0,0.002,0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = CM[idx] + (CM[idx+1]-CM[idx])*d;
	*cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.02 + oapiGetInducedDrag (*cl, 1.5, 0.6);
}


// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class ShuttlePB: public VESSEL2 {
public:
	ShuttlePB (OBJHANDLE hVessel, int flightmodel);
	//	: VESSEL2 (hVessel, flightmodel) {}
	void clbkSetClassCaps (FILEHANDLE cfg);
	void DefineAnimations(void);
	void Steering();
	void Lighting();
	void clbkPostCreation (void);
    void clbkPostStep (double simt, double simdt, double mjd);
    int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
   
    UINT anim_wheel;
    UINT anim_gear;
    UINT anim_gearlever;
    UINT anim_rudder;
    UINT anim_elevator;
    UINT anim_laileron;
    UINT anim_raileron;
    UINT anim_speedbrake;
    
    UINT anim_flaps;
    UINT anim_hook;
    
    UINT anim_power;
    UINT anim_dial44;
    
    UINT anim_wheel0;
    UINT anim_wheel1;
    UINT anim_wheel2;
    UINT anim_wheel3;
    UINT anim_wheel4;
    UINT anim_wheel5;
    
    UINT anim_dial1;
    UINT anim_dial2;
    UINT anim_dial3;
    UINT anim_bank;
    UINT anim_clock_s;
    UINT anim_clock_m;
    UINT anim_clock_h;
    UINT anim_tveca;
    
    bool clbkVCMouseEvent (int id, int event, VECTOR3 &p);

    bool clbkLoadVC (int id);
    
    int MySoundID;	


};

ShuttlePB::ShuttlePB (OBJHANDLE hVessel, int flightmodel)
		: VESSEL2 (hVessel, flightmodel)
{
	
	// preload meshes
	
    f150                 = oapiLoadMeshGlobal ("K33\\K-FA-22A");
    f151                 = oapiLoadMeshGlobal ("K33\\K-FA-22A-Cpit");
    f152                 = oapiLoadMeshGlobal ("K33\\K-FA-22A-Cpit_n");
   

	
	DefineAnimations ();
	
}


void ShuttlePB::clbkPostCreation (void)
{

int MySoundID=ConnectToOrbiterSoundDLL3(GetHandle());

//ReplaceStockSound3(MySoundID,"Sound\\Vessel\\K-Test-Car\\K-Test-Car.wav",	REPLACE_MAIN_THRUST);

RequestLoadVesselWave3(MySoundID,ENG,"Sound\\Vessel\\K-FA-22A\\K-FA-22A-Eng.wav",BOTHVIEW_FADED_FAR);

RequestLoadVesselWave3(MySoundID,BRAKE,"Sound\\Vessel\\K-FA-22A\\K-FA-22A-Brake.wav",BOTHVIEW_FADED_FAR);

RequestLoadVesselWave3(MySoundID,GEAR,"Sound\\Vessel\\K-FA-22A\\K-FA-22A-Gear.wav",BOTHVIEW_FADED_FAR);


SoundOptionOnOff3(MySoundID,PLAYMAINTHRUST,FALSE);

SoundOptionOnOff3(MySoundID,PLAYHOVERTHRUST,FALSE);


SoundOptionOnOff3(MySoundID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);

SoundOptionOnOff3(MySoundID,PLAYCABINAIRCONDITIONING,FALSE);

SoundOptionOnOff3(MySoundID,PLAYCABINRANDOMAMBIANCE,FALSE);

SoundOptionOnOff3(MySoundID,PLAYRADIOATC,FALSE);

SoundOptionOnOff3(MySoundID,DISPLAYTIMER,FALSE);

}


// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------

void ShuttlePB::clbkSetClassCaps (FILEHANDLE cfg)
{
	// THRUSTER_HANDLE th_main;


	// physical specs
	SetSize (8);
	SetEmptyMass (12000.0);
	SetCW (0.3, 0.3, 0.6, 0.9);
	SetWingAspect (1.7);
	SetWingEffectiveness (3.1);
	SetCrossSections (_V(5,5,15));
	SetRotDrag (_V(3.5,3.5,3.5));
	if (GetFlightModel() >= 1) {
		SetPitchMomentScale (1e-4);
		SetBankMomentScale (1e-4);
	}
	SetPMI (_V(15.5,22.1,7.7));
	SetTrimScale (0.05);
	SetCameraOffset (_V(0,0.433,4.1));
	
	SetTouchdownPoints (_V(0,-2,9), _V(-4,-2,-5), _V(4,-2,-5));

	// **************************** aerodynamics ***********************************

	CreateAirfoil (LIFT_VERTICAL,   _V(0,0,-0.5), VLiftCoeff, 20, 270, 2.266);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,-4), HLiftCoeff, 20,  50, 1.5);





	CreateControlSurface (AIRCTRL_ELEVATOR,     7, 0.7, _V(   0,0,-17), AIRCTRL_AXIS_XPOS,anim_elevator);
	CreateControlSurface (AIRCTRL_RUDDER,       2, 0.5, _V(   0,2,-15), AIRCTRL_AXIS_YPOS,anim_rudder);
	CreateControlSurface (AIRCTRL_AILERON,      4.8, 0.2, _V( 17.8,0,-17.8), AIRCTRL_AXIS_XPOS,anim_raileron);
	CreateControlSurface (AIRCTRL_AILERON,      4.8, 0.2, _V(-17.8,0,-17.8), AIRCTRL_AXIS_XNEG,anim_laileron);
	CreateControlSurface (AIRCTRL_ELEVATORTRIM, 5, 0.1, _V(   0,0,-15), AIRCTRL_AXIS_XPOS,anim_elevator);
    CreateControlSurface (AIRCTRL_FLAP,         5, 5, _V( 0,0,-1), AIRCTRL_AXIS_XPOS,anim_flaps);
	
	CreateVariableDragElement( &speedbrake, 8.0, _V(0,0,0) );

	CreateVariableDragElement (&gear_proc2, 0.8, _V(0, -1, 0));
	
	CreateVariableDragElement (&hook, 3.33, _V(0,1,0));


	
	// propellant resources
	PROPELLANT_HANDLE hpr = CreatePropellantResource (PB_FUELMASS);
	
	
	SURFHANDLE tex_main = oapiRegisterExhaustTexture ("K-FA-22A-Exhaust");



	// ***************** thruster definitions *******************

	PARTICLESTREAMSPEC contrail_main = {
		0, 0.3, 165, 19, 0.019, 0.111, 1.11, 1.11, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 2,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};
	
	th_main = CreateThruster (_V(0,0,-4.3), _V(0,0,1), PB_MAXMAINTH, hpr, PB_ISP);
	
	
	
	CreateThrusterGroup (&th_main, 1, THGROUP_MAIN);
	
	
	
	AddExhaust (th_main, 5, 0.5, _V(-0.555169,-0.4163767,-4.3), _V(0,0,-1) , tex_main);
	AddExhaust (th_main, 5, 0.5, _V(0.555169,-0.4163767,-4.3), _V(0,0,-1) , tex_main);

    
    AddExhaustStream (th_main, _V(-0.555169,-0.4163767,-5.6), &contrail_main);
    AddExhaustStream (th_main, _V(0.555169,-0.4163767,-5.6), &contrail_main);
    
    
      // Strobe lights
	
	//Blue strobe
	
	static VECTOR3 beacon1pos = {5,-0.48,-3.25};
	static VECTOR3 beacon1col = {0.043, 0.161, 0.992};
	
		beacon1.shape = (BEACONSHAPE_STAR);
		beacon1.pos = &beacon1pos;
		beacon1.col = &beacon1col;
		beacon1.size = (0.18);
		beacon1.falloff = (0.6);
		beacon1.period = (1.13);
		beacon1.duration = (0.05);
		beacon1.tofs = (0.6);
		beacon1.active = false;
		
		AddBeacon (&beacon1);
		
		//Red strobe
	
	static VECTOR3 beacon2pos = {-5,-0.48,-3.25};
	static VECTOR3 beacon2col = {0.988, 0.133, 0.133};
	
		beacon2.shape = (BEACONSHAPE_STAR);
		beacon2.pos = &beacon2pos;
		beacon2.col = &beacon2col;
		beacon2.size = (0.18);
		beacon2.falloff = (0.6);
		beacon2.period = (1.13);
		beacon2.duration = (0.05);
		beacon2.tofs = (0.6);
		beacon2.active = false;
		
		AddBeacon (&beacon2);
		

    

    
    
    
	// visual specs
	
	AddMesh (f150);
	
	SetMeshVisibilityMode( AddMesh (f151,&offset), MESHVIS_VC );

    

}

// ==============================================================
// API callback interface
// ==============================================================

void ShuttlePB::clbkPostStep (double simt, double simdt, double mjd)

{
	
	if( oapiGetTimeAcceleration() > 10.0 ) oapiSetTimeAcceleration(10);
	
	double vsAirSpeed = GetAirspeed();
	
	// Thrust Vectoring
	
	tvec = (GetControlSurfaceLevel(AIRCTRL_ELEVATOR)+1 )* 0.5;
	tvec2 = GetControlSurfaceLevel(AIRCTRL_ELEVATOR)* 0.1;
	
	
	SetThrusterDir (th_main ,_V(0,-tvec2,1));

	SetAnimation( anim_tveca, tvec );
	
	//sprintf(oapiDebugString(),"anim %5.5f", -tvec2);

	
	
    //speed and altitude control
	
	if (GetAirspeed() >= 700) 
	{
		SetCW (999, 999, 999, 999);
		SetThrusterMax0 (th_main ,0);

	
}

if (GetAltitude() >= 17000) 
	{
		SetCW (999, 999, 999, 999);
		SetThrusterMax0 (th_main ,0);

	
}



if (GetAirspeed() < 700 && GetAltitude() < 17000 ) 
	{
		SetCW (0.3, 0.3, 0.67, 0.92);
		SetThrusterMax0 (th_main ,4.4e5);

	
}

	double vsAirSpeed2 = (vsAirSpeed/1000)*3600;
    
    //Ground Steering
    
    if (GroundContact () ) Steering();
    
    
    //aoa = GetAOA();
    
    //sprintf(oapiDebugString(),"anim %2.2f", aoa );
    
  

    //Anims 
    
    if (iop > 1)iop=0;
    
    if (vsAirSpeed > 0) {
    
	double aspeed = vsAirSpeed;
    if (iop > 1)iop=0;
    if (vsAirSpeed > 30) aspeed = 30; 
    iop = iop + (aspeed /300);
     
    if (vsAirSpeed ==0) iop=0;
    
    
    if ( GroundContact() && gear_proc < 0.05) SetAnimation (anim_wheel, iop);
    

   }
	
	
// Gear animations


   if( animactive && !GroundContact()) {
		if( animactive == 1 ) {
			gear_proc += 0.00242;
			if( gear_proc > 1.0 ) {
				gear_proc = 1.0;
				animactive = 0;
			}
		}
		else  {
			gear_proc -= 0.00242;
			if( gear_proc < 0.0 ) {
				gear_proc = 0.0;
				animactive = 0;
			}
		}
		SetAnimation( anim_gear, gear_proc );
		gear_proc2 = -gear_proc + 1;
	}

	
	// Gear Lever animations


   if( animactivel && !GroundContact()) {
		if( animactivel == 1 ) {
			gear_procl += 0.00242;
			if( gear_procl > 1.0 ) {
				gear_procl = 1.0;
				animactivel = 0;
			}
		}
		else  {
			gear_procl -= 0.00242;
			if( gear_procl < 0.0 ) {
				gear_procl = 0.0;
				animactivel = 0;
			}
		}
		SetAnimation( anim_gearlever, gear_procl );
	}

	
	// Throttle animation

        power_proc = GetThrusterLevel(th_main);
	
		SetAnimation( anim_power, power_proc );
		
	
   
	
	

   // Speedbrake animations

   if( animactive2 ) {
		if( animactive2 == 1 ) {
			speedbrake += 0.006;
			if( speedbrake > 1.0 ) {
				speedbrake = 1.0;
				animactive2 = 0;
			}
		}
		else  {
			speedbrake -= 0.006;
			if( speedbrake < 0.0 ) {
				speedbrake = 0.0;
				animactive2 = 0;
			}
		}
		SetAnimation( anim_speedbrake, speedbrake );
	}

	
	
	// Canopy Animations


   if( animactive9 ) {
		if( animactive9 == 1 ) {
			hook += 0.003;
			if( hook > 1.0 ) {
				hook = 1.0;
				animactive9 = 0;
			}
		}
		else  {
			hook -= 0.003;
			if( hook < 0.0 ) {
				hook = 0.0;
				animactive9 = 0;
			}
		}
		SetAnimation( anim_hook, hook );
	}
	
	
   // Flaps


   if( animactive4 ) {
		if( animactive4 == 1 ) {
			flaps += 0.006;
			if( flaps > 1.0 ) {
				flaps = 1.0;
				animactive4 = 0;
			}
		}
		else  {
			flaps -= 0.006;
			if( flaps < 0.0 ) {
				flaps = 0.0;
				animactive4 = 0;
			}
		}
		SetAnimation( anim_flaps, flaps );
	}

   SetControlSurfaceLevel(AIRCTRL_FLAP,flaps);

 

// Engine sound
	
	
	
	
	if (GetThrusterLevel (th_main) > 0) 
	{
		
	double pit = ((GetThrusterLevel (th_main)*6000));
	
	eng_pitch = 13000 + pit;
	
	if (eng_pitch < 11500 ) eng_pitch = 11500;
	
	if (eng_pitch > 22500 ) eng_pitch = 22500;
	
	//sprintf(oapiDebugString(),"anim %7.7f", eng_pitch );
		
	PlayVesselWave3(MySoundID,ENG,LOOP,255,eng_pitch);
}


    if (GetThrusterLevel (th_main) == 0) 
	{
	
	StopVesselWave3(MySoundID,ENG);
}
   
 if (Lights == 1 && switched == 1)
 {
	 beacon1.active = true;
	 beacon2.active = true;

	 switched = 0;
	 Lighting();
}

if (Lights == 0 && switched == 1)
{
	beacon1.active = false;
	beacon2.active = false;

    switched = 0;
    Lighting();
}


   // Refuel
   
   if (refuel == 1)
   {
	   
	  SetFuelMass (20000.0);

	   refuel = 0;
   }
 

}

//Ground Steering


void ShuttlePB::Steering()
 {

	double pos = GetControlSurfaceLevel(AIRCTRL_RUDDER);
	double lev = 1e6 * pos * GetAirspeed() / 10.0;

	if( oapiGetTimeAcceleration() > 10.0 )
		return;

	AddForce( _V( lev,0,0 ), _V( 0,0,0.33 ) );
	AddForce( _V( -1.0*lev,0,0 ), _V( 0,0,-0.33 ) );
	
}

// Lighting


void ShuttlePB::Lighting()
 {
    if(Lights == 1)
    {
	
	SetMeshVisibilityMode( InsertMesh (f152,1,&offset), MESHVIS_VC );
	

}

    if(Lights == 0)
    {
	
	SetMeshVisibilityMode( InsertMesh (f151,1,&offset), MESHVIS_VC );
	

}

}


// --------------------------------------------------------------
// Keyboard interface handler (buffered key events)
// --------------------------------------------------------------
int ShuttlePB::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate)
{
	if (!down) return 0; // only process keydown events

	// if (KEYMOD_SHIFT (kstate)) {

		switch (key) {
		case OAPI_KEY_G: // Gear
		
		if( gear_proc > 0.5 )
					animactive = 2;
				else
					animactive = 1;

		if ( !GroundContact()) PlayVesselWave3(MySoundID,GEAR,NOLOOP,255);
		
		
		if( gear_procl > 0.5 )
					animactivel = 2;
				else
					animactivel = 1;

			
			return 1;
			
			
			case OAPI_KEY_B: // Speedbrake
		
		if( speedbrake > 0.5 )
					animactive2 = 2;
				else
					animactive2 = 1;

		PlayVesselWave3(MySoundID,BRAKE,NOLOOP,255);
		
			
			return 1;
			
			case OAPI_KEY_9: // Canopy
		
		if( hook > 0.5 )
					animactive9 = 2;
				else
					animactive9 = 1;

		PlayVesselWave3(MySoundID,BRAKE,NOLOOP,255);
		
			
			return 1;
			
			
			
			case OAPI_KEY_F: // Flaps
			
			
		if( flaps > 0.5 )
					animactive4 = 2;
				else
					animactive4 = 1;

		PlayVesselWave3(MySoundID,BRAKE,NOLOOP,255);
		
			
			return 1;
			
			
			case OAPI_KEY_J: // Refuel
			
			
		if (refuel == 0 && GetAirspeed() == 0)
		{
			refuel = 1;
		}
		
			
			return 1;
		
		

			case OAPI_KEY_K: // Lights
		
		if( Lights == 0 )
		{
			Lights = 1;
			switched =1;
					
			return 1;
}

		if( Lights == 1 )
		{
			Lights = 0;
			switched = 1;
					
			return 1;
}
	
    } 
	return 0;
}


void ShuttlePB::DefineAnimations(void)
{
   // Front Wheel
   static UINT WGrp1[1] = {19};
   
   static MGROUP_ROTATE wheel1 (0,WGrp1, 1, _V(0,-1.748517,3.173483), _V(1,0,0), (float)(360*RAD));
   
   //Rear left Wheel
   static UINT WGrp3[1] = {20};
   
   static MGROUP_ROTATE wheel2 (0,WGrp3, 1, _V(0,-1.613605,-1.71995), _V(1,0,0), (float)(360*RAD));
   
   //Rear Right wheel
   static UINT WGrp4[1] = {21};
   
   static MGROUP_ROTATE wheel3 (0,WGrp4, 1, _V(0,-1.613605,-1.71995), _V(1,0,0), (float)(360*RAD));
   
   anim_wheel = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel1);
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel2);
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel3);
   
   
   
   
   // Gear anims
  
   
   //Front Gear
   static UINT GGrp1[6] = {5,6,7,8,9,19};
   
   static MGROUP_ROTATE gear1 (0,GGrp1, 6, _V(0,-0.9840549,3.175663), _V(1,0,0), (float)(-123*RAD));
   
   
   //Rear Gear Left
   static UINT GGrp2[1] = {149};
   
   static MGROUP_ROTATE gear2 (0,GGrp2, 1, _V(-0.8897326,-0.7818472,-1.712807), _V(0,0,1), (float)(-110*RAD));
   
   //Rear Gear Right
   static UINT GGrp3[1] = {150};
   
   static MGROUP_ROTATE gear3 (0,GGrp3, 1, _V(0.8897326,-0.7818472,-1.712807), _V(0,0,1), (float)(110*RAD));
   
   //Front Gear Cover Left
   static UINT GGrp4[1] = {61};
   
   static MGROUP_ROTATE gear4 (0,GGrp4, 1, _V(-0.1476224,-0.978443,3.475985), _V(0,0,1), (float)(90*RAD));
   
   //Front Gear Cover Right
   static UINT GGrp5[1] = {62};
   
   static MGROUP_ROTATE gear5 (0,GGrp5, 1, _V(0.1476224,-0.978443,3.475985), _V(0,0,1), (float)(-90*RAD));
   
   //Rear Gear Left2
   static UINT GGrp8[4] = {20,146,147,148};
   
   static MGROUP_ROTATE gear8 (0,GGrp8, 4, _V(4,-1,0), _V(0,0,1), (float)(-10*RAD));
   
   //Rear Gear Right2
   static UINT GGrp9[4] = {21,151,152,153};
   
   static MGROUP_ROTATE gear9 (0,GGrp9, 4, _V(-4,-1,0), _V(0,0,1), (float)(10*RAD));
   
   
   //Rear Cover Left1
   static UINT GGrp12[1] = {108};
   
   static MGROUP_ROTATE gear12 (0,GGrp12, 1, _V(-0.7931949,-0.8982614,-1.546714), _V(0,0,1), (float)(-111*RAD));
   
   //Rear Cover Right1
   static UINT GGrp13[1] = {109};
   
   static MGROUP_ROTATE gear13 (0,GGrp13, 1, _V(0.7931949,-0.8982614,-1.546714), _V(0,0,1), (float)(111*RAD));
   
   //Rear Cover Left2
   static UINT GGrp14[1] = {110};
   
   static MGROUP_ROTATE gear14 (0,GGrp14, 1, _V(-1.41225,-0.517578,-1.710372), _V(0,0,1), (float)(90*RAD));
   
   //Rear Cover Right2
   static UINT GGrp15[1] = {111};
   
   static MGROUP_ROTATE gear15 (0,GGrp15, 1, _V(1.41225,-0.517578,-1.710372), _V(0,0,1), (float)(-90*RAD));
   
   //Rear Gear Left3
   static UINT GGrp10[1] = {20};
   
   static MGROUP_ROTATE gear10 (0,GGrp10, 1, _V(-1.188635,-0.5816181,0), _V(0,0,1), (float)(60*RAD));
   
   //Rear Gear Right3
   static UINT GGrp11[1] = {21};
   
   static MGROUP_ROTATE gear11 (0,GGrp11, 1, _V(1.188635,-0.5816181,0), _V(0,0,1), (float)(-60*RAD));
   
   
   anim_gear = CreateAnimation (0);
   
   AddAnimationComponent  (anim_gear, 0, 0.5, &gear1);
   AddAnimationComponent  (anim_gear, 0, 0.5, &gear2);
   AddAnimationComponent  (anim_gear, 0, 0.5, &gear3);
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear4);
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear5);
   
   AddAnimationComponent  (anim_gear, 0, 0.5, &gear8);
   AddAnimationComponent  (anim_gear, 0, 0.5, &gear9);
   
   AddAnimationComponent  (anim_gear, 0.5, 0.7, &gear10);
   AddAnimationComponent  (anim_gear, 0.5, 0.7, &gear11);
   
   AddAnimationComponent  (anim_gear, 0.7, 1, &gear12);
   AddAnimationComponent  (anim_gear, 0.7, 1, &gear13);
   AddAnimationComponent  (anim_gear, 0.7, 1, &gear14);
   AddAnimationComponent  (anim_gear, 0.7, 1, &gear15);
   
   
   
   //Rudder left
   static UINT RGrp1[2] = {88,89};
   
   static MGROUP_ROTATE rudderl (0,RGrp1, 2, _V(-1.769169,1.083998,-4.710566), _V(-0.4,1,0.3), (float)(-20*RAD));
   
   //Rudder Pedals left
   static UINT RGrp3[2] = {162,163};
   
   static MGROUP_ROTATE rudderlp (1,RGrp3, 2, _V(0,0.34,0.4480002), _V(1,0,0), (float)(6*RAD));
   
   
   //Rudder right
   static UINT RGrp2[2] = {90,91};
   
   static MGROUP_ROTATE rudderr (0,RGrp2, 2, _V(1.769169,1.083998,-4.710566), _V(0.4,1,0.3), (float)(-20*RAD));
   
   
   //Rudder Padals right
   static UINT RGrp4[2] = {164,165};
   
   static MGROUP_ROTATE rudderrp (1,RGrp4, 2, _V(0,0.34,0.4480002), _V(1,0,0), (float)(-6*RAD));
   
   
   
   anim_rudder = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderl);
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderr);
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderlp);
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderrp);
   
   
   //Elevator
   static UINT EGrp1[2] = {121,145};
   
   static MGROUP_ROTATE elevator (0,EGrp1, 2, _V(0,-0.3891779,-5.255791), _V(1,0,0), (float)(25*RAD));
   
   
   anim_elevator = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_elevator, 0, 1, &elevator);
   
   
   
   
   //Left Aleron
   static UINT LAGrp1[1] = {132};
   
   static MGROUP_ROTATE laileron (0,LAGrp1, 1, _V(-4.036558,-0.4652892,-3.758886), _V(1,0,-0.1), (float)(-45*RAD));
   
   
   anim_laileron = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_laileron, 0, 1, &laileron);
   
   
   //Right Aleron
   static UINT RAGrp1[1] = {142};
   
   static MGROUP_ROTATE raileron (0,RAGrp1, 1, _V(4.036558,-0.4652892,-3.758886), _V(1,0,0.1), (float)(45*RAD));
   
   
   anim_raileron = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_raileron, 0, 1, &raileron);
   
   
   //Speedbrake
   static UINT SBGrp1[2] = {88,89};
   
   static MGROUP_ROTATE speedbrake1 (0,SBGrp1, 2, _V(-1.769169,1.083998,-4.710566), _V(-0.4,1,0.3), (float)(25*RAD));
   
   static UINT SBGrp2[2] = {90,91};
   
   static MGROUP_ROTATE speedbrake2 (0,SBGrp2, 2, _V(1.769169,1.083998,-4.710566), _V(0.4,1,0.3), (float)(-25*RAD));
   
   
   anim_speedbrake = CreateAnimation (0);
   
   AddAnimationComponent  (anim_speedbrake, 0, 1, &speedbrake1);
   AddAnimationComponent  (anim_speedbrake, 0, 1, &speedbrake2);
   
  
   //Canopy
   static UINT HGrp1[3] = {0,1,172};
   
   static MGROUP_ROTATE hook (0,HGrp1, 3, _V(0,0.6331617,3.178498), _V(1,0,0), (float)(-40*RAD));
   
   static UINT HGrp2[2] = {76,77};
   
   static MGROUP_ROTATE hook2 (1,HGrp2, 2, _V(0,0.5193866,-1.254425), _V(1,0,0), (float)(-40*RAD));
   
   
   anim_hook = CreateAnimation (0);
   
   AddAnimationComponent  (anim_hook, 0, 1, &hook);
   AddAnimationComponent  (anim_hook, 0, 1, &hook2);
   
   //Thrust Vectoring Animation
   static UINT TVGrp1[2] = {98,114};
   
   static MGROUP_ROTATE tveca1 (0,TVGrp1, 2, _V(0,-0.1709794,-5.228415), _V(1,0,0), (float)(15*RAD));
   
   static UINT TVGrp2[2] = {99,115};
   
   static MGROUP_ROTATE tveca2 (0,TVGrp2, 2, _V(0,-0.6251161,-5.228415), _V(1,0,0), (float)(15*RAD));
   
   
   anim_tveca = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_tveca, 0, 1, &tveca1);
   AddAnimationComponent  (anim_tveca, 0, 1, &tveca2);
   
   //Flaps animation
   
   //Left Flap
   static UINT FLGrp1[2] = {135,136};
   
   static MGROUP_ROTATE flapsl (0,FLGrp1, 2, _V(-2.915093,-0.4297467,-3.703028), _V(1,0,-0.1), (float)(-45*RAD));
   
   
   //Right Flap
   static UINT FRGrp1[2] = {143,144};
   
   static MGROUP_ROTATE flapsr (0,FRGrp1, 2, _V(2.915093,-0.4297467,-3.703028), _V(1,0,0.1), (float)(-45*RAD));
   
   anim_flaps = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_flaps, 0, 1, &flapsl);
   AddAnimationComponent  (anim_flaps, 0, 1, &flapsr);
   
   
   // Gear Lever Animation
   
   //Gear Lever in Cockpit
   static UINT Glev0[2] = {110,114};
   
   static MGROUP_TRANSLATE gearlev (1,Glev0, 2,  _V(0,0.052,0.033));
   
   static UINT Glev2[3] = {166,167,168};
   
   static MGROUP_TRANSLATE gearlev2 (1,Glev2, 3,  _V(0.002,0.003,-0.0075));
   
   anim_gearlever = CreateAnimation (0);
   
   AddAnimationComponent  (anim_gearlever, 0, 1, &gearlev);
   AddAnimationComponent  (anim_gearlever, 0.9, 1, &gearlev2);
   
   
   // Throttle Animation
   
   static UINT Pow0[2] = {173,174};
   
   static MGROUP_TRANSLATE powg (1,Pow0, 2, _V(0,0,0.07));
   
   anim_power = CreateAnimation (0);
   
   AddAnimationComponent  (anim_power, 0, 1, &powg);
   
   
   
}
	
bool ShuttlePB::clbkVCMouseEvent (int id, int event, VECTOR3 &p)
{
	


	switch (id) {
	// panel 0 events:

	case AID_ENGINEMAIN_TOP:
	{
		//if (event & PANEL_MOUSE_LBDOWN) { // record which slider to operate
		if (GetThrusterLevel(th_main) == 0) mx1 = 0;
		mx1 = mx1 + 0.0055 ;
		//sprintf(oapiDebugString(),"mouse %19.19f", mx1 );
		if (mx1 < 0)mx1=0;
		if(mx1 >1)mx1=1;
			SetThrusterLevel(th_main ,mx1);
		
		
		return 1;
	}
	
	case AID_ENGINEMAIN_BOT:
	{
		//if (event & PANEL_MOUSE_LBDOWN) { // record which slider to operate
		if (GetThrusterLevel(th_main) == 0) mx1 = 0;
		mx1 = mx1 - 0.0055 ;
		//sprintf(oapiDebugString(),"mouse %19.19f", mx1 );
		if (mx1 < 0)mx1=0;
		if(mx1 >1)mx1=1;
			SetThrusterLevel(th_main ,mx1);
		
		
		return 1;
	}
	
	case AID_GEAR:
	{
		//if (event & PANEL_MOUSE_LBDOWN) { // record which slider to operate
	if( gear_proc > 0.5 )
					animactive = 2;
				else
					animactive = 1;

		if ( !GroundContact()) PlayVesselWave3(MySoundID,GEAR,NOLOOP,255);
		
		
		if( gear_procl > 0.5 )
					animactivel = 2;
				else
					animactivel = 1;
					
					return 1;
	}
					

    // power buttons
    
    case AID_MFD1_PWR:
     {
        
        oapiSendMFDKey(MFD_LEFT, OAPI_KEY_ESCAPE);
        
        return 1;
        } 
        
    case AID_MFD2_PWR:
     {
        
        oapiSendMFDKey(MFD_RIGHT, OAPI_KEY_ESCAPE);
        
        return 1;
        } 
        
    case AID_MFD3_PWR:
     {
        
        oapiSendMFDKey(MFD_USER1, OAPI_KEY_ESCAPE);
        
        return 1;
        } 
        
    case AID_MFD4_PWR:
     {
        
        oapiSendMFDKey(MFD_USER2, OAPI_KEY_ESCAPE);
        
        return 1;
        } 
        
    case AID_MFD5_PWR:
     {
        
        oapiSendMFDKey(MFD_USER3, OAPI_KEY_ESCAPE);
        
        return 1;
        } 
        
    case AID_MFD6_PWR:
    {
        
        oapiSendMFDKey(MFD_USER4, OAPI_KEY_ESCAPE);
        
        return 1;
        } 
        
        
        
        // Mode Select buttons
    
    case AID_MFD1_SEL:
     {
        
        oapiSendMFDKey(MFD_LEFT, OAPI_KEY_F1);
        
        return 1;
        } 
        
    case AID_MFD2_SEL:
     {
        
        oapiSendMFDKey(MFD_RIGHT, OAPI_KEY_F1);
        
        return 1;
        } 
        
    case AID_MFD3_SEL:
     {

        oapiSendMFDKey(MFD_USER1, OAPI_KEY_F1);
        
        return 1;
        } 

    case AID_MFD4_SEL:
     {
        
        oapiSendMFDKey(MFD_USER2, OAPI_KEY_F1);
        
        return 1;
        } 
        
    case AID_MFD5_SEL:
     {
        
        oapiSendMFDKey(MFD_USER3, OAPI_KEY_F1);
        
        return 1;
        } 
        
    case AID_MFD6_SEL:
    {

        oapiSendMFDKey(MFD_USER4, OAPI_KEY_F1);
        
        return 1;
        } 
        
        
        // Menu buttons
    
    case AID_MFD1_MEN:
     {
        
        oapiSendMFDKey(MFD_LEFT, OAPI_KEY_GRAVE);
        
        return 1;
        } 
        
    case AID_MFD2_MEN:
     {
        
        oapiSendMFDKey(MFD_RIGHT, OAPI_KEY_GRAVE);
        
        return 1;
        } 
        
    case AID_MFD3_MEN:
    {

        oapiSendMFDKey(MFD_USER1, OAPI_KEY_GRAVE);
        
        return 1;
        } 

   case AID_MFD4_MEN:
     {
        
        oapiSendMFDKey(MFD_USER2, OAPI_KEY_GRAVE);
        
        return 1;
        } 
        
    case AID_MFD5_MEN:
     {
        
        oapiSendMFDKey(MFD_USER3, OAPI_KEY_GRAVE);
        
        return 1;
        } 
        
    case AID_MFD6_MEN:
    {

        oapiSendMFDKey(MFD_USER4, OAPI_KEY_GRAVE);
        
        return 1;
        }      
        
        
         // MFD_LEFT buttons
    
    case AID_MFD1_BT1:
     {
        
        oapiProcessMFDButton (MFD_LEFT, 0, event);

        
        return 1;
        } 
        
    case AID_MFD1_BT2:
     {
        
       oapiProcessMFDButton (MFD_LEFT, 1, event);

        
        return 1;
        } 
        
    case AID_MFD1_BT3:
    {

        oapiProcessMFDButton (MFD_LEFT, 2, event);

        
        return 1;
        } 
        
	 case AID_MFD1_BT4:
     {
        
        oapiProcessMFDButton (MFD_LEFT, 3, event);

        
        return 1;
        } 
        
    case AID_MFD1_BT5:
     {
        
       oapiProcessMFDButton (MFD_LEFT, 4, event);

        
        return 1;
        } 
        
    case AID_MFD1_BT6:
    {

        oapiProcessMFDButton (MFD_LEFT, 5, event);

        
        return 1;
        } 
        
        case AID_MFD1_BT7:
     {
        
        oapiProcessMFDButton (MFD_LEFT, 6, event);

        
        return 1;
        } 
        
    case AID_MFD1_BT8:
     {
        
       oapiProcessMFDButton (MFD_LEFT, 7, event);

        
        return 1;
        } 
        
    case AID_MFD1_BT9:
    {

        oapiProcessMFDButton (MFD_LEFT, 8, event);

        
        return 1;
        } 
        
	 case AID_MFD1_BT10:
     {
        
        oapiProcessMFDButton (MFD_LEFT, 9, event);

        
        return 1;
        } 
        
    case AID_MFD1_BT11:
     {
        
       oapiProcessMFDButton (MFD_LEFT, 10, event);

        
        return 1;
        } 
        
    case AID_MFD1_BT12:
    {

        oapiProcessMFDButton (MFD_LEFT, 11, event);

        
        return 1;
        } 
	
         // MFD_RIGHT buttons
    
    case AID_MFD2_BT1:
     {
        
        oapiProcessMFDButton (MFD_RIGHT, 0, event);

        
        return 1;
        } 
        
    case AID_MFD2_BT2:
     {
        
       oapiProcessMFDButton (MFD_RIGHT, 1, event);

        
        return 1;
        } 
        
    case AID_MFD2_BT3:
    {

        oapiProcessMFDButton (MFD_RIGHT, 2, event);

        
        return 1;
        } 
        
	 case AID_MFD2_BT4:
     {
        
        oapiProcessMFDButton (MFD_RIGHT, 3, event);

        
        return 1;
        } 
        
    case AID_MFD2_BT5:
     {
        
       oapiProcessMFDButton (MFD_RIGHT, 4, event);

        
        return 1;
        } 
        
    case AID_MFD2_BT6:
    {

        oapiProcessMFDButton (MFD_RIGHT, 5, event);

        
        return 1;
        } 
        
    case AID_MFD2_BT7:
     {
        
        oapiProcessMFDButton (MFD_RIGHT, 6, event);

        
        return 1;
        } 
        
    case AID_MFD2_BT8:
     {
        
       oapiProcessMFDButton (MFD_RIGHT, 7, event);

        
        return 1;
        } 
        
    case AID_MFD2_BT9:
    {

        oapiProcessMFDButton (MFD_RIGHT, 8, event);

        
        return 1;
        } 
        
	 case AID_MFD2_BT10:
     {
        
        oapiProcessMFDButton (MFD_RIGHT, 9, event);

        
        return 1;
        } 
        
    case AID_MFD2_BT11:
     {
        
       oapiProcessMFDButton (MFD_RIGHT, 10, event);

        
        return 1;
        } 
        
    case AID_MFD2_BT12:
    {

        oapiProcessMFDButton (MFD_RIGHT, 11, event);

        
        return 1;
        } 
        
        
         // MFD_USER1 buttons
    
    case AID_MFD3_BT1:
     {
        
        oapiProcessMFDButton (MFD_USER1, 0, event);

        
        return 1;
        } 
        
    case AID_MFD3_BT2:
     {
        
       oapiProcessMFDButton (MFD_USER1, 1, event);

        
        return 1;
        } 
        
    case AID_MFD3_BT3:
    {

        oapiProcessMFDButton (MFD_USER1, 2, event);

        
        return 1;
        } 
        
	 case AID_MFD3_BT4:
     {
        
        oapiProcessMFDButton (MFD_USER1, 3, event);

        
        return 1;
        } 
        
    case AID_MFD3_BT5:
     {
        
       oapiProcessMFDButton (MFD_USER1, 4, event);

        
        return 1;
        } 
        
    case AID_MFD3_BT6:
    {

        oapiProcessMFDButton (MFD_USER1, 5, event);

        
        return 1;
        } 
        
        case AID_MFD3_BT7:
     {
        
        oapiProcessMFDButton (MFD_USER1, 6, event);

        
        return 1;
        } 
        
    case AID_MFD3_BT8:
     {
        
       oapiProcessMFDButton (MFD_USER1, 7, event);

        
        return 1;
        } 
        
    case AID_MFD3_BT9:
    {

        oapiProcessMFDButton (MFD_USER1, 8, event);

        
        return 1;
        } 
        
	 case AID_MFD3_BT10:
     {
        
        oapiProcessMFDButton (MFD_USER1, 9, event);

        
        return 1;
        } 
        
    case AID_MFD3_BT11:
     {
        
       oapiProcessMFDButton (MFD_USER1, 10, event);

        
        return 1;
        } 
        
    case AID_MFD3_BT12:
    {

        oapiProcessMFDButton (MFD_USER1, 11, event);

        
        return 1;
        } 
        
        
      // MFD_USER2 buttons
    
    case AID_MFD4_BT1:
     {
        
        oapiProcessMFDButton (MFD_USER2, 0, event);

        
        return 1;
        } 
        
    case AID_MFD4_BT2:
     {
        
       oapiProcessMFDButton (MFD_USER2, 1, event);

        
        return 1;
        } 
        
    case AID_MFD4_BT3:
    {

        oapiProcessMFDButton (MFD_USER2, 2, event);

        
        return 1;
        } 
        
	 case AID_MFD4_BT4:
     {
        
        oapiProcessMFDButton (MFD_USER2, 3, event);

        
        return 1;
        } 
        
    case AID_MFD4_BT5:
     {
        
       oapiProcessMFDButton (MFD_USER2, 4, event);

        
        return 1;
        } 
        
    case AID_MFD4_BT6:
    {

        oapiProcessMFDButton (MFD_USER2, 5, event);

        
        return 1;
        } 
        
        case AID_MFD4_BT7:
     {
        
        oapiProcessMFDButton (MFD_USER2, 6, event);

        
        return 1;
        } 
        
    case AID_MFD4_BT8:
     {
        
       oapiProcessMFDButton (MFD_USER2, 7, event);

        
        return 1;
        } 
        
    case AID_MFD4_BT9:
    {

        oapiProcessMFDButton (MFD_USER2, 8, event);

        
        return 1;
        } 
        
	 case AID_MFD4_BT10:
     {
        
        oapiProcessMFDButton (MFD_USER2, 9, event);

        
        return 1;
        } 
        
    case AID_MFD4_BT11:
     {
        
       oapiProcessMFDButton (MFD_USER2, 10, event);

        
        return 1;
        } 
        
    case AID_MFD4_BT12:
    {

        oapiProcessMFDButton (MFD_USER2, 11, event);

        
        return 1;
        }  
        
	
}

return 0;

}


bool ShuttlePB::clbkLoadVC (int id)
{
	
	SetCameraOffset (_V(0,0.433,4.1));
	
    SetCameraDefaultDirection (_V(0,0,1));

    SetCameraRotationRange (RAD*70, RAD*70, RAD*45, RAD*45);
	
	static VCHUDSPEC hud_pilot  = {1, 185,{0,0.7,4.7},0.5};
	static VCMFDSPEC mfds_left = {1, 180};
    static VCMFDSPEC mfds_mid = {1, 179};
    static VCMFDSPEC mfds_right = {1, 181};
    static VCMFDSPEC mfds_bottom = {1, 183};
    static VCMFDSPEC mfds_topleft = {1, 178};
    static VCMFDSPEC mfds_topright = {1, 182};
		

	
    oapiVCRegisterHUD (&hud_pilot);
    oapiVCRegisterMFD (MFD_LEFT, &mfds_left);
    oapiVCRegisterMFD (MFD_USER1, &mfds_mid);
    oapiVCRegisterMFD (MFD_RIGHT, &mfds_right);
    oapiVCRegisterMFD (MFD_USER2, &mfds_bottom);
    oapiVCRegisterMFD (MFD_USER3, &mfds_topleft);
    oapiVCRegisterMFD (MFD_USER4, &mfds_topright);

    
    
    // Mouse Event
    
    
    // Throttle lever Top animations
		oapiVCRegisterArea (AID_ENGINEMAIN_TOP, PANEL_REDRAW_ALWAYS, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_LBPRESSED);
		oapiVCSetAreaClickmode_Spherical (AID_ENGINEMAIN_TOP, _V(-0.3029756,-0.2528594,3.872622E-02)+offset, 0.05);
		
    // Throttle lever Bot animations
		oapiVCRegisterArea (AID_ENGINEMAIN_BOT, PANEL_REDRAW_ALWAYS, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_LBPRESSED);
		oapiVCSetAreaClickmode_Spherical (AID_ENGINEMAIN_BOT, _V(-0.3029756,-0.2528594,-8.428646E-02)+offset, 0.05);
		
		
		// Gear lever animations
		oapiVCRegisterArea (AID_GEAR, PANEL_REDRAW_ALWAYS, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_LBPRESSED);
		oapiVCSetAreaClickmode_Spherical (AID_GEAR, _V(-0.2933126,-0.1777,0.19)+offset, 0.051);
		
		//MFD Power Buttons
		const double powerButtonRadius = 0.0045; // radius of power button on each MFD
		
	oapiVCRegisterArea (AID_MFD1_PWR, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD2_PWR, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD3_PWR, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);

	oapiVCSetAreaClickmode_Spherical(AID_MFD1_PWR, _V(-0.2261539, -7.848702E-02, 0.2376726)+offset, powerButtonRadius); 
	oapiVCSetAreaClickmode_Spherical(AID_MFD2_PWR, _V(0.1896821, -7.853591E-02, 0.2376726)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD3_PWR, _V(-2.606179E-02, -9.323905E-02, 0.2376726)+offset, powerButtonRadius + 0.0025);  
    
    oapiVCRegisterArea (AID_MFD4_PWR, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD5_PWR, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD6_PWR, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);

	oapiVCSetAreaClickmode_Spherical(AID_MFD4_PWR, _V(-0.017075,-0.2830367, 0.1625559)+offset, powerButtonRadius); 
	oapiVCSetAreaClickmode_Spherical(AID_MFD5_PWR, _V(-0.1926139, 0.113828, 0.2376726)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD6_PWR, _V(0.1673187, 0.1140178, 0.2376726)+offset, powerButtonRadius);  
    
    
		//MFD Mode Select Buttons
		
	oapiVCRegisterArea (AID_MFD1_SEL, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD2_SEL, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD3_SEL, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);

	oapiVCSetAreaClickmode_Spherical(AID_MFD1_SEL, _V(-0.2083955, -7.878125E-02, 0.2376726)+offset, powerButtonRadius);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD2_SEL, _V(0.2084998, -7.859142E-02, 0.2376726)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD3_SEL, _V(-3.488956E-04, -9.263915E-02, 0.2376726)+offset, powerButtonRadius + 0.0025);  
   
    oapiVCRegisterArea (AID_MFD4_SEL, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD5_SEL, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD6_SEL, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);

	oapiVCSetAreaClickmode_Spherical(AID_MFD4_SEL, _V(-4.651739E-04,-0.2830367, 0.1625559)+offset, powerButtonRadius);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD5_SEL, _V(-0.1802692, 0.113828, 0.2376726)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD6_SEL, _V(0.1805514, 0.113828, 0.2376726)+offset, powerButtonRadius + 0.0025);  
   
    
    
    //MFD Menu Buttons
		
	oapiVCRegisterArea (AID_MFD1_MEN, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD2_MEN, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD3_MEN, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);

	oapiVCSetAreaClickmode_Spherical(AID_MFD1_MEN, _V(-0.1891487, -7.878125E-02, 0.2376726)+offset, powerButtonRadius);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD2_MEN, _V(0.2259218, -7.859142E-02, 0.2376726)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD3_MEN, _V(2.800996E-02, -9.263915E-02, 0.2376726)+offset, powerButtonRadius + 0.0025);  
   
    oapiVCRegisterArea (AID_MFD4_MEN, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD5_MEN, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD6_MEN, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);

	oapiVCSetAreaClickmode_Spherical(AID_MFD4_MEN, _V(1.897151E-02,-0.2830367, 0.1625559)+offset, powerButtonRadius);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD5_MEN, _V(-0.167434, 0.113828, 0.2376726)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD6_MEN, _V(0.1928906, 0.113828, 0.2376726)+offset, powerButtonRadius + 0.0025);  
   
    
    
    //MFD_LEFT Buttons
		
	oapiVCRegisterArea (AID_MFD1_BT1, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD1_BT2, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD1_BT3, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
    oapiVCRegisterArea (AID_MFD1_BT4, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD1_BT5, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD1_BT6, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
    oapiVCRegisterArea (AID_MFD1_BT7, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD1_BT8, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD1_BT9, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
    oapiVCRegisterArea (AID_MFD1_BT10, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD1_BT11, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD1_BT12, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);

	oapiVCSetAreaClickmode_Spherical(AID_MFD1_BT1, _V(-0.2818186, 3.322101E-02, 0.23601201)+offset, powerButtonRadius);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD1_BT2, _V(-0.2818186, 1.537658E-02, 0.23601201)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD1_BT3, _V(-0.2818186, -3.606856E-03, 0.23601201)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD1_BT4, _V(-0.2818186, -2.069194E-02, 0.23601201)+offset, powerButtonRadius);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD1_BT5, _V(-0.2818186, -3.929571E-02, 0.23601201)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD1_BT6, _V(-0.2818186, -0.0563808, 0.23601201)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD1_BT7, _V(-0.1295715, 3.322101E-02, 0.23601201)+offset, powerButtonRadius);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD1_BT8, _V(-0.1295715, 1.537658E-02, 0.23601201)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD1_BT9, _V(-0.1295715, -3.606856E-03, 0.23601201)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD1_BT10, _V(-0.1295715, -2.069194E-02, 0.23601201)+offset, powerButtonRadius);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD1_BT11, _V(-0.1295715, -3.929571E-02, 0.23601201)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD1_BT12, _V(-0.1295715, -0.0563808, 0.23601201)+offset, powerButtonRadius);  
   
    
    //MFD_RIGHT Buttons
		
	oapiVCRegisterArea (AID_MFD2_BT1, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD2_BT2, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD2_BT3, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
    oapiVCRegisterArea (AID_MFD2_BT4, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD2_BT5, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD2_BT6, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
    oapiVCRegisterArea (AID_MFD2_BT7, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD2_BT8, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD2_BT9, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
    oapiVCRegisterArea (AID_MFD2_BT10, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD2_BT11, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD2_BT12, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);

	oapiVCSetAreaClickmode_Spherical(AID_MFD2_BT1, _V(0.1299084, 3.322101E-02, 0.23601201)+offset, powerButtonRadius);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD2_BT2, _V(0.1299084, 1.537658E-02, 0.23601201)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD2_BT3, _V(0.1299084, -3.606856E-03, 0.23601201)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD2_BT4, _V(0.1299084, -2.069194E-02, 0.23601201)+offset, powerButtonRadius);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD2_BT5, _V(0.1299084, -3.929571E-02, 0.23601201)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD2_BT6, _V(0.1299084, -0.0563808, 0.23601201)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD2_BT7, _V(0.2817759, 3.322101E-02, 0.23601201)+offset, powerButtonRadius);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD2_BT8, _V(0.2817759, 1.537658E-02, 0.23601201)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD2_BT9, _V(0.2817759, -3.606856E-03, 0.23601201)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD2_BT10, _V(0.2817759, -2.069194E-02, 0.23601201)+offset, powerButtonRadius);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD2_BT11, _V(0.2817759, -3.929571E-02, 0.23601201)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD2_BT12, _V(0.2817759, -0.0563808, 0.23601201)+offset, powerButtonRadius);  
   
    
    //MFD_USER1 Buttons
		
	oapiVCRegisterArea (AID_MFD3_BT1, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD3_BT2, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD3_BT3, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
    oapiVCRegisterArea (AID_MFD3_BT4, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD3_BT5, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD3_BT6, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
    oapiVCRegisterArea (AID_MFD3_BT7, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD3_BT8, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD3_BT9, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
    oapiVCRegisterArea (AID_MFD3_BT10, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD3_BT11, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD3_BT12, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);

	oapiVCSetAreaClickmode_Spherical(AID_MFD3_BT1, _V(-9.006701E-02, 4.798551E-02, 0.2376726)+offset, powerButtonRadius + 0.0025);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD3_BT2, _V(-9.006701E-02, 2.520539E-02, 0.2376726)+offset, powerButtonRadius + 0.0025);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD3_BT3, _V(-9.006701E-02, 1.286258E-03, 0.2376726)+offset, powerButtonRadius + 0.0025);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD3_BT4, _V(-9.006701E-02, -2.149386E-02, 0.2376726)+offset, powerButtonRadius + 0.0025);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD3_BT5, _V(-9.006701E-02, -4.427398E-02, 0.2376726)+offset, powerButtonRadius + 0.0025);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD3_BT6, _V(-9.006701E-02, -6.743377E-02, 0.2376726)+offset, powerButtonRadius + 0.0025);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD3_BT7, _V(8.973698E-02, 4.798551E-02, 0.2376726)+offset, powerButtonRadius + 0.0025);   
	oapiVCSetAreaClickmode_Spherical(AID_MFD3_BT8, _V(8.973698E-02, 2.520539E-02, 0.2376726)+offset, powerButtonRadius + 0.0025);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD3_BT9, _V(8.973698E-02, 1.286258E-03, 0.2376726)+offset, powerButtonRadius + 0.0025);   
    oapiVCSetAreaClickmode_Spherical(AID_MFD3_BT10, _V(8.973698E-02, -2.149386E-02, 0.2376726)+offset, powerButtonRadius + 0.0025);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD3_BT11, _V(8.973698E-02, -4.427398E-02, 0.2376726)+offset, powerButtonRadius + 0.0025);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD3_BT12, _V(8.973698E-02, -6.743377E-02, 0.2376726)+offset, powerButtonRadius + 0.0025);  
   
    
    //MFD_USER2 Buttons
		
	oapiVCRegisterArea (AID_MFD4_BT1, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD4_BT2, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD4_BT3, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
    oapiVCRegisterArea (AID_MFD4_BT4, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD4_BT5, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD4_BT6, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
    oapiVCRegisterArea (AID_MFD4_BT7, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD4_BT8, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD4_BT9, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
    oapiVCRegisterArea (AID_MFD4_BT10, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD4_BT11, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD4_BT12, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);

	oapiVCSetAreaClickmode_Spherical(AID_MFD4_BT1, _V(-6.321591E-02, -0.1845854, 0.2119598)+offset, powerButtonRadius);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD4_BT2, _V(-6.321591E-02, -0.1983538, 0.2024679)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD4_BT3, _V(-6.321591E-02, -0.2142173, 0.1922167)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD4_BT4, _V(-6.321591E-02, -0.2285843, 0.1827248)+offset, powerButtonRadius);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD4_BT5, _V(-6.321591E-02, -0.2429513, 0.1728532)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD4_BT6, _V(-6.321591E-02, -0.2573183, 0.1641207)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD4_BT7, _V(6.283623E-02, -0.1845854, 0.2119598)+offset, powerButtonRadius);   
	oapiVCSetAreaClickmode_Spherical(AID_MFD4_BT8, _V(6.283623E-02, -0.1916471, 0.2024679)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD4_BT9, _V(6.283623E-02, -0.2142173, 0.1922167)+offset, powerButtonRadius);   
    oapiVCSetAreaClickmode_Spherical(AID_MFD4_BT10, _V(6.283623E-02, -0.2285843, 0.1827248)+offset, powerButtonRadius);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD4_BT11, _V(6.283623E-02, -0.2429513, 0.1728532)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD4_BT12, _V(6.283623E-02, -0.2573183, 0.1641207)+offset, powerButtonRadius);  
   
   
    
    
    
    // End
		
	
	
 return 1;

}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new ShuttlePB (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (ShuttlePB*)vessel;
}
