// ==============================================================
//                 ORBITER MODULE: ShuttlePB
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// ShuttlePB.cpp
// Control module for ShuttlePB vessel class
//
// Notes:
// This is an example for a "minimal" vessel implementation which
// only overloads the clbkSetClassCaps method to define vessel
// capabilities and otherwise uses the default VESSEL class
// behaviour.
// ==============================================================

#define STRICT
#define ORBITER_MODULE

//#define ENG 				1	
//#define GEAR				2	
//#define BRAKE               3

#include <orbitersdk.h>
#include <cstdio>
#include <cmath>


//#include <string>
#include "OrbiterSoundSDK35.h"

//#include <stdio.h>
//#include <stdlib.h>

//using namespace std;


// ==============================================================
// Some vessel parameters
// ==============================================================
const double PB_FUELMASS = 20000.0;
const double PB_ISP = 50000.0;
const double PB_MAXMAINTH =0;

const double PB_ISP2 = 0.1;
const double PB_MAXHOVERTH =0;



double t0;

double tvec = 0;
double tvec2 = 0;

double i77 = 0;
double mx1 = 0;
double iop = 0;
double st = 0;
double pld = 0;
double cen = 0;
double gear_proc = 0;
double gear_proc2 = 0;
double gear_procl = 0;
double power_proc = 0;
double fuel_proc = 0;
//ouble speedbrake = 0;
//double swingwing = 0;
//double swing = 0;
//double swing3 = 0;
double hook = 0;
double canopy = 0;
//double flaps = 0;
//double flaps2 = 0;
double exproc = 0;
double Lights = 0;
double switched = 0;
double aoa;
double dil = 0;
double eng_pitch = 22000;

double start = 0;
double full = 0.001;
double i = 0;
double i1 = 0;
double i2 = 0;
double i3 = 0;
double i4 = 0;
double i5 = 0;
double i7 = 0;
double i9 = 0;
double i1a = 0;
double i2a = 0;
double i3a = 0;
double i4a = 0;
double i5a = 0;
double s1 = 0;
double s2 = 0;
double s3 = 0;
double s4 = 0;
double s5 = 0;
double ga = 0;
double diff = 0.1;
double k3;
int k4;
char k3buffer [33];
double amet = 0;
double dee = 0;
double i13 = 0;
double i14 = 0;
double i15 = 0;
double i16 = 0;
double i17 = 0;
double i18 = 0;
double i19 = 0;
double i20 = 0;
double i21 = 0;
double droptank = 0;

double i33 = 0;
double dee33 = 0;


double secondtime = 0;

double refuel = 0;

UINT animactive;

UINT animactivel;

UINT animactive2;

UINT animactive3;

UINT animactive4;

UINT animactive5;

UINT animactive7;

UINT animactive9;

const VECTOR3 offset = {0,0.2,4.5};

UINT anim_bank;


THRUSTER_HANDLE th_main;

THRUSTER_HANDLE th_hover;

BEACONLIGHTSPEC beacon1;    

BEACONLIGHTSPEC beacon2;   

MESHHANDLE f150;
MESHHANDLE f151;





// 1. vertical lift component (wings and body)

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	static const double CL[nabsc] = {0.1, 0.17, 0.2, 0.2, 0.17, 0.1, 0, -0.11, -0.24, -0.38,  -0.5,  -0.5, -0.02, 0.6355,    0.63,   0.46, 0.28, 0.13, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,    0,0.002,0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = CM[idx] + (CM[idx+1]-CM[idx])*d;
	*cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.02 + oapiGetInducedDrag (*cl, 1.5, 0.6);
}


// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class ShuttlePB: public VESSEL2 {
public:
	ShuttlePB (OBJHANDLE hVessel, int flightmodel);
	//	: VESSEL2 (hVessel, flightmodel) {}
	void clbkSetClassCaps (FILEHANDLE cfg);
	void DefineAnimations(void);
	void Steering();
	void clbkPostCreation (void);
    void clbkPostStep (double simt, double simdt, double mjd);
    int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
   
    UINT anim_wheel;
    UINT anim_gear;
    UINT anim_gearlever;
    //UINT anim_rudder;
    UINT anim_elevator;
    UINT anim_laileron;
    UINT anim_raileron;
   // UINT anim_speedbrake;
    
    //UINT anim_flaps;
    UINT anim_hook;
    
    UINT anim_power;
   
   UINT anim_wheel0;

    UINT anim_dial2;

   
    bool clbkLoadVC (int id);
    
    int MySoundID;	


};

ShuttlePB::ShuttlePB (OBJHANDLE hVessel, int flightmodel)
		: VESSEL2 (hVessel, flightmodel)
{
	
	// preload meshes
	
    f150                 = oapiLoadMeshGlobal ("K33\\K-Mirage2000");
    f151                 = oapiLoadMeshGlobal ("K33\\K-Mirage2000-Cpit");
    

	
	DefineAnimations ();
	
}


void ShuttlePB::clbkPostCreation (void)
{

int MySoundID=ConnectToOrbiterSoundDLL3(GetHandle());

ReplaceStockSound3(MySoundID,"Sound\\K-Mirage2000\\K-Mirage2000-Eng.wav",	REPLACE_MAIN_THRUST);

//RequestLoadVesselWave3(MySoundID,ENG,"Sound\\Vessel\\K-FA-22A\\K-FA-22A-Eng.wav",BOTHVIEW_FADED_FAR);

//RequestLoadVesselWave3(MySoundID,BRAKE,"Sound\\Vessel\\K-FA-22A\\K-FA-22A-Brake.wav",BOTHVIEW_FADED_FAR);

//RequestLoadVesselWave3(MySoundID,GEAR,"Sound\\Vessel\\K-FA-22A\\K-FA-22A-Gear.wav",BOTHVIEW_FADED_FAR);


//SoundOptionOnOff3(MySoundID,PLAYMAINTHRUST,FALSE);

SoundOptionOnOff3(MySoundID,PLAYHOVERTHRUST,FALSE);


SoundOptionOnOff3(MySoundID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);

SoundOptionOnOff3(MySoundID,PLAYCABINAIRCONDITIONING,FALSE);

SoundOptionOnOff3(MySoundID,PLAYCABINRANDOMAMBIANCE,FALSE);

SoundOptionOnOff3(MySoundID,PLAYRADIOATC,FALSE);

SoundOptionOnOff3(MySoundID,DISPLAYTIMER,FALSE);

}


// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------

void ShuttlePB::clbkSetClassCaps (FILEHANDLE cfg)
{
	// THRUSTER_HANDLE th_main;


	// physical specs
	SetSize (8);
	SetEmptyMass (12000.0);
	SetCW (0.3, 0.3, 0.6, 0.9);
	SetWingAspect (1.7);
	SetWingEffectiveness (3.1);
	SetCrossSections (_V(5,5,15));
	SetRotDrag (_V(3.5,3.5,3.5));
	if (GetFlightModel() >= 1) {
		SetPitchMomentScale (1e-4);
		SetBankMomentScale (1e-4);
	}
	SetPMI (_V(15.5,22.1,7.7));
	SetTrimScale (0.05);
	SetCameraOffset (_V(0,0.4,4.4));
	
	SetTouchdownPoints (_V(0,-2.428501,2.879999), _V(-1.627791,-2.420502,-1.916788), _V(1.627791,-2.420502,-1.916788));

	// **************************** aerodynamics ***********************************

	CreateAirfoil (LIFT_VERTICAL,   _V(0,0,-0.5), VLiftCoeff, 20, 270, 2.266);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,-4), HLiftCoeff, 20,  50, 1.5);





	CreateControlSurface (AIRCTRL_ELEVATOR,     7, 0.7, _V(   0,0,-17), AIRCTRL_AXIS_XPOS,anim_elevator);
	CreateControlSurface (AIRCTRL_RUDDER,       2, 0.5, _V(   0,2,-15), AIRCTRL_AXIS_YPOS);
	CreateControlSurface (AIRCTRL_AILERON,      4.8, 0.2, _V( 17.8,0,-17.8), AIRCTRL_AXIS_XPOS,anim_raileron);
	CreateControlSurface (AIRCTRL_AILERON,      4.8, 0.2, _V(-17.8,0,-17.8), AIRCTRL_AXIS_XNEG,anim_laileron);
	CreateControlSurface (AIRCTRL_ELEVATORTRIM, 5, 0.1, _V(   0,0,-15), AIRCTRL_AXIS_XPOS,anim_elevator);
    CreateControlSurface (AIRCTRL_FLAP,         5, 5, _V( 0,0,-1), AIRCTRL_AXIS_XPOS);
	
	//CreateVariableDragElement( &speedbrake, 8.0, _V(0,0,0) );

	CreateVariableDragElement (&gear_proc2, 0.8, _V(0, -1, 0));
	
	//CreateVariableDragElement (&hook, 3.33, _V(0,1,0));


	
	// propellant resources
	PROPELLANT_HANDLE hpr = CreatePropellantResource (PB_FUELMASS);
	
	
	
	
	SURFHANDLE tex_main = oapiRegisterExhaustTexture ("K-Mirage-Exhaust");



	// ***************** thruster definitions *******************

	PARTICLESTREAMSPEC contrail_main = {
		0, 0.4, 165, 19, 0.019, 0.111, 1.11, 1.11, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 2,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};
	
	PARTICLESTREAMSPEC contrail_hover = {
		0, 0.033, 111, 1, 0.01, 2, 0.333, 1, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 2,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};
	
	th_main = CreateThruster (_V(0,0,-4.3), _V(0,0,1), PB_MAXMAINTH, hpr, PB_ISP);
	
	th_hover = CreateThruster (_V(0,0,-4.3), _V(0,0,1), PB_MAXHOVERTH, hpr, PB_ISP2);
	
	
	
	CreateThrusterGroup (&th_main, 1, THGROUP_MAIN);
	
	CreateThrusterGroup (&th_hover, 1, THGROUP_HOVER);
	
	
	
	AddExhaust (th_main, 5.5, 0.65, _V(0,-0.4581167,-6.3), _V(0,0,-1) , tex_main);
	//AddExhaust (th_main, 5, 0.5, _V(0.555169,-0.4163767,-4.3), _V(0,0,-1) , tex_main);

    
    AddExhaustStream (th_main, _V(0,-0.4581167,-6.9), &contrail_main);
    
    AddExhaustStream (th_hover, _V(-4.349663,-1.187651,-4.745327), &contrail_hover);
    AddExhaustStream (th_hover, _V(4.349663,-1.187651,-4.745327), &contrail_hover);
    
    
    
      // Strobe lights
	
	//Blue strobe
	
	static VECTOR3 beacon1pos = {4.349663,-1.187651,-4.745327};
	static VECTOR3 beacon1col = {0.043, 0.161, 0.992};
	
		beacon1.shape = (BEACONSHAPE_STAR);
		beacon1.pos = &beacon1pos;
		beacon1.col = &beacon1col;
		beacon1.size = (0.18);
		beacon1.falloff = (0.6);
		beacon1.period = (1.13);
		beacon1.duration = (0.05);
		beacon1.tofs = (0.6);
		beacon1.active = false;
		
		AddBeacon (&beacon1);
		
		//Red strobe
	
	static VECTOR3 beacon2pos = {-4.349663,-1.187651,-4.745327};
	static VECTOR3 beacon2col = {0.988, 0.133, 0.133};
	
		beacon2.shape = (BEACONSHAPE_STAR);
		beacon2.pos = &beacon2pos;
		beacon2.col = &beacon2col;
		beacon2.size = (0.18);
		beacon2.falloff = (0.6);
		beacon2.period = (1.13);
		beacon2.duration = (0.05);
		beacon2.tofs = (0.6);
		beacon2.active = false;
		
		AddBeacon (&beacon2);
		

    

    
    
    
	// visual specs
	
	AddMesh (f150);
	
	SetMeshVisibilityMode( AddMesh (f151,&offset), MESHVIS_VC );

    

}

// ==============================================================
// API callback interface
// ==============================================================

void ShuttlePB::clbkPostStep (double simt, double simdt, double mjd)

{
	
	if( oapiGetTimeAcceleration() > 10.0 ) oapiSetTimeAcceleration(10);
	
	double vsAirSpeed = GetAirspeed();
	
	
	
	
    //speed and altitude control
	
	if (GetAirspeed() >= 700) 
	{
		SetCW (999, 999, 999, 999);
		SetThrusterMax0 (th_main ,0);

	
}

if (GetAltitude() >= 17000) 
	{
		SetCW (999, 999, 999, 999);
		SetThrusterMax0 (th_main ,0);

	
}



if (GetAirspeed() < 700 && GetAltitude() < 17000 ) 
	{
		SetCW (0.3, 0.3, 0.67, 0.92);
		SetThrusterMax0 (th_main ,4.4e5);

	
}

	double vsAirSpeed2 = (vsAirSpeed/1000)*3600;
    
    //Ground Steering
    
    if (GroundContact () ) Steering();
    
    
    //aoa = GetAOA();
    
    //sprintf(oapiDebugString(),"anim %2.2f", aoa );
    
  

    //Anims 
    
    if (iop > 1)iop=0;
    
    if (vsAirSpeed > 0) {
    
	double aspeed = vsAirSpeed;
    if (iop > 1)iop=0;
    if (vsAirSpeed > 30) aspeed = 30; 
    iop = iop + (aspeed /300);
     
    if (vsAirSpeed ==0) iop=0;
    
    
    if ( GroundContact() && gear_proc < 0.05) SetAnimation (anim_wheel, iop);
    

   }
	
	
// Gear animations


   if( animactive && !GroundContact()) {
		if( animactive == 1 ) {
			gear_proc += 0.00242;
			if( gear_proc > 1.0 ) {
				gear_proc = 1.0;
				animactive = 0;
			}
		}
		else  {
			gear_proc -= 0.00242;
			if( gear_proc < 0.0 ) {
				gear_proc = 0.0;
				animactive = 0;
			}
		}
		SetAnimation( anim_gear, gear_proc );
		gear_proc2 = -gear_proc + 1;
	}

	
	// Gear Lever animations


   if( animactivel && !GroundContact()) {
		if( animactivel == 1 ) {
			gear_procl += 0.00242;
			if( gear_procl > 1.0 ) {
				gear_procl = 1.0;
				animactivel = 0;
			}
		}
		else  {
			gear_procl -= 0.00242;
			if( gear_procl < 0.0 ) {
				gear_procl = 0.0;
				animactivel = 0;
			}
		}
		SetAnimation( anim_gearlever, gear_procl );
	}

	
	// Throttle animation

        power_proc = GetThrusterLevel(th_main);
	
		SetAnimation( anim_power, power_proc );
		
	
   
	
	

	
	// Canopy Animations


   if( animactive9 ) {
		if( animactive9 == 1 ) {
			hook += 0.003;
			if( hook > 1.0 ) {
				hook = 1.0;
				animactive9 = 0;
			}
		}
		else  {
			hook -= 0.003;
			if( hook < 0.0 ) {
				hook = 0.0;
				animactive9 = 0;
			}
		}
		SetAnimation( anim_hook, hook );
	}
	
	
  
   
 if (Lights == 1 && switched == 1)
 {
	 beacon1.active = true;
	 beacon2.active = true;

	 switched = 0;
	 
}

if (Lights == 0 && switched == 1)
{
	beacon1.active = false;
	beacon2.active = false;

    switched = 0;
    
}


   // Refuel
   
   if (refuel == 1)
   {
	   
	  SetFuelMass (20000.0);

	   refuel = 0;
   }
 
  // Contrails
  
   aoa = GetAOA();
    
    
   i77 = aoa * 6;

  
  if ( GetAirspeed() >= 170 && i77 >= 1)
  
  {
	  SetThrusterMax0 (th_hover ,1);
	  SetThrusterLevel (th_hover ,1);
	  
  }
  
  if ( i77 < 1)
  
  {
	  SetThrusterMax0 (th_hover ,0);
	  SetThrusterLevel (th_hover ,0);
  }
  

  if ( GetAirspeed() < 170)
  
  {
	  SetThrusterMax0 (th_hover ,0);
	  SetThrusterLevel (th_hover ,0);
  }
  
  
  
  // Altimeter animation
   
   
    //Anims 
    
     k3 = GetAltitude();
    
     i = k3 /3333;
	
     if( i >1 ) i = 1;
	
	SetAnimation (anim_wheel0, i);
	
	

	// Horizon Bank Guage Animation
    
   i15 = GetBank();
   
   i16 = (i15 + 3) / 6;
   
   if (i16 < 0) i16 = 0;
   
   if (i16 > 1) i16 = 1;
   
   SetAnimation (anim_bank, i16);
   
    //sprintf(oapiDebugString(),"hed %5.5f", i15 );
    

   // Direction Dial Animation

   oapiGetFocusHeading(&dil);
    
   i9 = (dil / 6.25) + 0.001;
   
   //sprintf(oapiDebugString(),"hed %2.2f", dil );

   SetAnimation (anim_dial2, i9);

  
  
}

//Ground Steering


void ShuttlePB::Steering()
 {

	double pos = GetControlSurfaceLevel(AIRCTRL_RUDDER);
	double lev = 3e6 * pos * GetAirspeed() / 10.0;

	if( oapiGetTimeAcceleration() > 10.0 )
		return;

	AddForce( _V( lev,0,0 ), _V( 0,0,0.77 ) );
	AddForce( _V( -1.0*lev,0,0 ), _V( 0,0,-0.77 ) );
	
}



// --------------------------------------------------------------
// Keyboard interface handler (buffered key events)
// --------------------------------------------------------------
int ShuttlePB::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate)
{
	if (!down) return 0; // only process keydown events

	// if (KEYMOD_SHIFT (kstate)) {

		switch (key) {
		case OAPI_KEY_G: // Gear
		
		if( gear_proc > 0.5 )
					animactive = 2;
				else
					animactive = 1;

		
		
		if( gear_procl > 0.5 )
					animactivel = 2;
				else
					animactivel = 1;

			
			return 1;
			
			
			
			
			case OAPI_KEY_9: // Canopy
		
		if( hook > 0.5 )
					animactive9 = 2;
				else
					animactive9 = 1;

		
			
			return 1;
			
			
			
			
			
			case OAPI_KEY_J: // Refuel
			
			
		if (refuel == 0 && GetAirspeed() == 0)
		{
			refuel = 1;
		}
		
			
			return 1;
		
		

			case OAPI_KEY_K: // Lights
		
		if( Lights == 0 )
		{
			Lights = 1;
			switched =1;
					
			return 1;
}

		if( Lights == 1 )
		{
			Lights = 0;
			switched = 1;
					
			return 1;
}
	
    } 
	return 0;
}


void ShuttlePB::DefineAnimations(void)
{
   // Front Wheel
   static UINT WGrp1[2] = {124,125};
   
   static MGROUP_ROTATE wheel1 (0,WGrp1, 2, _V(0,-2.24121,2.879426), _V(1,0,0), (float)(360*RAD));
   
   //Rear left Wheel
   static UINT WGrp3[1] = {126};
   
   static MGROUP_ROTATE wheel2 (0,WGrp3, 1, _V(0,-2.057619,-1.916788), _V(1,0,0), (float)(360*RAD));
   
   //Rear Right wheel
   static UINT WGrp4[1] = {127};
   
   static MGROUP_ROTATE wheel3 (0,WGrp4, 1, _V(0,-2.057619,-1.916788), _V(1,0,0), (float)(360*RAD));
   
   anim_wheel = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel1);
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel2);
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel3);
   
   
   
   
   // Gear anims
  
   
   //Front Gear
   static UINT GGrp1[7] = {6,7,8,13,52,124,125};
   
   static MGROUP_ROTATE gear1 (0,GGrp1, 7, _V(0,-1.204259,3.27775), _V(1,0,0), (float)(95*RAD));
   
   
   //Rear Gear Left
   static UINT GGrp2[2] = {11,126};
   
   static MGROUP_ROTATE gear2 (0,GGrp2, 2, _V(-1.69,-1.0,0), _V(0,0,1), (float)(91*RAD));
   
   //Rear Gear Right
   static UINT GGrp3[2] = {9,127};
   
   static MGROUP_ROTATE gear3 (0,GGrp3, 2, _V(1.69,-1.0,0), _V(0,0,1), (float)(-91*RAD));
   
   //Front Gear Cover Left
   static UINT GGrp4[1] = {67};
   
   static MGROUP_ROTATE gear4 (0,GGrp4, 1, _V(-0.1351325,-1.175551,0), _V(0,0,1), (float)(90*RAD));
   
   //Front Gear Cover Right
   static UINT GGrp5[1] = {68};
   
   static MGROUP_ROTATE gear5 (0,GGrp5, 1, _V(0.1351325,-1.175551,0), _V(0,0,1), (float)(-90*RAD));
   
   
   //Rear Cover Left1
   static UINT GGrp12[1] = {108};
   
   static MGROUP_ROTATE gear12 (0,GGrp12, 1, _V(-6.171267E-02,-1.031777,0), _V(0,0,1), (float)(-111*RAD));
   
   //Rear Cover Right1
   static UINT GGrp13[1] = {109};
   
   static MGROUP_ROTATE gear13 (0,GGrp13, 1, _V(6.171267E-02,-1.031777,0), _V(0,0,1), (float)(111*RAD));
   
   
     //Rear Gear Left2
   static UINT GGrp14[1] = {106};
   
   static MGROUP_ROTATE gear14 (0,GGrp14, 1, _V(-1.802648,-1.050733,0), _V(0,0,1), (float)(91*RAD));
   
   //Rear Gear Right2
   static UINT GGrp15[1] = {107};
   
   static MGROUP_ROTATE gear15 (0,GGrp15, 1, _V(1.802648,-1.050733,0), _V(0,0,1), (float)(-91*RAD));
  
    //Rear Gear Left3
   static UINT GGrp16[1] = {12};
   
   static MGROUP_ROTATE gear16 (0,GGrp16, 1, _V(-1.281403,-1.027126,0), _V(0,0,1), (float)(-55*RAD));
   
   //Rear Gear Right3
   static UINT GGrp17[1] = {10};
   
   static MGROUP_ROTATE gear17 (0,GGrp17, 1, _V(1.281403,-1.027126,0), _V(0,0,1), (float)(55*RAD));
  
   
   
   anim_gear = CreateAnimation (0);
   
   AddAnimationComponent  (anim_gear, 0, 0.5, &gear1);
   AddAnimationComponent  (anim_gear, 0, 0.5, &gear2);
   AddAnimationComponent  (anim_gear, 0, 0.5, &gear3);
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear4);
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear5);
    
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear12);
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear13);
  
   AddAnimationComponent  (anim_gear, 0.1, 0.5, &gear14);
   AddAnimationComponent  (anim_gear, 0.1, 0.5, &gear15);
   
   AddAnimationComponent  (anim_gear, 0.1, 0.5, &gear16);
   AddAnimationComponent  (anim_gear, 0.1, 0.5, &gear17);
   
   
   //Elevator
   static UINT EGrp1[2] = {134,142};
   
   static MGROUP_ROTATE elevator (0,EGrp1, 2, _V(0,-0.9223039,-4.268489), _V(1,0,0), (float)(33*RAD));
   
   
   anim_elevator = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_elevator, 0, 1, &elevator);
   
   
   
   
   //Left Aleron
   static UINT LAGrp1[1] = {133};
   
   static MGROUP_ROTATE laileron (0,LAGrp1, 1, _V(-3.117516,-1.067539,-4.285933), _V(1,0,-0.1), (float)(-40*RAD));
   
   
   anim_laileron = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_laileron, 0, 1, &laileron);
   
   
   //Right Aleron
   static UINT RAGrp1[1] = {141};
   
   static MGROUP_ROTATE raileron (0,RAGrp1, 1, _V(3.117516,-1.067539,-4.285933), _V(1,0,0.1), (float)(40*RAD));
   
   
   anim_raileron = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_raileron, 0, 1, &raileron);
   
   
  
   //Canopy
   static UINT HGrp1[4] = {18,69,166,167};
   
   static MGROUP_ROTATE hook (0,HGrp1, 4, _V(0,0.2247857,1.918806), _V(1,0,0), (float)(-40*RAD));
    
   
   anim_hook = CreateAnimation (0);
   
   AddAnimationComponent  (anim_hook, 0, 1, &hook);
   
  
   
   // Gear Lever Animation
   
   //Gear Lever in Cockpit
   static UINT Glev0[2] = {43,44};
   
   static MGROUP_TRANSLATE gearlev (1,Glev0, 2,  _V(0,0.015,0.007));
   
   anim_gearlever = CreateAnimation (0);
   
   AddAnimationComponent  (anim_gearlever, 0, 1, &gearlev);
   
   
   
   // Throttle Animation
   
   static UINT Pow0[5] = {89,90,91,92,93};
   
   static MGROUP_TRANSLATE powg (1,Pow0, 5, _V(0,0,0.07));
   
   anim_power = CreateAnimation (0);
   
   AddAnimationComponent  (anim_power, 0, 1, &powg);
   
   
   
    // Altimeter Animations
   
   // Dial Long
   static UINT wWGrp0[1] = {2};
   
   static MGROUP_ROTATE wwheel0 (1,wWGrp0, 1, _V(-0.1192043,6.317467E-03,0), _V(0,0,1), (float)(-360*RAD));
   
   static UINT wWGrp1[1] = {3};
   
   static MGROUP_ROTATE wwheel1 (1,wWGrp1, 1, _V(0.141381,5.510744E-02,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_wheel0 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel0, 0, 1, &wwheel0);
   AddAnimationComponent  (anim_wheel0, 0, 1, &wwheel1);

   
   // Horizon Bank Guage Animations
   
   static UINT BAGrp0[1] = {45};
   
   static MGROUP_ROTATE bank (1,BAGrp0, 1, _V(-8.477551E-02,5.119979E-02,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_bank = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_bank, 0, 1, &bank);

   
   // Direction Dial Animations
   
   // Dial2
   static UINT d2Grp0[1] = {42};
   
   static MGROUP_ROTATE dial2 (1,d2Grp0, 1, _V(9.352531E-02,2.041882E-03,0), _V(0,0,1), (float)(360*RAD));
   
   
   anim_dial2 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_dial2, 0, 1, &dial2);

   
}
	


bool ShuttlePB::clbkLoadVC (int id)
{
	
	SetCameraOffset (_V(0,0.4,4.2));
	
    SetCameraDefaultDirection (_V(0,0,1));

    SetCameraRotationRange (RAD*70, RAD*70, RAD*45, RAD*45);
	
	static VCHUDSPEC hud_pilot  = {1, 101,{0,0.7,4.7},0.5};
	static VCMFDSPEC mfds_left = {1, 100};
    

	
    oapiVCRegisterHUD (&hud_pilot);
    oapiVCRegisterMFD (MFD_LEFT, &mfds_left);
    
    
    
    
	
	
 return 1;

}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new ShuttlePB (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (ShuttlePB*)vessel;
}
