// ==============================================================
//                 ORBITER MODULE: ShuttlePB
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// ShuttlePB.cpp
// Control module for ShuttlePB vessel class
//
// Notes:
// This is an example for a "minimal" vessel implementation which
// only overloads the clbkSetClassCaps method to define vessel
// capabilities and otherwise uses the default VESSEL class
// behaviour.
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#define ENG 				1	
#define GEAR				2	
#define FLAPS               3

#include <orbitersdk.h>
#include <cstdio>
#include <cmath>


#include <string>
#include "OrbiterSoundSDK35.h"

#include <stdio.h>
#include <stdlib.h>

using namespace std;


// ==============================================================
// Some vessel parameters
// ==============================================================
const double PB_FUELMASS = 20000.0;
const double PB_ISP = 50000.0;
const double PB_MAXMAINTH =0;

#define AID_ENGINEMAIN_TOP           4
#define AID_ENGINEMAIN_BOT           5
#define AID_GEAR                     6

// power buttons
#define AID_MFD1_PWR      7
#define AID_MFD2_PWR      8

// Mode Select buttons
#define AID_MFD1_SEL      13
#define AID_MFD2_SEL      14

// Menu buttons
#define AID_MFD1_MEN      19
#define AID_MFD2_MEN      20

// MFD_LEFT buttons
#define AID_MFD1_BT1      25
#define AID_MFD1_BT2      26
#define AID_MFD1_BT3      27
#define AID_MFD1_BT4      28
#define AID_MFD1_BT5      29
#define AID_MFD1_BT6      30
#define AID_MFD1_BT7      31
#define AID_MFD1_BT8      32
#define AID_MFD1_BT9      33
#define AID_MFD1_BT10     34
#define AID_MFD1_BT11     35
#define AID_MFD1_BT12     36

// MFD_RIGHT buttons
#define AID_MFD2_BT1      37
#define AID_MFD2_BT2      38
#define AID_MFD2_BT3      39
#define AID_MFD2_BT4      40
#define AID_MFD2_BT5      41
#define AID_MFD2_BT6      42
#define AID_MFD2_BT7      43
#define AID_MFD2_BT8      44
#define AID_MFD2_BT9      45
#define AID_MFD2_BT10     46
#define AID_MFD2_BT11     47
#define AID_MFD2_BT12     48

#define AID_FLAPS         49
#define AID_BRAKES        50
#define AID_LIGHTS        51

double t0;

double tvec = 0;
double tvec2 = 0;

double mx1 = 0;
double iop = 0;
double st = 0;
double pld = 0;
double cen = 0;
double gear_proc = 0;
double gear_proc2 = 0;
double gear_procl = 0;
double power_proc = 0;
double fuel_proc = 0;
double speedbrake = 0;
//double swingwing = 0;
//double swing = 0;
//double swing3 = 0;
//double hook = 0;
//double canopy = 0;
double flaps = 0;
double flaps2 = 0;
double exproc = 0;
double Lights = 0;
double switched = 0;
double aoa;
double dil = 0;
double eng_pitch = 22000;

double tpp = 0;


double start = 0;
double full = 0.001;
double i = 0;
double i1 = 0;
double i2 = 0;
double i3 = 0;
double i4 = 0;
double i5 = 0;
double i7 = 0;
double i9 = 0;
double i1a = 0;
double i2a = 0;
double i3a = 0;
double i4a = 0;
double i5a = 0;
double s1 = 0;
double s2 = 0;
double s3 = 0;
double s4 = 0;
double s5 = 0;
double ga = 0;
double diff = 0.1;
double k3;
int k4;
char k3buffer [33];
double amet = 0;
double dee = 0;
double i13 = 0;
double i14 = 0;
double i15 = 0;
double i16 = 0;
double i17 = 0;
double i18 = 0;
double i19 = 0;
double i20 = 0;
double i21 = 0;

double aoa2 = 0;

//double droptank = 0;

double i25 = 0;
double i26 = 0;


double secondtime = 0;

double refuel = 0;

UINT animactive;

UINT animactivel;

UINT animactive2;

UINT animactive3;

UINT animactive4;

UINT animactive5;

UINT animactive7;

UINT animactive9;

const VECTOR3 offset = {0,0.2,4.5};


THRUSTER_HANDLE th_main;

BEACONLIGHTSPEC beacon1;    

BEACONLIGHTSPEC beacon2;   

MESHHANDLE f150;
MESHHANDLE f151;
MESHHANDLE f152;




// 1. vertical lift component (wings and body)

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	static const double CL[nabsc] = {0.1, 0.17, 0.2, 0.2, 0.17, 0.1, 0, -0.11, -0.24, -0.38,  -0.5,  -0.5, -0.02, 0.6355,    0.63,   0.46, 0.28, 0.13, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,    0,0.002,0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = CM[idx] + (CM[idx+1]-CM[idx])*d;
	*cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.02 + oapiGetInducedDrag (*cl, 1.5, 0.6);
}


// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class ShuttlePB: public VESSEL2 {
public:
	ShuttlePB (OBJHANDLE hVessel, int flightmodel);
	//	: VESSEL2 (hVessel, flightmodel) {}
	void clbkSetClassCaps (FILEHANDLE cfg);
	
	~ShuttlePB();

	
	void DefineAnimations(void);
	void Steering();
	void Lighting();
	void clbkPostCreation (void);
    void clbkPostStep (double simt, double simdt, double mjd);
    int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
   
    UINT anim_wheel;
    UINT anim_gear;
    UINT anim_gearlever;
    UINT anim_rudder;
    UINT anim_elevator;
    UINT anim_laileron;
    UINT anim_raileron;
    UINT anim_speedbrake;
    
    UINT anim_flaps;
    //UINT anim_hook;
    
    UINT anim_power;
    UINT anim_dial44;
    
    UINT anim_wheel0;
    UINT anim_wheel1;
    UINT anim_wheel2;
    UINT anim_wheel3;
    UINT anim_wheel4;
    UINT anim_wheel5;
    UINT anim_fuel;

    UINT anim_turn;

    UINT anim_dial1;
    UINT anim_dial2;
    UINT anim_dial3;
    UINT anim_bank;
    UINT anim_clock_s;
    UINT anim_clock_m;
    UINT anim_clock_h;
    
    
     MGROUP_TRANSFORM *wheel1;

    
    bool clbkVCMouseEvent (int id, int event, VECTOR3 &p);

    bool clbkLoadVC (int id);
    
    int MySoundID;	


};

ShuttlePB::ShuttlePB (OBJHANDLE hVessel, int flightmodel)
		: VESSEL2 (hVessel, flightmodel)
{
	
	// preload meshes
	
    f150                 = oapiLoadMeshGlobal ("K33\\K-T-38-v4");
    f151                 = oapiLoadMeshGlobal ("K33\\K-T-38-v4-Cpit");
    f152                 = oapiLoadMeshGlobal ("K33\\K-T-38-v4-Cpit_n");
   

	
	DefineAnimations ();
	
}


void ShuttlePB::clbkPostCreation (void)
{

int MySoundID=ConnectToOrbiterSoundDLL3(GetHandle());

//ReplaceStockSound3(MySoundID,"Sound\\Vessel\\K-Test-Car\\K-Test-Car.wav",	REPLACE_MAIN_THRUST);

RequestLoadVesselWave3(MySoundID,ENG,"Sound\\K-T-38-v4\\K-T-38-v4-Eng.wav",BOTHVIEW_FADED_FAR);

RequestLoadVesselWave3(MySoundID,FLAPS,"Sound\\K-T-38-v4\\K-T-38-v4-Flaps.wav",BOTHVIEW_FADED_FAR);

RequestLoadVesselWave3(MySoundID,GEAR,"Sound\\K-T-38-v4\\K-T-38-v4-Gear.wav",BOTHVIEW_FADED_FAR);


SoundOptionOnOff3(MySoundID,PLAYMAINTHRUST,FALSE);

SoundOptionOnOff3(MySoundID,PLAYHOVERTHRUST,FALSE);


SoundOptionOnOff3(MySoundID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);

SoundOptionOnOff3(MySoundID,PLAYCABINAIRCONDITIONING,FALSE);

SoundOptionOnOff3(MySoundID,PLAYCABINRANDOMAMBIANCE,FALSE);

SoundOptionOnOff3(MySoundID,PLAYRADIOATC,FALSE);

SoundOptionOnOff3(MySoundID,DISPLAYTIMER,FALSE);

}


// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------

void ShuttlePB::clbkSetClassCaps (FILEHANDLE cfg)
{
	// THRUSTER_HANDLE th_main;


	// physical specs
	SetSize (8);
	SetEmptyMass (12000.0);
	SetCW (0.3, 0.3, 0.6, 0.9);
	SetWingAspect (1.7);
	SetWingEffectiveness (3.1);
	SetCrossSections (_V(5,5,15));
	SetRotDrag (_V(3.5,3.5,3.5));
	if (GetFlightModel() >= 1) {
		SetPitchMomentScale (1e-4);
		SetBankMomentScale (1e-4);
	}
	SetPMI (_V(15.5,22.1,7.7));
	SetTrimScale (0.05);
	SetCameraOffset (_V(0,1.433,2.1));
	
	SetTouchdownPoints (_V(0,-2.15,4.013654), _V(-1.518829,-2.15,-1.547508), _V(1.518829,-2.15,-1.547508));

	// **************************** aerodynamics ***********************************

	CreateAirfoil (LIFT_VERTICAL,   _V(0,0,-0.5), VLiftCoeff, 20, 270, 2.266);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,-4), HLiftCoeff, 20,  50, 1.5);





	CreateControlSurface (AIRCTRL_ELEVATOR,     7, 0.7, _V(   0,0,-17), AIRCTRL_AXIS_XPOS,anim_elevator);
	CreateControlSurface (AIRCTRL_RUDDER,       2, 0.5, _V(   0,2,-15), AIRCTRL_AXIS_YPOS,anim_rudder);
	CreateControlSurface (AIRCTRL_AILERON,      4.8, 0.2, _V( 17.8,0,-17.8), AIRCTRL_AXIS_XPOS,anim_raileron);
	CreateControlSurface (AIRCTRL_AILERON,      4.8, 0.2, _V(-17.8,0,-17.8), AIRCTRL_AXIS_XNEG,anim_laileron);
	CreateControlSurface (AIRCTRL_ELEVATORTRIM, 5, 0.1, _V(   0,0,-15), AIRCTRL_AXIS_XPOS,anim_elevator);
    CreateControlSurface (AIRCTRL_FLAP,         5, 5, _V( 0,0,-1), AIRCTRL_AXIS_XPOS,anim_flaps);
	
	CreateVariableDragElement( &speedbrake, 8.0, _V(0,0,0) );

	CreateVariableDragElement (&gear_proc2, 0.8, _V(0, -1, 0));
	
	//CreateVariableDragElement (&hook, 3.33, _V(0,1,0));


	
	// propellant resources
	PROPELLANT_HANDLE hpr = CreatePropellantResource (PB_FUELMASS);
	
	
	SURFHANDLE tex_main = oapiRegisterExhaustTexture ("K-T-38-v4-Exhaust");



	// ***************** thruster definitions *******************

	PARTICLESTREAMSPEC contrail_main = {
		0, 0.3, 165, 19, 0.019, 0.111, 1.11, 1.11, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 2,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};
	
	th_main = CreateThruster (_V(0,0,-4.3), _V(0,0,1), PB_MAXMAINTH, hpr, PB_ISP);
	
	
	
	CreateThrusterGroup (&th_main, 1, THGROUP_MAIN);
	
	
	
	AddExhaust (th_main, 4, 0.4, _V(-0.1870975,-0.4776336,-6.6), _V(0,0,-1) , tex_main);
	AddExhaust (th_main, 4, 0.4, _V(0.1870975,-0.4776336,-6.6), _V(0,0,-1) , tex_main);

    
    AddExhaustStream (th_main, _V(-0.1870975,-0.4776336,-6.87), &contrail_main);
    AddExhaustStream (th_main, _V(0.1870975,-0.4776336,-6.87), &contrail_main);
    
    
      // Strobe lights
	
	//Blue strobe
	
	static VECTOR3 beacon1pos = {0,-1.215817,2.438052};
	static VECTOR3 beacon1col = {0.043, 0.161, 0.992};
	
		beacon1.shape = (BEACONSHAPE_STAR);
		beacon1.pos = &beacon1pos;
		beacon1.col = &beacon1col;
		beacon1.size = (0.18);
		beacon1.falloff = (0.6);
		beacon1.period = (1.13);
		beacon1.duration = (0.05);
		beacon1.tofs = (0.6);
		beacon1.active = false;
		
		AddBeacon (&beacon1);
		
		//Red strobe
	
	static VECTOR3 beacon2pos = {0,0.3330124,0.2956965};
	static VECTOR3 beacon2col = {0.988, 0.133, 0.133};
	
		beacon2.shape = (BEACONSHAPE_STAR);
		beacon2.pos = &beacon2pos;
		beacon2.col = &beacon2col;
		beacon2.size = (0.18);
		beacon2.falloff = (0.6);
		beacon2.period = (1.13);
		beacon2.duration = (0.05);
		beacon2.tofs = (0.6);
		beacon2.active = false;
		
		AddBeacon (&beacon2);
		

    

    
    
    
	// visual specs
	
	AddMesh (f150);
	
	SetMeshVisibilityMode( AddMesh (f151,&offset), MESHVIS_VC );

    

}

// ==============================================================
// API callback interface
// ==============================================================

void ShuttlePB::clbkPostStep (double simt, double simdt, double mjd)

{
	
	if( oapiGetTimeAcceleration() > 10.0 ) oapiSetTimeAcceleration(10);
	
	double vsAirSpeed = GetAirspeed();
	
	

	
	
    //speed and altitude control
	
	if (GetAirspeed() >= 700) 
	{
		SetCW (999, 999, 999, 999);
		SetThrusterMax0 (th_main ,0);

	
}

if (GetAltitude() >= 17000) 
	{
		SetCW (999, 999, 999, 999);
		SetThrusterMax0 (th_main ,0);

	
}



if (GetAirspeed() < 700 && GetAltitude() < 17000 ) 
	{
		SetCW (0.3, 0.3, 0.67, 0.92);
		SetThrusterMax0 (th_main ,4.4e5);

	
}

	double vsAirSpeed2 = (vsAirSpeed/1000)*3600;
    
    //Ground Steering
    
    if (GroundContact () && vsAirSpeed > 0.6 && vsAirSpeed < 90 &&  gear_proc < 0.05) Steering();
    
    
    //aoa = GetAOA();
    
    //sprintf(oapiDebugString(),"anim %2.2f", aoa );
    
  
    
    if (  gear_proc < 0.05 ) SetTouchdownPoints (_V(0,-2.15,4.013654), _V(-1.518829,-2.15,-1.547508), _V(1.518829,-2.15,-1.547508));

    if (  gear_proc > 0.95 ) SetTouchdownPoints (_V(0,-1.179783,4.013654), _V(-1.518829,-1.179783,-1.547508), _V(1.518829,-1.179783,-1.547508));

    
    //Anims 
    
    if (iop > 1)iop=0;
    
    if (vsAirSpeed > 0.6) {
    
	double aspeed = vsAirSpeed;
    if (iop > 1)iop=0;
    if (vsAirSpeed > 30) aspeed = 30; 
    iop = iop + (aspeed /300);
     
    if (vsAirSpeed ==0) iop=0;
    
    
    if ( GroundContact() && gear_proc < 0.05) SetAnimation (anim_wheel, iop);
    

   }
	
	 // Front Gear Turn Anims1
   if (!GroundContact() && gear_proc < 1.0 && vsAirSpeed > 0.6)
   {
	   tpp = 0.5;
	   SetAnimation( anim_turn, tpp );
}

   // Front Gear Turn Anims2
   if (GroundContact() && gear_proc < 0.05 )
   {
	   
   tpp = (GetControlSurfaceLevel (AIRCTRL_RUDDER) +1) *0.5;
   if (!GroundContact())tpp = 0.5;
   SetAnimation( anim_turn, tpp );
   
}

   
   
   
// Gear animations


   if( animactive && !GroundContact()) {
		if( animactive == 1 ) {
			gear_proc += 0.00242;
			if( gear_proc > 1.0 ) {
				gear_proc = 1.0;
				animactive = 0;
			}
		}
		else  {
			gear_proc -= 0.00242;
			if( gear_proc < 0.0 ) {
				gear_proc = 0.0;
				animactive = 0;
			}
		}
		SetAnimation( anim_gear, gear_proc );
		gear_proc2 = -gear_proc + 1;
	}

	
	// Gear Lever animations


   if( animactivel && !GroundContact()) {
		if( animactivel == 1 ) {
			gear_procl += 0.00242;
			if( gear_procl > 1.0 ) {
				gear_procl = 1.0;
				animactivel = 0;
			}
		}
		else  {
			gear_procl -= 0.00242;
			if( gear_procl < 0.0 ) {
				gear_procl = 0.0;
				animactivel = 0;
			}
		}
		SetAnimation( anim_gearlever, gear_procl );
	}

	
	// Throttle animation

        power_proc = GetThrusterLevel(th_main);
	
		SetAnimation( anim_power, power_proc );
		
	
   
   // Speedbrake animations

   if( animactive2 ) {
		if( animactive2 == 1 ) {
			speedbrake += 0.006;
			if( speedbrake > 1.0 ) {
				speedbrake = 1.0;
				animactive2 = 0;
			}
		}
		else  {
			speedbrake -= 0.006;
			if( speedbrake < 0.0 ) {
				speedbrake = 0.0;
				animactive2 = 0;
			}
		}
		SetAnimation( anim_speedbrake, speedbrake );
	}

	
	
   // Flaps


   if( animactive4 ) {
		if( animactive4 == 1 ) {
			flaps += 0.006;
			if( flaps > 1.0 ) {
				flaps = 1.0;
				animactive4 = 0;
			}
		}
		else  {
			flaps -= 0.006;
			if( flaps < 0.0 ) {
				flaps = 0.0;
				animactive4 = 0;
			}
		}
		SetAnimation( anim_flaps, flaps );
	}

   SetControlSurfaceLevel(AIRCTRL_FLAP,flaps);

 

// Engine sound
	
	
	
	
	if (GetThrusterLevel (th_main) > 0) 
	{
		
	double pit = ((GetThrusterLevel (th_main)*6000));
	
	eng_pitch = 13000 + pit;
	
	if (eng_pitch < 11500 ) eng_pitch = 11500;
	
	if (eng_pitch > 22500 ) eng_pitch = 22500;
	
	//sprintf(oapiDebugString(),"anim %7.7f", eng_pitch );
		
	PlayVesselWave3(MySoundID,ENG,LOOP,255,eng_pitch);
}


    if (GetThrusterLevel (th_main) == 0) 
	{
	
	StopVesselWave3(MySoundID,ENG);
}
   
 if (Lights == 1 && switched == 1)
 {
	 beacon1.active = true;
	 beacon2.active = true;

	 switched = 0;
	 Lighting();
}

if (Lights == 0 && switched == 1)
{
	beacon1.active = false;
	beacon2.active = false;

    switched = 0;
    Lighting();
}


// Altimeter animation
   
   
    //Anims 
    
     k3 = GetAltitude();
    
     k4 = int (k3);
      
     gcvt (k3,6,k3buffer);
     
     

   
     string stringOne(k3buffer);
     
     if (k3 >= 0 )
     {

     string is1 = stringOne.substr(0 + s2 + s3 + s4 + s5, 1) ;
       
     char const *cString1 = is1.c_str();
     
     i1a = atol(cString1);
     
     i1 = i1a * 0.1;
     
 }
 
 if (k3 >= 10 ) 
 {
	 
     string is2 = stringOne.substr(0 + s3 + s4 + s5 , 1) ;
       
     char const *cString2 = is2.c_str();
     
     i2a = atol(cString2);
     
     i2 = i2a * 0.1;
      s2 = 1;
 }
 
 if (k3 >= 100 )
 {
     string is3 = stringOne.substr(0 + s4 + s5, 1) ;
       
     char const *cString3 = is3.c_str();
     
     i3a = atol(cString3);
     
     i3 = i3a * 0.1;
     
     s3 = 1;
 }
 
 if (k3 >= 1000)
 {
     string is4 = stringOne.substr(0 + s5, 1) ;
       
     char const *cString4 = is4.c_str();
     
     i4a = atol(cString4);
     
     i4 = i4a * 0.1;
     
     s4 = 1;
 }
 
 if (k3 >= 10000)
 {
     string is5 = stringOne.substr(0, 1) ;
       
     char const *cString5 = is5.c_str();
     
     i5a = atol(cString5);
     
     i5 = i5a * 0.1;
     
     s5 = 1;
}

      if(k3<10)
      {
	      s2=0;
	      i2=0;
	      s3=0;
	      i3=0;
	      s4=0;
	      i4=0;
	      s5=0;
	      i5=0;
    }
         
    if(k3<100)
      {
	     
	      s3=0;
	      i3=0;
	      s4=0;
	      i4=0;
	      s5=0;
	      i5=0;
    }
    
    if(k3<1000)
      {
	      
	      s4=0;
	      i4=0;
	      s5=0;
	      i5=0;
    }
   
    if(k3<10000)
      {
	     
	      s5=0;
	      i5=0;
	      
    }
    
    
   i=(k3 * 0.01)- dee;
   
   if (i>=1)
   {
	   dee = dee + 1;
   }
   
   if (i <= 0)
   {
	   dee = dee - 1;
	   
	   if (dee < 0)dee = 0;
   }
   
    
	//sprintf(oapiDebugString(),"force %2.2f", i1 );
	
	
	SetAnimation (anim_wheel0, i);
	SetAnimation (anim_wheel1, i1);
	SetAnimation (anim_wheel2, i2);
	SetAnimation (anim_wheel3, i3);
	SetAnimation (anim_wheel4, i4);
	SetAnimation (anim_wheel5, i5);
	
	


// AOA Dial Animation

    aoa = GetAOA();
    
   if (vsAirSpeed > 0.6) {
   i7 = aoa * 6;

   SetAnimation (anim_dial1, i7);
   
}

if (vsAirSpeed <= 0.6) {
	i7 = 0;

   SetAnimation (anim_dial1, i7);
   
}

   // Direction Dial Animation

   oapiGetFocusHeading(&dil);
    
   i9 = (dil / 6.25) + 0.001;
   
   //sprintf(oapiDebugString(),"hed %2.2f", dil );

   SetAnimation (anim_dial2, i9);
   
   
   // Speed Dial Animation
    
   i13 = GetAirspeed();
   
   if (i13 > 600) i13 = 600;
   
   if (i13 < 0) i13 = 0;
   
   i14 = (-i13 + 600)/600;
   
   if (i14 < 0) i14 = 0;
   
   if (i14 > 1) i14 = 1;
   
   SetAnimation (anim_dial3, i14);
   
    // Bank Guage Animation
    
   i15 = GetBank();
   
   i16 = (i15 + 3) / 6;
   
   if (i16 < 0) i16 = 0;
   
   if (i16 > 1) i16 = 1;
   
   SetAnimation (anim_bank, i16);
   
 
    
    
     // Clock Dial Animation
     
     //Second Hand
     
    
   i17 = i17 + ( 0.0002 * oapiGetTimeAcceleration() );
   
   if (i17 < 0) i17 = 0;
   
   if (i17 >= 1) 
   {
	   i17 = 0;
	   i18 = i18 + 0.016666666;
}
   SetAnimation (anim_clock_s, i17);
   
  // sprintf(oapiDebugString(),"time %19.19f", i17 );
   
    //Minute Hand
    
   if (i18 < 0) i18 = 0;
   
   if (i18 >= 1) 
   {
	   i18 = 0;
	   i19 = i19 + 0.016666666;
   }
   
   
   SetAnimation (anim_clock_m, i18);
   
    //Hour Hand
   
   if (i19 < 0) i19 = 0;
   
   if (i19 >= 1) i19 = 0;
   
   SetAnimation (anim_clock_h, i19);
   
   
   
    
	// Fuel Guage Animation
   
   fuel_proc = (-GetFuelMass() + 20000)/20000; 

   i21 = fuel_proc / 3;
   
   if (i21 < 0) i21 = 0;
   
   if (i21 > 1) i21 = 1;
   
   SetAnimation (anim_fuel, i21);
   


   // Refuel
   
   if (refuel == 1)
   {
	   
	  SetFuelMass (20000.0);

	   refuel = 0;
   }
 

   // Head Gyroscopic and G-Force movement
   
    if (vsAirSpeed > 0.6) {
		aoa2 = (GetAOA()* 1.2);
		
		if (aoa2 > 0.333) aoa2 = 0.333;
		
		if (aoa2 < -0.333) aoa2 = -0.333;
		
		//SetCameraOffset (_V(0,1.433,2.1));
		SetCameraOffset (_V(0,1.433 + aoa2,2.1));
		
}
   
if (vsAirSpeed <= 0.6) {
	aoa2 = 0;
			
		//SetCameraOffset (_V(0,1.433,2.1));
		SetCameraOffset (_V(0,1.433 + aoa2,2.1));
	}


}

//Ground Steering


void ShuttlePB::Steering()
 {

	double pos = GetControlSurfaceLevel(AIRCTRL_RUDDER);
	double lev = 3e6 * pos ;

	if( oapiGetTimeAcceleration() > 10.0 )
		return;

	AddForce( _V( lev,0,0 ), _V( 0,0,3.33 ) );
	AddForce( _V( -1.0*lev,0,0 ), _V( 0,0,-3.33 ) );
	
}

// Lighting


void ShuttlePB::Lighting()
 {
    if(Lights == 1)
    {
	
	SetMeshVisibilityMode( InsertMesh (f152,1,&offset), MESHVIS_VC );
	

}

    if(Lights == 0)
    {
	
	SetMeshVisibilityMode( InsertMesh (f151,1,&offset), MESHVIS_VC );
	

}

}


// --------------------------------------------------------------
// Keyboard interface handler (buffered key events)
// --------------------------------------------------------------
int ShuttlePB::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate)
{
	if (!down) return 0; // only process keydown events

	// if (KEYMOD_SHIFT (kstate)) {

		switch (key) {
		case OAPI_KEY_G: // Gear
		
		if ( GroundContact() ) return 1;
		
		if( gear_proc > 0.5 )
					animactive = 2;
				else
					animactive = 1;

		if ( !GroundContact()) PlayVesselWave3(MySoundID,GEAR,NOLOOP,255);
		
		
		if( gear_procl > 0.5 )
					animactivel = 2;
				else
					animactivel = 1;

			
			return 1;
				
			case OAPI_KEY_B: // Speedbrake
		
		if( speedbrake > 0.5 )
					animactive2 = 2;
				else
					animactive2 = 1;

		PlayVesselWave3(MySoundID,FLAPS,NOLOOP,255);
		
			
			return 1;

			
			case OAPI_KEY_F: // Flaps
			
			
		if( flaps > 0.5 )
					animactive4 = 2;
				else
					animactive4 = 1;

		PlayVesselWave3(MySoundID,FLAPS,NOLOOP,255);
		
			
			return 1;
			
			
			case OAPI_KEY_J: // Refuel
			
			
		if (refuel == 0 && GetAirspeed() == 0)
		{
			refuel = 1;
		}
		
			
			return 1;
		
		

			case OAPI_KEY_K: // Lights
		
		if( Lights == 0 )
		{
			Lights = 1;
			switched =1;
					
			return 1;
}

		if( Lights == 1 )
		{
			Lights = 0;
			switched = 1;
					
			return 1;
}
	
    } 
	return 0;
}


// --------------------------------------------------------------
// Destructor
// --------------------------------------------------------------
ShuttlePB::~ShuttlePB ()
{
	delete wheel1;
	
}



void ShuttlePB::DefineAnimations(void)
{
	
	ANIMATIONCOMPONENT_HANDLE parent1;

	
   // Front Wheel
   static UINT WGrp1[1] = {187};
   
   wheel1 = new MGROUP_ROTATE  (0,WGrp1, 1, _V(0,-1.929798,4.011903), _V(1,0,0), (float)(360*RAD));
   
   
    //Turn Front Gear
   static UINT Trn1[4] = {8,10,11,12};
   
   static MGROUP_ROTATE turn1 (0,Trn1, 4, _V(0,-1.929798,4.009077), _V(0,1,0), (float)(90*RAD));
   

   
   //Rear left Wheel
   static UINT WGrp3[1] = {188};
   
   static MGROUP_ROTATE wheel2 (0,WGrp3, 1, _V(0,-1.89037,-1.545275), _V(1,0,0), (float)(360*RAD));
   
   //Rear Right wheel
   static UINT WGrp4[1] = {189};
   
   static MGROUP_ROTATE wheel3 (0,WGrp4, 1, _V(0,-1.89037,-1.545275), _V(1,0,0), (float)(360*RAD));
   
   anim_turn = CreateAnimation (0.5);
   anim_wheel = CreateAnimation (0);
   
   parent1 = AddAnimationComponent  (anim_turn, 0, 1, &turn1);
   AddAnimationComponent  (anim_wheel, 0, 1, wheel1, parent1);
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel2);
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel3);
   
   
   
   
   // Gear anims
  
   
   //Front Gear
   static UINT GGrp1[6] = {8,9,10,11,12,187};
   
   static MGROUP_ROTATE gear1 (0,GGrp1, 6, _V(0,-1.118236,3.964859), _V(1,0,0), (float)(-111*RAD));
   
   
   //Rear Gear Left
   static UINT GGrp2[3] = {16,17,189};
   
   static MGROUP_ROTATE gear2 (0,GGrp2, 3, _V(-1.470682,-0.9093096,-1.54335), _V(0,0,1), (float)(91*RAD));
   
   //Rear Gear Right
   static UINT GGrp3[3] = {14,15,188};
   
   static MGROUP_ROTATE gear3 (0,GGrp3, 3, _V(1.470682,-0.9093096,-1.54335), _V(0,0,1), (float)(-91*RAD));
   
    //Front Gear Cover mid
   static UINT GGrp4[1] = {119};
   
   static MGROUP_ROTATE gear4 (0,GGrp4, 1, _V(0,-1.094748,3.949046), _V(1,0,0), (float)(-123*RAD));
   
   
   //Front Gear Cover Right
   static UINT GGrp5[1] = {120};
   
   static MGROUP_ROTATE gear5 (0,GGrp5, 1, _V(0.1278255,-1.089871,4.75985), _V(0,0,1), (float)(-90*RAD));
   
   
   //Rear Cover Left1
   static UINT GGrp12[2] = {132,133};
   
   static MGROUP_ROTATE gear12 (0,GGrp12, 2, _V(-1.470682,-0.9093096,-1.54335), _V(0,0,1), (float)(99*RAD));
   
   //Rear Cover Right1
   static UINT GGrp13[2] = {130,131};
   
   static MGROUP_ROTATE gear13 (0,GGrp13, 2, _V(1.470682,-0.9093096,-1.54335), _V(0,0,1), (float)(-99*RAD));
   
   //Rear Cover Left2
   static UINT GGrp14[1] = {121};
   
   static MGROUP_ROTATE gear14 (0,GGrp14, 1, _V(-0.182842,-0.9707013,-1.510372), _V(0,0,1), (float)(-90*RAD));
   
   //Rear Cover Right2
   static UINT GGrp15[1] = {122};
   
   static MGROUP_ROTATE gear15 (0,GGrp15, 1, _V(0.182842,-0.9707013,-1.510372), _V(0,0,1), (float)(90*RAD));
   
  
    //Rear Gear Left2
   static UINT GGrp22[2] = {17,189};
   
   static MGROUP_ROTATE gear22 (0,GGrp22, 2, _V(-1.470682,-0.9093096,-1.54335), _V(0,0,1), (float)(4.35*RAD));
   
   //Rear Gear Right2
   static UINT GGrp23[2] = {15,188};
   
   static MGROUP_ROTATE gear23 (0,GGrp23, 2, _V(1.470682,-0.9093096,-1.54335), _V(0,0,1), (float)(-4.35*RAD));
   
    //Rear Gear Left3
   static UINT GGrp24[1] = {18};
   
   static MGROUP_ROTATE gear24 (0,GGrp24, 1, _V(-0.6571674,-0.92709,-1.54335), _V(0,0,1), (float)(-45*RAD));
   
   //Rear Gear Right2
   static UINT GGrp25[1] = {13};
   
   static MGROUP_ROTATE gear25 (0,GGrp25, 1, _V(0.6571674,-0.92709,-1.54335), _V(0,0,1), (float)(45*RAD));
   
    //Rear Gear Left4
   static UINT GGrp26[1] = {18};
   
   static MGROUP_TRANSLATE gear26 (0,GGrp26, 1, _V(0.3,0,0));
   
   //Rear Gear Right2
   static UINT GGrp27[1] = {13};
   
   static MGROUP_TRANSLATE gear27 (0,GGrp27, 1, _V(-0.3,0,0));
   
   
   
   
   anim_gear = CreateAnimation (0);
   
   AddAnimationComponent  (anim_gear, 0, 0.5, &gear1);
   AddAnimationComponent  (anim_gear, 0, 0.4, &gear2);
   AddAnimationComponent  (anim_gear, 0, 0.4, &gear3);
   AddAnimationComponent  (anim_gear, 0.1, 0.5, &gear4);
  
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear5);
   
   
   
   AddAnimationComponent  (anim_gear, 0.1, 0.5, &gear12);
   AddAnimationComponent  (anim_gear, 0.1, 0.5, &gear13);
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear14);
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear15);
   
   AddAnimationComponent  (anim_gear, 0.4, 0.5, &gear22);
   AddAnimationComponent  (anim_gear, 0.4, 0.5, &gear23);
   
   AddAnimationComponent  (anim_gear, 0.2, 0.4, &gear24);
   AddAnimationComponent  (anim_gear, 0.2, 0.4, &gear25);
   AddAnimationComponent  (anim_gear, 0, 0.2, &gear26);
   AddAnimationComponent  (anim_gear, 0, 0.2, &gear27);
   
   
   //Rudder
   static UINT RGrp1[2] = {182,185};
   
   static MGROUP_ROTATE rudderl (0,RGrp1, 2, _V(0,0.8505662,-5.413699), _V(0,1,0), (float)(-20*RAD));
   
   //Rudder Pedals left
   static UINT RGrp3[2] = {170,171};
   
   static MGROUP_ROTATE rudderlp (1,RGrp3, 2, _V(0,-0.9629048,3.608708), _V(1,0,0), (float)(16*RAD));
   
   
   
   
   //Rudder Padals right
   static UINT RGrp4[2] = {172,173};
   
   static MGROUP_ROTATE rudderrp (1,RGrp4, 2, _V(0,-0.9629048,3.608708), _V(1,0,0), (float)(-16*RAD));
   
   
   
   anim_rudder = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderl);
   
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderlp);
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderrp);
   
   
   //Elevator
   static UINT EGrp1[2] = {191,206};
   
   static MGROUP_ROTATE elevator (0,EGrp1, 2, _V(0,-0.8380946,-5.047291), _V(1,0,0), (float)(25*RAD));
   
   
   anim_elevator = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_elevator, 0, 1, &elevator);
   
   
   
   
   //Left Aleron
   static UINT LAGrp1[2] = {199,200};
   
   static MGROUP_ROTATE laileron (0,LAGrp1, 2, _V(-2.49599,-0.8865513,-2.35824), _V(1,0,-0.1), (float)(-33*RAD));
   
   
   anim_laileron = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_laileron, 0, 1, &laileron);
   
   
   //Right Aleron
   static UINT RAGrp1[2] = {203,204};
   
   static MGROUP_ROTATE raileron (0,RAGrp1, 2, _V(2.49599,-0.8865513,-2.35824), _V(1,0,0.1), (float)(33*RAD));
   
   
   anim_raileron = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_raileron, 0, 1, &raileron);
   
   
  //Speedbrake
   static UINT SBGrp1[6] = {19,20,21,22,23,24};
   
   static MGROUP_ROTATE speedbrake1 (0,SBGrp1, 6, _V(0,-1.020337,-0.635115), _V(1,0,0), (float)(-45*RAD));
   
   //Brake Lever
   static UINT SBGrp2[3] = {2,3,4};
   
   static MGROUP_ROTATE speedbrake2 (1,SBGrp2, 3, _V(0,-1.058364,2.747205), _V(1,0,0), (float)(-45*RAD));
   
   
   
   
   anim_speedbrake = CreateAnimation (0);
   
   AddAnimationComponent  (anim_speedbrake, 0, 1, &speedbrake1);
   AddAnimationComponent  (anim_speedbrake, 0, 1, &speedbrake2);
   
   
  
   
   
   //Flaps animation
   
   //Left Flap
   static UINT FLGrp1[2] = {197,198};
   
   static MGROUP_ROTATE flapsl (0,FLGrp1, 2, _V(-1.34617,-0.88278,-2.294914), _V(1,0,0), (float)(-33*RAD));
   
   
   //Right Flap
   static UINT FRGrp1[2] = {201,202};
   
   static MGROUP_ROTATE flapsr (0,FRGrp1, 2, _V(1.34617,-0.88278,-2.294914), _V(1,0,0), (float)(-33*RAD));
   
   //Flap Dial
   static UINT FLGrp3[1] = {46};
   
   static MGROUP_ROTATE flaps3 (1,FLGrp3, 1, _V(-0.9318672,-0.750552,0), _V(0,0,1), (float)(145*RAD));
   
   //Flap Lever
   static UINT FLGrp4[2] = {48,49};
   
   static MGROUP_ROTATE flaps4 (1,FLGrp4, 2, _V(0,-1.058364,2.747205), _V(1,0,0), (float)(-95*RAD));
   
   
   anim_flaps = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_flaps, 0, 1, &flapsl);
   AddAnimationComponent  (anim_flaps, 0, 1, &flapsr);
   AddAnimationComponent  (anim_flaps, 0, 1, &flaps3);
   AddAnimationComponent  (anim_flaps, 0, 1, &flaps4);
   
   
   // Gear Lever Animation
   
   //Gear Lever in Cockpit
   static UINT Glev0[2] = {57,58};
   
   static MGROUP_ROTATE gearlev (1,Glev0, 2,  _V(0,-0.1082646,2.791955), _V(1,0,0), (float)(45*RAD));
   
   static UINT Glev2[3] = {59,60,61};
   
   static MGROUP_TRANSLATE gearlev2 (1,Glev2, 3, _V(0,0,-0.02));
   
   anim_gearlever = CreateAnimation (0);
   
   AddAnimationComponent  (anim_gearlever, 0, 1, &gearlev);
   AddAnimationComponent  (anim_gearlever, 0, 1, &gearlev2);
   
   
   // Throttle Animation
   
   static UINT Pow0[2] = {188,189};
   
   static MGROUP_TRANSLATE powg (1,Pow0, 2, _V(0,0,0.38));
   
   anim_power = CreateAnimation (0);
   
   AddAnimationComponent  (anim_power, 0, 1, &powg);
   
   
   // Altimeter Animations
   
   // Dial
   static UINT wWGrp0[1] = {6};
   
   static MGROUP_ROTATE wwheel0 (1,wWGrp0, 1, _V(-0.9318853,0.5609121,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_wheel0 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel0, 0, 1, &wwheel0);
   
   // 1st dial from Left
   static UINT wWGrp1[10] = {110,111,112,113,114,115,116,117,118,119};
   
   static MGROUP_ROTATE wwheel1 (1,wWGrp1, 10, _V(0,0.6143472,2.75), _V(1,0,0), (float)(360*RAD));
   
   
   anim_wheel5 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel5, 0, 1, &wwheel1);
   
   // 2nd dial from Left
   static UINT wWGrp2[10] = {120,121,122,123,124,125,126,127,128,129};
   
   static MGROUP_ROTATE wwheel2 (1,wWGrp2, 10, _V(0,0.6143472,2.75), _V(1,0,0), (float)(360*RAD));
   
   
   anim_wheel4 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel4, 0, 1, &wwheel2);
   
   // 3rd dial from Left
   static UINT wWGrp3[10] = {80,81,82,83,84,85,86,87,88,89};
   
   static MGROUP_ROTATE wwheel3 (1,wWGrp3, 10, _V(0,0.6143472,2.74), _V(1,0,0), (float)(360*RAD));
   
   
   anim_wheel3 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel3, 0, 1, &wwheel3);
   
   // 4th dial from Left
   static UINT wWGrp4[10] = {90,91,92,93,94,95,96,97,98,99};
   
   static MGROUP_ROTATE wwheel4 (1,wWGrp4, 10, _V(0,0.6143472,2.74), _V(1,0,0), (float)(360*RAD));
   
   
   anim_wheel2 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel2, 0, 1, &wwheel4);
   
   // 5th dial from Left
   static UINT wWGrp5[10] = {100,101,102,103,104,105,106,107,108,109};
   
   static MGROUP_ROTATE wwheel5 (1,wWGrp5, 10, _V(0,0.6143472,2.74), _V(1,0,0), (float)(360*RAD));
   
   
   anim_wheel1 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel1, 0, 1, &wwheel5);
   
   
   
   // AOA Animations
   
   // Dial1
   static UINT dGrp0[1] = {1};
   
   static MGROUP_ROTATE dial1 (1,dGrp0, 1, _V(-5.916421E-05,-1.890585,0), _V(0,0,1), (float)(311*RAD));
   
   
   anim_dial1 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_dial1, 0, 1, &dial1);
   

    // Direction Dial Animations
   
   // Dial2
   static UINT d2Grp0[1] = {134};
   
   static MGROUP_ROTATE dial2 (1,d2Grp0, 1, _V(2.603407E-03,-1.143012,0), _V(0,0,1), (float)(360*RAD));
   
   //Compass
   static UINT d2Grp2[4] = {63,64,65,66};
   
   static MGROUP_ROTATE dial22 (1,d2Grp2, 4, _V(1.159894,2.018126,0.7345342), _V(0,1,0), (float)(-360*RAD));
   
   
   anim_dial2 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_dial2, 0, 1, &dial2);
   AddAnimationComponent  (anim_dial2, 0, 1, &dial22);
   

   // Speed Dial Animations
   
   // Dial3
   static UINT d3Grp0[2] = {185,186};
   
   static MGROUP_ROTATE dial3 (1,d3Grp0, 2, _V(-0.9266605,-0.159134,0), _V(0,0,1), (float)(360*RAD));
   
   
   anim_dial3 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_dial3, 0, 1, &dial3);
   
   
   // Bank Guage Animations
   
   static UINT BAGrp0[2] = {7,191};
   
   static MGROUP_ROTATE bank (1,BAGrp0, 2, _V(1.010653,-0.3959423,0), _V(0,0,1), (float)(-360*RAD));
   
   
   
   anim_bank = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_bank, 0, 1, &bank);
   
   
   
   
   // Clock
   
   // Second Hand
   static UINT dGrp7[1] = {178};
   
   static MGROUP_ROTATE dial7 (1,dGrp7, 1, _V(1.563198,-0.4318777,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_clock_s = CreateAnimation (0);
   
   AddAnimationComponent  (anim_clock_s, 0, 1, &dial7);
   
   // Minute Hand
   static UINT dGrp8[1] = {169};
   
   static MGROUP_ROTATE dial8 (1,dGrp8, 1, _V(1.563198,-0.4318777,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_clock_m = CreateAnimation (0);
   
   AddAnimationComponent  (anim_clock_m, 0, 1, &dial8);
   
   // Hour Hand
   static UINT dGrp9[1] = {62};
   
   static MGROUP_ROTATE dial9 (1,dGrp9, 1, _V(1.563198,-0.4318777,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_clock_h = CreateAnimation (0);
   
   AddAnimationComponent  (anim_clock_h, 0, 1, &dial9);
   
   
   // Fuel Guage Animation
   
   static UINT Ful0[1] = {51};
   
   static MGROUP_ROTATE fulg (1,Ful0, 1, _V(1.028024,-1.003252,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_fuel = CreateAnimation (0);
   
   AddAnimationComponent  (anim_fuel, 0, 1, &fulg);

   

   
   
   
}
	
bool ShuttlePB::clbkVCMouseEvent (int id, int event, VECTOR3 &p)
{
	


	switch (id) {
	// panel 0 events:

	case AID_ENGINEMAIN_TOP:
	{
		//if (event & PANEL_MOUSE_LBDOWN) { // record which slider to operate
		if (GetThrusterLevel(th_main) == 0) mx1 = 0;
		mx1 = mx1 + 0.0055 ;
		//sprintf(oapiDebugString(),"mouse %19.19f", mx1 );
		if (mx1 < 0)mx1=0;
		if(mx1 >1)mx1=1;
			SetThrusterLevel(th_main ,mx1);
		
		
		return 1;
	}
	
	case AID_ENGINEMAIN_BOT:
	{
		//if (event & PANEL_MOUSE_LBDOWN) { // record which slider to operate
		if (GetThrusterLevel(th_main) == 0) mx1 = 0;
		mx1 = mx1 - 0.0055 ;
		//sprintf(oapiDebugString(),"mouse %19.19f", mx1 );
		if (mx1 < 0)mx1=0;
		if(mx1 >1)mx1=1;
			SetThrusterLevel(th_main ,mx1);
		
		
		return 1;
	}
	
	case AID_GEAR:
	{
		//if (event & PANEL_MOUSE_LBDOWN) { // record which slider to operate
	if( gear_proc > 0.5 )
					animactive = 2;
				else
					animactive = 1;

		if ( !GroundContact()) PlayVesselWave3(MySoundID,GEAR,NOLOOP,255);
		
		
		if( gear_procl > 0.5 )
					animactivel = 2;
				else
					animactivel = 1;
					
					return 1;
	}
			
	
		case AID_FLAPS:
	{
		//if (event & PANEL_MOUSE_LBDOWN) { // record which slider to operate
	if( flaps > 0.5 )
					animactive4 = 2;
				else
					animactive4 = 1;

		PlayVesselWave3(MySoundID,FLAPS,NOLOOP,255);
		
			
			return 1;
	}
	
		case AID_BRAKES:
	{
		//if (event & PANEL_MOUSE_LBDOWN) { // record which slider to operate
	if( speedbrake > 0.5 )
					animactive2 = 2;
				else
					animactive2 = 1;

		PlayVesselWave3(MySoundID,FLAPS,NOLOOP,255);
		
			
			return 1;
	}
	
	case AID_LIGHTS:
	{
		//if (event & PANEL_MOUSE_LBDOWN) { // record which slider to operate
	if( Lights == 0 )
		{
			Lights = 1;
			switched =1;
					
			return 1;
}

		if( Lights == 1 )
		{
			Lights = 0;
			switched = 1;
					
			return 1;
	}
}

    // power buttons
    
    case AID_MFD1_PWR:
     {
        
        oapiSendMFDKey(MFD_LEFT, OAPI_KEY_ESCAPE);
        
        return 1;
        } 
        
    case AID_MFD2_PWR:
     {
        
        oapiSendMFDKey(MFD_RIGHT, OAPI_KEY_ESCAPE);
        
        return 1;
        } 
        
    
        
        
        // Mode Select buttons
    
    case AID_MFD1_SEL:
     {
        
        oapiSendMFDKey(MFD_LEFT, OAPI_KEY_F1);
        
        return 1;
        } 
        
    case AID_MFD2_SEL:
     {
        
        oapiSendMFDKey(MFD_RIGHT, OAPI_KEY_F1);
        
        return 1;
        } 
        
   
        // Menu buttons
    
    case AID_MFD1_MEN:
     {
        
        oapiSendMFDKey(MFD_LEFT, OAPI_KEY_GRAVE);
        
        return 1;
        } 
        
    case AID_MFD2_MEN:
     {
        
        oapiSendMFDKey(MFD_RIGHT, OAPI_KEY_GRAVE);
        
        return 1;
        } 
        
    
        
         // MFD_LEFT buttons
    
    case AID_MFD1_BT1:
     {
        
        oapiProcessMFDButton (MFD_LEFT, 0, event);

        
        return 1;
        } 
        
    case AID_MFD1_BT2:
     {
        
       oapiProcessMFDButton (MFD_LEFT, 1, event);

        
        return 1;
        } 
        
    case AID_MFD1_BT3:
    {

        oapiProcessMFDButton (MFD_LEFT, 2, event);

        
        return 1;
        } 
        
	 case AID_MFD1_BT4:
     {
        
        oapiProcessMFDButton (MFD_LEFT, 3, event);

        
        return 1;
        } 
        
    case AID_MFD1_BT5:
     {
        
       oapiProcessMFDButton (MFD_LEFT, 4, event);

        
        return 1;
        } 
        
    case AID_MFD1_BT6:
    {

        oapiProcessMFDButton (MFD_LEFT, 5, event);

        
        return 1;
        } 
        
        case AID_MFD1_BT7:
     {
        
        oapiProcessMFDButton (MFD_LEFT, 6, event);

        
        return 1;
        } 
        
    case AID_MFD1_BT8:
     {
        
       oapiProcessMFDButton (MFD_LEFT, 7, event);

        
        return 1;
        } 
        
    case AID_MFD1_BT9:
    {

        oapiProcessMFDButton (MFD_LEFT, 8, event);

        
        return 1;
        } 
        
	 case AID_MFD1_BT10:
     {
        
        oapiProcessMFDButton (MFD_LEFT, 9, event);

        
        return 1;
        } 
        
    case AID_MFD1_BT11:
     {
        
       oapiProcessMFDButton (MFD_LEFT, 10, event);

        
        return 1;
        } 
        
    case AID_MFD1_BT12:
    {

        oapiProcessMFDButton (MFD_LEFT, 11, event);

        
        return 1;
        } 
	
         // MFD_RIGHT buttons
    
    case AID_MFD2_BT1:
     {
        
        oapiProcessMFDButton (MFD_RIGHT, 0, event);

        
        return 1;
        } 
        
    case AID_MFD2_BT2:
     {
        
       oapiProcessMFDButton (MFD_RIGHT, 1, event);

        
        return 1;
        } 
        
    case AID_MFD2_BT3:
    {

        oapiProcessMFDButton (MFD_RIGHT, 2, event);

        
        return 1;
        } 
        
	 case AID_MFD2_BT4:
     {
        
        oapiProcessMFDButton (MFD_RIGHT, 3, event);

        
        return 1;
        } 
        
    case AID_MFD2_BT5:
     {
        
       oapiProcessMFDButton (MFD_RIGHT, 4, event);

        
        return 1;
        } 
        
    case AID_MFD2_BT6:
    {

        oapiProcessMFDButton (MFD_RIGHT, 5, event);

        
        return 1;
        } 
        
    case AID_MFD2_BT7:
     {
        
        oapiProcessMFDButton (MFD_RIGHT, 6, event);

        
        return 1;
        } 
        
    case AID_MFD2_BT8:
     {
        
       oapiProcessMFDButton (MFD_RIGHT, 7, event);

        
        return 1;
        } 
        
    case AID_MFD2_BT9:
    {

        oapiProcessMFDButton (MFD_RIGHT, 8, event);

        
        return 1;
        } 
        
	 case AID_MFD2_BT10:
     {
        
        oapiProcessMFDButton (MFD_RIGHT, 9, event);

        
        return 1;
        } 
        
    case AID_MFD2_BT11:
     {
        
       oapiProcessMFDButton (MFD_RIGHT, 10, event);

        
        return 1;
        } 
        
    case AID_MFD2_BT12:
    {

        oapiProcessMFDButton (MFD_RIGHT, 11, event);

        
        return 1;
        } 
        
        
       
        
	
}

return 0;

}


bool ShuttlePB::clbkLoadVC (int id)
{
	
	SetCameraOffset (_V(0,1.433,2.1));
	
    SetCameraDefaultDirection (_V(0,0,1));

    SetCameraRotationRange (RAD*70, RAD*70, RAD*45, RAD*45);
	
	static VCHUDSPEC hud_pilot  = {1, 199,{0,1.4,4.1},0.8};
	static VCMFDSPEC mfds_left = {1, 196};
    static VCMFDSPEC mfds_right = {1, 197};
   
	
    oapiVCRegisterHUD (&hud_pilot);
    oapiVCRegisterMFD (MFD_LEFT, &mfds_left);
    oapiVCRegisterMFD (MFD_RIGHT, &mfds_right);
   
    
    
    // Mouse Event
    
    
    // Throttle lever Top animations
		oapiVCRegisterArea (AID_ENGINEMAIN_TOP, PANEL_REDRAW_ALWAYS, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_LBPRESSED);
		oapiVCSetAreaClickmode_Spherical (AID_ENGINEMAIN_TOP, _V(-1.701009,-1.744822,0.3680018)+offset, 0.25);
		
    // Throttle lever Bot animations
		oapiVCRegisterArea (AID_ENGINEMAIN_BOT, PANEL_REDRAW_ALWAYS, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_LBPRESSED);
		oapiVCSetAreaClickmode_Spherical (AID_ENGINEMAIN_BOT, _V(-1.701009,-1.744822,-0.3068204)+offset, 0.25);
		
		
		// Gear lever animations
		oapiVCRegisterArea (AID_GEAR, PANEL_REDRAW_ALWAYS, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_LBPRESSED);
		oapiVCSetAreaClickmode_Spherical (AID_GEAR, _V(-1.595257,-0.1138984,2.696076)+offset, 0.25);
		
		
		// Flaps lever animations
		oapiVCRegisterArea (AID_FLAPS, PANEL_REDRAW_ALWAYS, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_LBPRESSED);
		oapiVCSetAreaClickmode_Spherical (AID_FLAPS, _V(-1.561316,-1.000344,2.696076)+offset, 0.2);
		
		
		// AirBrakes lever animations
		oapiVCRegisterArea (AID_BRAKES, PANEL_REDRAW_ALWAYS, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_LBPRESSED);
		oapiVCSetAreaClickmode_Spherical (AID_BRAKES, _V(1.754886,-1.029909,2.696076)+offset, 0.2);
		
		// Lights Button
		oapiVCRegisterArea (AID_LIGHTS, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
		oapiVCSetAreaClickmode_Spherical (AID_LIGHTS, _V(-1.521535,0.5728442,2.696076)+offset, 0.04);
		
		
		//MFD Power Buttons
		const double powerButtonRadius = 0.04; // radius of power button on each MFD1
		const double powerButtonRadius2 = 0.027; // radius of power button on each MFD2
		
	oapiVCRegisterArea (AID_MFD1_PWR, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD2_PWR, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	
	oapiVCSetAreaClickmode_Spherical(AID_MFD1_PWR, _V(-0.1268893, -0.5734969, 2.6)+offset, powerButtonRadius); 
	oapiVCSetAreaClickmode_Spherical(AID_MFD2_PWR, _V(0.9234712, -1.324777E-02,2.6)+offset, powerButtonRadius2);  
    
		//MFD Mode Select Buttons
		
	oapiVCRegisterArea (AID_MFD1_SEL, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD2_SEL, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	
	oapiVCSetAreaClickmode_Spherical(AID_MFD1_SEL, _V(4.668726E-04, -0.5734969, 2.6)+offset, powerButtonRadius);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD2_SEL, _V(1.017748, -1.159377E-02, 2.6)+offset, powerButtonRadius2);  
    
    
    
    //MFD Menu Buttons
		
	oapiVCRegisterArea (AID_MFD1_MEN, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD2_MEN, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	
	oapiVCSetAreaClickmode_Spherical(AID_MFD1_MEN, _V(0.1377469, -0.5734969, 2.6)+offset, powerButtonRadius);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD2_MEN, _V(1.103755, -1.159377E-02, 2.6)+offset, powerButtonRadius2);  
    
	
    //MFD_LEFT Buttons
		
	oapiVCRegisterArea (AID_MFD1_BT1, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD1_BT2, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD1_BT3, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
    oapiVCRegisterArea (AID_MFD1_BT4, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD1_BT5, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD1_BT6, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
    oapiVCRegisterArea (AID_MFD1_BT7, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD1_BT8, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD1_BT9, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
    oapiVCRegisterArea (AID_MFD1_BT10, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD1_BT11, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD1_BT12, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);

	oapiVCSetAreaClickmode_Spherical(AID_MFD1_BT1, _V(-0.4554956, 0.1192771, 2.6)+offset, powerButtonRadius);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD1_BT2, _V(-0.4554956, 8.460711E-03, 2.6)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD1_BT3, _V(-0.4554956, -0.1106255, 2.6)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD1_BT4, _V(-0.4554956, -0.2230959, 2.6)+offset, powerButtonRadius);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD1_BT5, _V(-0.4554956, -0.3388742, 2.6)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD1_BT6, _V(-0.4554956, -0.4480366, 2.6)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD1_BT7, _V(0.4326894, 0.1192771, 2.6)+offset, powerButtonRadius);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD1_BT8, _V(0.4326894, 8.460711E-03, 2.6)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD1_BT9, _V(0.4326894, -0.1106255, 2.6)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD1_BT10, _V(0.4326894, -0.2230959, 2.6)+offset, powerButtonRadius);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD1_BT11, _V(0.4326894, -0.3388742, 2.6)+offset, powerButtonRadius);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD1_BT12, _V(0.4326894, -0.4480366, 2.6)+offset, powerButtonRadius);  
   
    
    //MFD_RIGHT Buttons
		
	oapiVCRegisterArea (AID_MFD2_BT1, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD2_BT2, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD2_BT3, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
    oapiVCRegisterArea (AID_MFD2_BT4, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD2_BT5, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD2_BT6, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
    oapiVCRegisterArea (AID_MFD2_BT7, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD2_BT8, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD2_BT9, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
    oapiVCRegisterArea (AID_MFD2_BT10, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD2_BT11, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
	oapiVCRegisterArea (AID_MFD2_BT12, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);

	oapiVCSetAreaClickmode_Spherical(AID_MFD2_BT1, _V(0.6308033, 0.545, 2.6)+offset, powerButtonRadius2);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD2_BT2, _V(0.6308033, 0.4562395, 2.6)+offset, powerButtonRadius2);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD2_BT3, _V(0.6308033, 0.3636169, 2.6)+offset, powerButtonRadius2);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD2_BT4, _V(0.6308033, 0.2759562, 2.6)+offset, powerButtonRadius2);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD2_BT5, _V(0.6308033, 0.1882955, 2.6)+offset, powerButtonRadius2);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD2_BT6, _V(0.6308033, 0.1006347, 2.6)+offset, powerButtonRadius2);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD2_BT7, _V(1.380054, 0.545, 2.6)+offset, powerButtonRadius2);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD2_BT8, _V(1.380054, 0.4562395, 2.6)+offset, powerButtonRadius2);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD2_BT9, _V(1.380054, 0.3636169, 2.6)+offset, powerButtonRadius2);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD2_BT10, _V(1.380054, 0.2759562, 2.6)+offset, powerButtonRadius2);  
	oapiVCSetAreaClickmode_Spherical(AID_MFD2_BT11, _V(1.380054, 0.1882955, 2.6)+offset, powerButtonRadius2);  
    oapiVCSetAreaClickmode_Spherical(AID_MFD2_BT12, _V(1.380054, 0.1006347, 2.6)+offset, powerButtonRadius2);  
   
    
    
    
    
    // End
		
	
	
 return 1;

}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new ShuttlePB (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (ShuttlePB*)vessel;
}
