// ==============================================================
//                 ORBITER MODULE: ShuttlePB
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// ShuttlePB.cpp
// Control module for ShuttlePB vessel class
//
// Notes:
// This is an example for a "minimal" vessel implementation which
// only overloads the clbkSetClassCaps method to define vessel
// capabilities and otherwise uses the default VESSEL class
// behaviour.
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#define ENG 				1	
#define GEAR				2	
#define FLAPS               3

#include <orbitersdk.h>
#include <cstdio>
#include <cmath>


#include <string>
#include "OrbiterSoundSDK35.h"

#include <stdio.h>
#include <stdlib.h>

using namespace std;


// ==============================================================
// Some vessel parameters
// ==============================================================
const double PB_FUELMASS = 20000.0;

const double PB_ISP = 50000.0;
const double PB_MAXMAINTH =0;

double main_level = 0;

double t0;

double tvec = 0;
double tvec2 = 0;

double mx1 = 0;
double iop = 0;
double st = 0;
double pld = 0;
double cen = 0;
double gear_proc = 0;
double gear_proc2 = 0;
double gear_procl = 0;
double power_proc = 0;
double fuel_proc = 0;
double speedbrake = 0;
//double swingwing = 0;
//double swing = 0;
double stick3 = 0;
double stick4 = 0;


double eng_turn = 0;

double flaps = 0;
double flaps2 = 0;
double exproc = 0;
double Lights = 0;
double switched = 0;
double aoa;
double dil = 0;
double eng_pitch = 22000;

double tpp = 0;


double start = 0;
double full = 0.001;
double i = 0;
double i1 = 0;
double i2 = 0;
double i3 = 0;
double i4 = 0;
double i5 = 0;
double i7 = 0;
double i9 = 0;
double i1a = 0;
double i2a = 0;
double i3a = 0;
double i4a = 0;
double i5a = 0;
double s1 = 0;
double s2 = 0;
double s3 = 0;
double s4 = 0;
double s5 = 0;
double ga = 0;
double diff = 0.1;
double k3;
int k4;
char k3buffer [33];
double amet = 0;
double dee = 0;
double i13 = 0;
double i14 = 0;
double i15 = 0;
double i16 = 0;
double i17 = 0;
double i18 = 0;
double i19 = 0;
double i20 = 0;
double i21 = 0;

double aoa2 = 0;

//double droptank = 0;

double i25 = 0;
double i26 = 0;

double Engine_Start = 0;
double prop_speed = 0;


double secondtime = 0;

double refuel = 0;

UINT animactive;

UINT animactivel;

UINT animactive2;

UINT animactive3;

UINT animactive4;

UINT animactive5;

UINT animactive7;

UINT animactive9;

UINT animactive10;

const VECTOR3 offset = {0,0.2,4.5};

const VECTOR3 offsetpropl = {-6.66107,-0.5,1.7};

const VECTOR3 offsetpropr = {6.66107,-0.5,1.7};

THRUSTER_HANDLE thg_main;
//THRUSTER_HANDLE thg_hover;

//THGROUP_HANDLE thg_main;

BEACONLIGHTSPEC beacon1;    

BEACONLIGHTSPEC beacon2;   

MESHHANDLE f150;
MESHHANDLE f151;
MESHHANDLE f152;
MESHHANDLE f153;
MESHHANDLE f154;
MESHHANDLE f155;




// 1. vertical lift component (wings and body)

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	static const double CL[nabsc] = {0.1, 0.17, 0.2, 0.2, 0.17, 0.1, 0, -0.11, -0.24, -0.38,  -0.5,  -0.5, -0.02, 0.6355,    0.63,   0.46, 0.28, 0.13, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,    0,0.002,0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = CM[idx] + (CM[idx+1]-CM[idx])*d;
	*cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.02 + oapiGetInducedDrag (*cl, 1.5, 0.6);
}


// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class ShuttlePB: public VESSEL2 {
public:
	ShuttlePB (OBJHANDLE hVessel, int flightmodel);
	//	: VESSEL2 (hVessel, flightmodel) {}
	void clbkSetClassCaps (FILEHANDLE cfg);
	
	~ShuttlePB();

	
	void DefineAnimations(void);
	void Steering();
	void Vectrl();

	//void Lighting();
	void clbkPostCreation (void);
    void clbkPostStep (double simt, double simdt, double mjd);
    int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
   
    UINT anim_wheel;
    UINT anim_gear;
    UINT anim_gearlever;
    UINT anim_rudder;
    UINT anim_elevator;
    UINT anim_laileron;
    UINT anim_raileron;
    //UINT anim_speedbrake;
    UINT anim_engturn;
    
    UINT anim_flaps;
    //UINT anim_hook;
    
    UINT anim_power;
    UINT anim_dial44;
    
    UINT anim_wheel0;
    UINT anim_wheel1;
    UINT anim_wheel2;
    UINT anim_wheel3;
    UINT anim_wheel4;
    UINT anim_wheel5;
    UINT anim_fuel;

    UINT anim_turn;

    UINT anim_dial1;
    UINT anim_dial2;
    UINT anim_dial3;
    UINT anim_bank;
    UINT anim_clock_s;
    UINT anim_clock_m;
    UINT anim_clock_h;
    UINT anim_prop;
    UINT anim_props;
    UINT anim_propb;
     UINT anim_propbs;
    
    UINT anim_stick0;
    UINT anim_stick1;
    UINT anim_stick2;
    
    UINT anim_engturnlb;
    UINT anim_engturnrb;

    
     MGROUP_TRANSFORM *wheel1;
     MGROUP_TRANSFORM *prop1;
     MGROUP_TRANSFORM *prop3;
     MGROUP_TRANSFORM *prop1s;
     MGROUP_TRANSFORM *prop3s;
     MGROUP_TRANSFORM *prop1b;
     MGROUP_TRANSFORM *prop3b;
     MGROUP_TRANSFORM *prop1bs;
     MGROUP_TRANSFORM *prop3bs;
     MGROUP_TRANSFORM *stick0;
     MGROUP_TRANSFORM *stick1;

    
    //bool clbkVCMouseEvent (int id, int event, VECTOR3 &p);

    bool clbkLoadVC (int id);
    
    int MySoundID;	


};

ShuttlePB::ShuttlePB (OBJHANDLE hVessel, int flightmodel)
		: VESSEL2 (hVessel, flightmodel)
{
	
	// preload meshes
	
    f150                 = oapiLoadMeshGlobal ("K33\\K-V-22-Osprey");
    f151                 = oapiLoadMeshGlobal ("K33\\K-V-22-Osprey-Cpit");
    f152                 = oapiLoadMeshGlobal ("K33\\K-V-22-Osprey-Cpit_n");
    f153                 = oapiLoadMeshGlobal ("K33\\K-V-22-Nothing");
    f154                 = oapiLoadMeshGlobal ("K33\\K-V-22-Props-Blurred-L");
    f155                 = oapiLoadMeshGlobal ("K33\\K-V-22-Props-Blurred-R");


	
	DefineAnimations ();
	
}


void ShuttlePB::clbkPostCreation (void)
{

int MySoundID=ConnectToOrbiterSoundDLL3(GetHandle());

//ReplaceStockSound3(MySoundID,"Sound\\Vessel\\K-Test-Car\\K-Test-Car.wav",	REPLACE_MAIN_THRUST);

RequestLoadVesselWave3(MySoundID,ENG,"Sound\\K-V-22\\K-V-22-Eng.wav",BOTHVIEW_FADED_FAR);

RequestLoadVesselWave3(MySoundID,FLAPS,"Sound\\K-V-22\\K-V-22-Flaps.wav",BOTHVIEW_FADED_FAR);

RequestLoadVesselWave3(MySoundID,GEAR,"Sound\\K-V-22\\K-V-22-Gear.wav",BOTHVIEW_FADED_FAR);


SoundOptionOnOff3(MySoundID,PLAYMAINTHRUST,FALSE);

SoundOptionOnOff3(MySoundID,PLAYHOVERTHRUST,FALSE);


SoundOptionOnOff3(MySoundID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);

SoundOptionOnOff3(MySoundID,PLAYCABINAIRCONDITIONING,FALSE);

SoundOptionOnOff3(MySoundID,PLAYCABINRANDOMAMBIANCE,FALSE);

SoundOptionOnOff3(MySoundID,PLAYRADIOATC,FALSE);

SoundOptionOnOff3(MySoundID,DISPLAYTIMER,FALSE);

}


// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------

void ShuttlePB::clbkSetClassCaps (FILEHANDLE cfg)
{
	// THRUSTER_HANDLE th_main;


	// physical specs
	SetSize (8);
	SetEmptyMass (12000.0);
	SetCW (0.3, 0.3, 0.6, 0.9);
	SetWingAspect (1.7);
	SetWingEffectiveness (3.1);
	SetCrossSections (_V(5,5,15));
	SetRotDrag (_V(3.5,3.5,3.5));
	if (GetFlightModel() >= 1) {
		SetPitchMomentScale (1e-4);
		SetBankMomentScale (1e-4);
	}
	SetPMI (_V(15.5,22.1,7.7));
	SetTrimScale (0.05);
	SetCameraOffset (_V(0.65,1,3.7));


	
	SetTouchdownPoints (_V(0,-4.25,7.068734), _V(-2,-4.25,0.9311422), _V(2,-4.25,0.9311422));

	// **************************** aerodynamics ***********************************
	
    CreateAirfoil (LIFT_VERTICAL,   _V(0,0,-0.5), VLiftCoeff, 20, 270, 2.266);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,-4), HLiftCoeff, 20,  50, 1.5);





	CreateControlSurface (AIRCTRL_ELEVATOR,     7, 0.7, _V(   0,0,-17), AIRCTRL_AXIS_XPOS,anim_elevator);
	CreateControlSurface (AIRCTRL_RUDDER,       2, 0.5, _V(   0,2,-15), AIRCTRL_AXIS_YPOS,anim_rudder);
	CreateControlSurface (AIRCTRL_AILERON,      4.8, 0.2, _V( 17.8,0,-17.8), AIRCTRL_AXIS_XPOS,anim_raileron);
	CreateControlSurface (AIRCTRL_AILERON,      4.8, 0.2, _V(-17.8,0,-17.8), AIRCTRL_AXIS_XNEG,anim_laileron);
	CreateControlSurface (AIRCTRL_ELEVATORTRIM, 5, 0.1, _V(   0,0,-15), AIRCTRL_AXIS_XPOS,anim_elevator);
    CreateControlSurface (AIRCTRL_FLAP,         5, 5, _V( 0,0,-1), AIRCTRL_AXIS_XPOS,anim_flaps);
	
	//CreateVariableDragElement( &speedbrake, 5.0, _V(0,0,0) );

	//CreateVariableDragElement (&gear_proc2, 0.8, _V(0, -1, 0));
	
	//CreateVariableDragElement (&hook, 3.33, _V(0,1,0));


	
	// propellant resources
	PROPELLANT_HANDLE hpr = CreatePropellantResource (PB_FUELMASS);
	
	
	

	// ***************** thruster definitions *******************

	
	
	thg_main =  CreateThruster (_V(0,0,-0.01), _V(0,0,1), PB_MAXMAINTH, hpr, PB_ISP);
	//thg_hover =  CreateThruster (_V(0,0,-7.01), _V(0,0,1), PB_MAXMAINTH, hpr, PB_ISP);
	
	
	
	CreateThrusterGroup (&thg_main, 1, THGROUP_MAIN);
	//CreateThrusterGroup (&thg_hover, 1, THGROUP_HOVER);
	
	
	

      // Strobe lights
	
	//Red1 strobe
	
	static VECTOR3 beacon1pos = {-2.475028,0.8775481,-7.051601};
	static VECTOR3 beacon1col = {0.988, 0.133, 0.133};
	
		beacon1.shape = (BEACONSHAPE_STAR);
		beacon1.pos = &beacon1pos;
		beacon1.col = &beacon1col;
		beacon1.size = (0.33);
		beacon1.falloff = (0.6);
		beacon1.period = (1.13);
		beacon1.duration = (0.05);
		beacon1.tofs = (0.6);
		beacon1.active = false;
		
		AddBeacon (&beacon1);
		
		//Red2 strobe
	
	static VECTOR3 beacon2pos = {0.4108646,-3.872945,4.168705};
	static VECTOR3 beacon2col = {0.988, 0.133, 0.133};
	
		beacon2.shape = (BEACONSHAPE_STAR);
		beacon2.pos = &beacon2pos;
		beacon2.col = &beacon2col;
		beacon2.size = (0.33);
		beacon2.falloff = (0.6);
		beacon2.period = (1.13);
		beacon2.duration = (0.05);
		beacon2.tofs = (0.6);
		beacon2.active = false;
		
		AddBeacon (&beacon2);
		

    

    
    
    
	// visual specs
	
	AddMesh (f150);
	
	AddMesh (f153);
	
	SetMeshVisibilityMode( AddMesh (f151,&offset), MESHVIS_VC );
	
	AddMesh (f154, &offsetpropl);
	
	AddMesh (f155, &offsetpropr);

    

}

// ==============================================================
// API callback interface
// ==============================================================

void ShuttlePB::clbkPostStep (double simt, double simdt, double mjd)

{
	
	
	if( oapiGetTimeAcceleration() > 10.0 ) oapiSetTimeAcceleration(10);
	
	double vsAirSpeed = GetAirspeed();
	
	
    //Thrust vectoring
	
	//SetThrusterDir (thg_main ,_V(0,exproc*3,1));
	
	SetThrusterDir (thg_main ,_V(0,exproc*2,1));
	
	if (!GroundContact() && exproc ==1) AddForce( _V( 0,0,-170000 ), _V( 0,0,0 ) );
	
	if (vsAirSpeed >= 33 && exproc ==1) AddForce( _V( 0,-5000,0 ), _V( 0,0,7 ) );

	if (vsAirSpeed >= 44 && exproc ==1) AddForce( _V( 0,-5000,0 ), _V( 0,0,7 ) );
	
    //speed and altitude control
	
	if ( eng_turn < 0.5 && GetAirspeed() >= 700) 
	{
		SetCW (999, 999, 999, 999);
		SetThrusterMax0 (thg_main ,0);
		
}

if ( eng_turn >= 0.5 && GetAirspeed() >= 155) 
	{
		SetCW (999, 999, 999, 999);
		SetThrusterMax0 (thg_main ,0);
		
}

if (GetAltitude() >= 15000) 
	{
		SetCW (999, 999, 999, 999);
		SetThrusterMax0 (thg_main ,0);
			
}

if (eng_turn < 0.5 && GetAirspeed() < 700 && GetAltitude() < 15000 ) 

	{
		
		SetCW (0.3, 0.3, 0.67, 0.92);
		SetThrusterMax0 (thg_main ,4.4e5);
}


if (eng_turn >= 0.5 && GetAirspeed() < 155 && GetAltitude() < 15000)

		{
		SetCW (0.3, 0.3, 0.67, 0.92);
		SetThrusterMax0 (thg_main ,2.2e5);
}
	

	double vsAirSpeed2 = (vsAirSpeed/1000)*3600;
    
    //Ground Steering
    
    if (GroundContact () && vsAirSpeed > 0.6 && vsAirSpeed < 90 &&  gear_proc < 0.05) Steering();
    
    if (!GroundContact() ) Vectrl();

    //aoa = GetAOA();
    
    //sprintf(oapiDebugString(),"anim %2.2f", aoa );
    
  
    
    if (  gear_proc < 0.05 ) SetTouchdownPoints (_V(0,-4.25,7.068734), _V(-2,-4.25,0.9311422), _V(2,-4.25,0.9311422));


    if (  gear_proc > 0.95 ) SetTouchdownPoints (_V(0,-3.79355,7.068734), _V(-2,-3.79355,0.9311422), _V(2,-3.79355,0.9311422));

    
    //Anims 
    
    if (iop > 1)iop=0;
    
    if (vsAirSpeed > 0.6) {
    
	double aspeed = vsAirSpeed;
    if (iop > 1)iop=0;
    if (vsAirSpeed > 30) aspeed = 30; 
    iop = iop + (aspeed /300);
     
    if (vsAirSpeed ==0) iop=0;
    
    
    if ( GroundContact() && gear_proc < 0.05) SetAnimation (anim_wheel, iop);
    

   }
	
	 // Front Gear Turn Anims1
   if (!GroundContact() && gear_proc < 1.0 && vsAirSpeed > 0.6)
   {
	   tpp = 0.5;
	   SetAnimation( anim_turn, tpp );
}

   // Front Gear Turn Anims2
   if (GroundContact() && gear_proc < 0.05 )
   {
	   
   tpp = (GetControlSurfaceLevel (AIRCTRL_RUDDER) +1) *0.5;
   if (!GroundContact())tpp = 0.5;
   SetAnimation( anim_turn, tpp );
   
}

   
   // Engine Prop Animations
    
    if( GetThrusterLevel (thg_main) > 0)
    
    {
	    
	    Engine_Start = 1;
	    
	    prop_speed = prop_speed + (0.035 + (GetThrusterLevel (thg_main) * 0.1));
    
	    if (prop_speed > 1 ) prop_speed = 0;
	    
	    SetAnimation( anim_prop, prop_speed );
	    SetAnimation( anim_propb, prop_speed );
	    SetAnimation( anim_props, 1 );
	    SetAnimation( anim_propbs, 0 );
    
}
    

if( GetThrusterLevel (thg_main) == 0 && vsAirSpeed <= 10)
    
    {
	    Engine_Start = 0;
	    
	    prop_speed = prop_speed;
	    
	     SetAnimation( anim_prop, prop_speed );
	     SetAnimation( anim_propb, prop_speed );
	     SetAnimation( anim_props, 0 );
	     SetAnimation( anim_propbs, 1 );
    
    
}

if( GetThrusterLevel (thg_main) == 0 && vsAirSpeed > 10)
    
    {
	    
	    Engine_Start = 0;
	    
	    double aspeed2 = vsAirSpeed;
	    
	    if (vsAirSpeed > 44) aspeed2 = 44; 
	    
	    prop_speed = prop_speed + (0.01 + (aspeed2 * 0.001));
    
	    if (prop_speed > 1 ) prop_speed = 0;
	    
	    SetAnimation( anim_prop, prop_speed );
	    SetAnimation( anim_propb, prop_speed );
	    SetAnimation( anim_props, 1 );
	    SetAnimation( anim_propbs, 0 );
    
}




   
// Gear animations


   if( animactive && !GroundContact()) {
		if( animactive == 1 ) {
			gear_proc += 0.00242 * oapiGetTimeAcceleration();
			if( gear_proc > 1.0 ) {
				gear_proc = 1.0;
				animactive = 0;
			}
		}
		else  {
			gear_proc -= 0.00242 * oapiGetTimeAcceleration();
			if( gear_proc < 0.0 ) {
				gear_proc = 0.0;
				animactive = 0;
			}
		}
		SetAnimation( anim_gear, gear_proc );
		gear_proc2 = -gear_proc + 1;
	}

	
	

	
	// Engine Turn animations

   if(animactive10 == 0 && GroundContact() && eng_turn < 0.4) {
	   
	   eng_turn += 0.0015 * oapiGetTimeAcceleration();
	   
	   SetAnimation( anim_engturn, eng_turn );
	   SetAnimation( anim_engturnlb, eng_turn );
	   SetAnimation( anim_engturnrb, eng_turn );
	   exproc = eng_turn;
	   
   }
	
   if( animactive10 ) {
		if( animactive10 == 1 ) {
			eng_turn += 0.0015 * oapiGetTimeAcceleration();
			if( eng_turn > 1.0 ) {
				eng_turn = 1.0;
				animactive10 = 0;
			}
		}
		else  {
			eng_turn -= 0.0015 * oapiGetTimeAcceleration();
			if (GroundContact() && eng_turn < 0.4)
			{
				eng_turn = 0.4;
				animactive10 = 0;
			}
			
			if( eng_turn < 0.0 ) {
				eng_turn = 0.0;
				animactive10 = 0;
			}
		}
		SetAnimation( anim_engturn, eng_turn );
		SetAnimation( anim_engturnlb, eng_turn );
	    SetAnimation( anim_engturnrb, eng_turn );
		exproc = eng_turn;
	}
	
	
	exproc = eng_turn;
	
	
	
	
   // Flaps


   if( animactive4 ) {
		if( animactive4 == 1 ) {
			flaps += 0.006 * oapiGetTimeAcceleration();
			if( flaps > 1.0 ) {
				flaps = 1.0;
				animactive4 = 0;
			}
		}
		else  {
			flaps -= 0.006 * oapiGetTimeAcceleration();
			if( flaps < 0.0 ) {
				flaps = 0.0;
				animactive4 = 0;
			}
		}
		SetAnimation( anim_flaps, flaps );
	}

   SetControlSurfaceLevel(AIRCTRL_FLAP,flaps);

 

// Engine sound
	
	
	
	
	if (GetThrusterLevel (thg_main) > 0) 
	{
		
	double pit = ((GetThrusterLevel (thg_main)*6000));
	
	eng_pitch = 14000 + pit;
	
	if (eng_pitch < 11500 ) eng_pitch = 11500;
	
	if (eng_pitch > 22500 ) eng_pitch = 22500;
	
	//sprintf(oapiDebugString(),"anim %7.7f", eng_pitch );
		
	if ( oapiCameraInternal ())
	{
	PlayVesselWave3(MySoundID,ENG,LOOP,220,eng_pitch - 800);
}

else

	PlayVesselWave3(MySoundID,ENG,LOOP,255,eng_pitch);
	
}


    if (GetThrusterLevel (thg_main) == 0) 
	{
	
	StopVesselWave3(MySoundID,ENG);
}
   



 if (Lights == 1 && switched == 1)
 {
	 beacon1.active = true;
	 beacon2.active = true;

	 switched = 0;
	
	 SetMeshVisibilityMode( InsertMesh (f152,2,&offset), MESHVIS_VC );	
	 
}

if (Lights == 0 && switched == 1)
{
	beacon1.active = false;
	beacon2.active = false;

    switched = 0;
    
    SetMeshVisibilityMode( InsertMesh (f151,2,&offset), MESHVIS_VC );
 	
}






   
   // Speed Dial Animation
    
   i13 = GetAirspeed();
   
   if (i13 > 600) i13 = 600;
   
   if (i13 < 0) i13 = 0;
   
   i14 = (-i13 + 600)/600;
   
   if (i14 < 0) i14 = 0;
   
   if (i14 > 1) i14 = 1;
   
   SetAnimation (anim_dial3, i14);
   
    // Bank Guage Animation
    
   i15 = GetBank();
   
   i16 = (i15 + 3) / 6;
   
   if (i16 < 0) i16 = 0;
   
   if (i16 > 1) i16 = 1;
   
   SetAnimation (anim_bank, i16);
   
 
    
    
     // Clock Dial Animation
     
     //Second Hand
     
    
   i17 = i17 + ( 0.0002 * oapiGetTimeAcceleration() );
   
   if (i17 < 0) i17 = 0;
   
   if (i17 >= 1) 
   {
	   i17 = 0;
	   i18 = i18 + 0.016666666;
}
   SetAnimation (anim_clock_s, i17);
   
  // sprintf(oapiDebugString(),"time %19.19f", i17 );
   
    //Minute Hand
    
   if (i18 < 0) i18 = 0;
   
   if (i18 >= 1) 
   {
	   i18 = 0;
	   i19 = i19 + 0.016666666;
   }
   
   
   SetAnimation (anim_clock_m, i18);
   
    //Hour Hand
   
   if (i19 < 0) i19 = 0;
   
   if (i19 >= 1) i19 = 0;
   
   SetAnimation (anim_clock_h, i19);
   
   
   
    
	// Fuel Guage Animation
   
   fuel_proc = (-GetFuelMass() + 20000)/20000; 

   i21 = fuel_proc / 3;
   
   if (i21 < 0) i21 = 0;
   
   if (i21 > 1) i21 = 1;
   
   SetAnimation (anim_fuel, i21);
   


   // Refuel
   
   if (refuel == 1)
   {
	   
	  SetFuelMass (20000.0);

	   refuel = 0;
   }
 

   // Head Gyroscopic and G-Force movement
   
    if (vsAirSpeed > 0.65) {
		aoa2 = (GetAOA()* 0.5);
		
		if (aoa2 > 0.1333) aoa2 = 0.1333;
		
		if (aoa2 < -0.1333) aoa2 = -0.1333;
		
		//SetCameraOffset (_V(0.65,1,3.7));
		SetCameraOffset (_V(0.65,1 + aoa2,3.7));
		
}
   
//sprintf(oapiDebugString(),"aoa %19.19f", aoa2 );

if (vsAirSpeed <= 0.65) {
	
	aoa2 = 0;
			
	
		//SetCameraOffset (_V(0.45,1,3.7));
		SetCameraOffset (_V(0.65,1 + aoa2,3.7));
	}


	// Stick Animation
		
		
		stick3 = (GetControlSurfaceLevel (AIRCTRL_ELEVATOR) + 1) * 0.5;
		stick4 = (GetControlSurfaceLevel (AIRCTRL_AILERON) + 1) * 0.5 ;
		
		SetAnimation( anim_stick0, stick4 );
		SetAnimation( anim_stick1, stick4 );
		SetAnimation( anim_stick2, stick3 );

	
	
	
	
}

//Ground Steering


void ShuttlePB::Steering()
 {

	double pos = GetControlSurfaceLevel(AIRCTRL_RUDDER);
	double lev = 3e6 * pos ;

	if( oapiGetTimeAcceleration() > 10.0 )
		return;

	AddForce( _V( lev,0,0 ), _V( 0,0,3.33 ) );
	AddForce( _V( -1.0*lev,0,0 ), _V( 0,0,-3.33 ) );
	
}


void ShuttlePB::Vectrl()
 {
	 
	 // Rear Rotaion Thruster
	double pos2 = GetControlSurfaceLevel(AIRCTRL_RUDDER);
	double lev2 = 1e6 * pos2 * GetAirspeed() / 10.0;

	if( oapiGetTimeAcceleration() > 10.0 )
		return;

		
    if (GetAirspeed() > 155) return; 
    
    
	if (GetAirspeed() > 5 && GetAirspeed() < 33) 
	{
	AddForce( _V( lev2,0,0 ), _V( 0,0,0.05 ) );
	AddForce( _V( -1.0*lev2,0,0 ), _V( 0,0,-0.05 ) );
}

if (GetAirspeed() >= 33) 
	{
	AddForce( _V( lev2,0,0 ), _V( 0,0,0.01 ) );
	AddForce( _V( -1.0*lev2,0,0 ), _V( 0,0,-0.01 ) );
}



if (GetAirspeed() <= 5) 
	{
	AddForce( _V( lev2,0,0 ), _V( 0,0,0.1 ) );
	AddForce( _V( -1.0*lev2,0,0 ), _V( 0,0,-0.1 ) );
}

	
	 // Elevation Thruster
	 double pos3 = GetControlSurfaceLevel(AIRCTRL_AILERON);
	double lev3 = 1e6 * pos3 * GetAirspeed() / 10.0;

	if( oapiGetTimeAcceleration() > 10.0 )
		return;

    if (GetAirspeed() > 155) return; 
	
    	if (GetAirspeed() > 5 && GetAirspeed() < 33) 
	{
	AddForce( _V( 0,lev3,0 ), _V( -0.05,0,0 ) );
	AddForce( _V( 0,-1.0*lev3,0 ), _V( 0.05,0,0 ) );
}

if (GetAirspeed() >= 33) 
	{
	AddForce( _V( 0,lev3,0 ), _V( -0.01,0,0 ) );
	AddForce( _V( 0,-1.0*lev3,0 ), _V( 0.01,0,0 ) );
}

	 // Bank Thruster
	 double pos = GetControlSurfaceLevel(AIRCTRL_ELEVATOR);
	double lev = 1e6 * pos * GetAirspeed() / 10.0;

	if( oapiGetTimeAcceleration() > 10.0 )
		return;

    if (GetAirspeed() > 155) return; 
	
    if (GetAirspeed() > 5 && GetAirspeed() < 33) 
	{	
	AddForce( _V( 0,0,lev ), _V( 0,-0.05,0 ) );
	AddForce( _V( 0,0,-1.0*lev ), _V( 0,0.05,0 ) );
}

if (GetAirspeed() >= 33) 
	{	
	AddForce( _V( 0,0,lev ), _V( 0,-0.01,0 ) );
	AddForce( _V( 0,0,-1.0*lev ), _V( 0,0.01,0 ) );
}
	
}



// --------------------------------------------------------------
// Keyboard interface handler (buffered key events)
// --------------------------------------------------------------
int ShuttlePB::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate)
{
	if (!down) return 0; // only process keydown events

	// if (KEYMOD_SHIFT (kstate)) {

		switch (key) {
		case OAPI_KEY_G: // Gear
		
		if ( GroundContact() ) return 1;
		
		if( gear_proc > 0.5 )
					animactive = 2;
				else
					animactive = 1;

		if ( !GroundContact()) PlayVesselWave3(MySoundID,GEAR,NOLOOP,255);
		
		
		if( gear_procl > 0.5 )
					animactivel = 2;
				else
					animactivel = 1;

			
			return 1;
				
			case OAPI_KEY_0: // Engine Turn Up and Down
		
		if( eng_turn > 0.5 )
					animactive10 = 2;
				else
					animactive10 = 1;

		PlayVesselWave3(MySoundID,FLAPS,NOLOOP,255);
		
			
			return 1;

			case OAPI_KEY_9: // Engine Turn Up 10 deg
	
		eng_turn += 0.1;
		
			if( eng_turn > 1.0 ) {
				eng_turn = 1.0;
				}
				
	    SetAnimation( anim_engturn, eng_turn );
	    SetAnimation( anim_engturnlb, eng_turn );
	    SetAnimation( anim_engturnrb, eng_turn );
		exproc = eng_turn;

		PlayVesselWave3(MySoundID,FLAPS,NOLOOP,255);
		
			
			return 1;
			
			case OAPI_KEY_8: // Engine Turn Down 10 deg
		
		eng_turn -= 0.1;
		
			if( eng_turn < 0.0 ) {
				eng_turn = 0.0;
				}
				
	    SetAnimation( anim_engturn, eng_turn );
	    SetAnimation( anim_engturnlb, eng_turn );
	    SetAnimation( anim_engturnrb, eng_turn );
		exproc = eng_turn;

		PlayVesselWave3(MySoundID,FLAPS,NOLOOP,255);
		
			
			return 1;
			
			
			case OAPI_KEY_F: // Flaps
			
		
		if( flaps > 0.5 )
					animactive4 = 2;
				else
					animactive4 = 1;

		PlayVesselWave3(MySoundID,FLAPS,NOLOOP,255);
		
			
			return 1;
			
			
			case OAPI_KEY_J: // Refuel
			
			
		if (refuel == 0 && GetAirspeed() == 0)
		{
			refuel = 1;
		}
		
			
			return 1;
		
		

			case OAPI_KEY_K: // Lights
			
			if (GetControlSurfaceLevel (AIRCTRL_RUDDER) > 0) return 1;
			
			if (GetControlSurfaceLevel (AIRCTRL_RUDDER) < 0) return 1;
		
			if (flaps > 0.01) return 1;	
			
			
		if( Lights == 0 )
		{
			Lights = 1;
			switched =1;
					
			return 1;
}

		if( Lights == 1 )
		{
			Lights = 0;
			switched = 1;
					
			return 1;
}
	
    } 
	return 0;
}


// --------------------------------------------------------------
// Destructor
// --------------------------------------------------------------
ShuttlePB::~ShuttlePB ()
{
	delete wheel1;
	delete prop1;
	delete prop3;
	delete prop1s;
	delete prop3s;
	delete prop1b;
	delete prop3b;
	delete prop1bs;
	delete prop3bs;
	delete stick0;
	delete stick1;

}



void ShuttlePB::DefineAnimations(void)
{
	
	ANIMATIONCOMPONENT_HANDLE parent1;
	ANIMATIONCOMPONENT_HANDLE parent2;
    ANIMATIONCOMPONENT_HANDLE parent3;
    ANIMATIONCOMPONENT_HANDLE parent2lb;
    ANIMATIONCOMPONENT_HANDLE parent2rb;
    ANIMATIONCOMPONENT_HANDLE parentb;
    ANIMATIONCOMPONENT_HANDLE parentb2;
	
   // Front Wheel
   static UINT WGrp1[2] = {203,204};
   
   wheel1 = new MGROUP_ROTATE  (0,WGrp1, 2, _V(0,-3.972657,7.070827), _V(1,0,0), (float)(360*RAD));
   
   
    //Turn Front Gear
   static UINT Trn1[4] = {37,38,39,40};
   
   static MGROUP_ROTATE turn1 (0,Trn1, 4, _V(0,-3.566964,7.072397), _V(0,1,0), (float)(70*RAD));
   

   
   //Rear left Wheel
   static UINT WGrp3[2] = {205,206};
   
   static MGROUP_ROTATE wheel2 (0,WGrp3, 2, _V(0,-3.933631,0.9315668), _V(1,0,0), (float)(360*RAD));
   
   //Rear Right wheel
   static UINT WGrp4[2] = {207,208};
   
   static MGROUP_ROTATE wheel3 (0,WGrp4, 2, _V(0,-3.933631,0.9315668), _V(1,0,0), (float)(360*RAD));
   
   anim_turn = CreateAnimation (0.5);
   anim_wheel = CreateAnimation (0);
   
   parent1 = AddAnimationComponent  (anim_turn, 0, 1, &turn1);
   AddAnimationComponent  (anim_wheel, 0, 1, wheel1, parent1);
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel2);
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel3);
   
   
   
   
   // Gear anims
  
   
   //Front Gear
   static UINT GGrp1[6] = {37,38,39,40,203,204};
   
   static MGROUP_ROTATE gear1 (0,GGrp1, 6, _V(0,-3.566964,7.072397), _V(1,0,0), (float)(115*RAD));
   
   
   //Rear Gear Left
   static UINT GGrp2[6] = {41,42,43,44,205,206};
   
   static MGROUP_ROTATE gear2 (0,GGrp2, 6, _V(-1.470682,-3.433169,0.9335396), _V(1,0,0), (float)(110*RAD));
   
   //Rear Gear Right
   static UINT GGrp3[6] = {45,46,47,48,207,208};
   
   static MGROUP_ROTATE gear3 (0,GGrp3, 6, _V(1.470682,-3.433169,0.9335396), _V(1,0,0), (float)(110*RAD));
   
    
    //Front Gear Cover Left
   static UINT GGrp5[1] = {141};
   
   static MGROUP_ROTATE gear5 (0,GGrp5, 1, _V(-0.3675379,-3.57,6.08281), _V(0,0,1), (float)(93*RAD));
   
    //Front Gear Cover Right
   static UINT GGrp6[1] = {166};
   
   static MGROUP_ROTATE gear6 (0,GGrp6, 1, _V(0.3675379,-3.57,6.08281), _V(0,0,1), (float)(-93*RAD));
   
   
   //Rear Cover Left
   static UINT GGrp12[2] = {49,142};
   
   static MGROUP_ROTATE gear12 (0,GGrp12, 2, _V(-1.705111,-3.3181,1.36434), _V(0,0,1), (float)(115*RAD));
   
   //Rear Cover Right
   static UINT GGrp13[2] = {50,167};
   
   static MGROUP_ROTATE gear13 (0,GGrp13, 2, _V(1.705111,-3.3181,1.36434), _V(0,0,1), (float)(-115*RAD));
   
   //Hole Anims
   static UINT GGrp18[3] = {51,52,53};
   
   static MGROUP_TRANSLATE gear18 (0,GGrp18, 3,  _V(0,0.2,0));
   
    //Rear Gear Left2
   static UINT GGrp20[6] = {41,42,43,44,205,206};
   
   static MGROUP_ROTATE gear20 (0,GGrp20, 6, _V(-1.470682,-3.433169,0.9335396), _V(0,0,1), (float)(33*RAD));
   
   //Rear Gear Right2
   static UINT GGrp21[6] = {45,46,47,48,207,208};
   
   static MGROUP_ROTATE gear21 (0,GGrp21, 6, _V(1.470682,-3.433169,0.9335396), _V(0,0,1), (float)(-33*RAD));
   
    
   
   anim_gear = CreateAnimation (0);
   
   AddAnimationComponent  (anim_gear, 0, 0.5, &gear1);
   AddAnimationComponent  (anim_gear, 0.2, 0.5, &gear2);
   AddAnimationComponent  (anim_gear, 0.2, 0.5, &gear3);
   
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear5);
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear6);
   
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear12);
   AddAnimationComponent  (anim_gear, 0.5, 1, &gear13);
   
   AddAnimationComponent  (anim_gear, 0.4, 0.7, &gear18);
   
   AddAnimationComponent  (anim_gear, 0, 0.2, &gear20);
   AddAnimationComponent  (anim_gear, 0, 0.2, &gear21);
   
   
   //Rudder Left
   static UINT RGrp1[5] = {175,176,177,193,194};
   
   static MGROUP_ROTATE rudderl (0,RGrp1, 5, _V(-2.476552,3.198984E-03,-7.42393), _V(0,1,0), (float)(-33*RAD));
   
   //Rudder Right
   static UINT RGrp2[5] = {179,180,181,195,196};
   
   static MGROUP_ROTATE rudderr (0,RGrp2, 5, _V(2.476552,3.198984E-03,-7.42393), _V(0,1,0), (float)(-33*RAD));
   
//Cpit Rudder Pedals Pilot Left
   static UINT RGrp3[2] = {61,62};
   
   static MGROUP_ROTATE ruddercl (2,RGrp3, 2, _V(0,6,0), _V(1,0,0), (float)(1.33*RAD));
   
   //Cpit Rudder Pedals Pilot Right
   static UINT RGrp4[2] = {59,60};
   
   static MGROUP_ROTATE ruddercr (2,RGrp4, 2, _V(0,6,0), _V(1,0,0), (float)(-1.33*RAD));

   //Cpit Rudder Pedals Co-Pilot Left
   static UINT RGrp5[2] = {55,57};
   
   static MGROUP_ROTATE rudderccl (2,RGrp5, 2, _V(0,6,0), _V(1,0,0), (float)(1.33*RAD));
   
   //Cpit Rudder Pedals Co-Pilot Right
   static UINT RGrp6[2] = {56,58};
   
   static MGROUP_ROTATE rudderccr (2,RGrp6, 2, _V(0,6,0), _V(1,0,0), (float)(-1.33*RAD));

   
   anim_rudder = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderl);
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderr);
   AddAnimationComponent  (anim_rudder, 0, 1, &ruddercl);
   AddAnimationComponent  (anim_rudder, 0, 1, &ruddercr);
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderccl);
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderccr);

   
   //Elevator
   static UINT EGrp1[1] = {198};
   
   static MGROUP_ROTATE elevator (0,EGrp1, 1, _V(0,-1.575147,-7.142082), _V(1,0,0), (float)(33*RAD));
   
   
   anim_elevator = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_elevator, 0, 1, &elevator);
   
   
   
   
   
   //Left Aleron
   static UINT LAGrp1[2] = {212,213};
   
   static MGROUP_ROTATE laileron (0,LAGrp1, 2, _V(-4.558668,-0.6,1.448453), _V(1,-0.1,-0.1), (float)(-45*RAD));
   
   
   anim_laileron = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_laileron, 0, 1, &laileron);
   
   
   //Right Aleron
   static UINT RAGrp1[2] = {217,218};
   
   static MGROUP_ROTATE raileron (0,RAGrp1, 2, _V(4.558668,-0.6,1.448453), _V(1,0.1,0.1), (float)(45*RAD));
   
   
   anim_raileron = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_raileron, 0, 1, &raileron);
   
   
  
   
   
   //Flaps animation
   
   //Left Flap
   static UINT FLGrp1[2] = {210,211};
   
   static MGROUP_ROTATE flapsl (0,FLGrp1, 2, _V(-2.358121,-0.8,1.226563), _V(1,-0.1,-0.1), (float)(-45*RAD));
   
   
   //Right Flap
   static UINT FRGrp1[2] = {215,216};
   
   static MGROUP_ROTATE flapsr (0,FRGrp1, 2, _V(2.358121,-0.8,1.226563), _V(1,0.1,0.1), (float)(-45*RAD));
   
   //Flap Dial
   static UINT FLGrp3[1] = {48};
   
   static MGROUP_ROTATE flaps3 (2,FLGrp3, 1, _V(-0.4297518,0.3358538,0), _V(0,0,1), (float)(145*RAD));
   
  
   
   anim_flaps = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_flaps, 0, 1, &flapsl);
   AddAnimationComponent  (anim_flaps, 0, 1, &flapsr);
   AddAnimationComponent  (anim_flaps, 0, 1, &flaps3);
  
   
   
   
  
   
  

   // Speed Dial Animations
   
   // Dial3
   static UINT d3Grp0[2] = {89,90};
   
   static MGROUP_ROTATE dial3 (2,d3Grp0, 2, _V(0.4632326,0.3308442,0), _V(0,0,1), (float)(360*RAD));
   
   
   anim_dial3 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_dial3, 0, 1, &dial3);
   
   
   // Bank Guage Animations
   
   static UINT BAGrp0[2] = {0,98};
   
   static MGROUP_ROTATE bank (2,BAGrp0, 2, _V(0.2910122,0.2731915,0), _V(0,0,1), (float)(-360*RAD));
   
   
   
   anim_bank = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_bank, 0, 1, &bank);
   
   
   
   
   // Clock
   
   // Second Hand
   static UINT dGrp7[1] = {82};
   
   static MGROUP_ROTATE dial7 (2,dGrp7, 1, _V(-0.4338541,0.2369146,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_clock_s = CreateAnimation (0);
   
   AddAnimationComponent  (anim_clock_s, 0, 1, &dial7);
   
   // Minute Hand
   static UINT dGrp8[1] = {76};
   
   static MGROUP_ROTATE dial8 (2,dGrp8, 1, _V(-0.4338541,0.2369146,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_clock_m = CreateAnimation (0);
   
   AddAnimationComponent  (anim_clock_m, 0, 1, &dial8);
   
   // Hour Hand
   static UINT dGrp9[1] = {52};
   
   static MGROUP_ROTATE dial9 (2,dGrp9, 1, _V(-0.4338541,0.2369146,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_clock_h = CreateAnimation (0);
   
   AddAnimationComponent  (anim_clock_h, 0, 1, &dial9);
   
   
   // Fuel Guage Animation
   
   static UINT Ful0[1] = {51};
   
   static MGROUP_ROTATE fulg (2,Ful0, 1, _V(0.4646553,0.4646553,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_fuel = CreateAnimation (0);
   
   AddAnimationComponent  (anim_fuel, 0, 1, &fulg);

   
    
   
   
// Props Anim and Engine Turn Animations

   static UINT ETrn0[14] = {32,33,67,68,69,70,71,72,73,74,75,76,77,78};
   
   static MGROUP_ROTATE engineturn0 (0,ETrn0, 14, _V(0,-0.7600124,1.967322), _V(1,0,0), (float)(-90*RAD));
   


   //Left Prop
   static UINT PropGrp1[1] = {126};
   
  prop1 = new MGROUP_ROTATE  (0,PropGrp1, 1, _V(-6.66107,-0.5687591,3.856969), _V(0,0,1), (float)(-360*RAD));
   
  
   //Right Prop
   static UINT PropGrp3[1] = {130};
   
   prop3 = new MGROUP_ROTATE  (0,PropGrp3, 1, _V(6.66107,-0.5687591,3.856969), _V(0,0,1), (float)(360*RAD));
   
   
   //Left Prop Scaled
   static UINT PropGrp1s[3] = {127,128,129};
   
  prop1s = new MGROUP_SCALE  (0,PropGrp1s, 3, _V(0,0,0), _V(0.001,0.001,0.001));
   
  
   //Right Prop Scaled
   static UINT PropGrp3s[3] = {131,132,133};
   
   prop3s = new MGROUP_SCALE  (0,PropGrp3s, 3, _V(0,0,0), _V(0.001,0.001,0.001));
   
   
   
   anim_engturn = CreateAnimation (0);
   anim_prop = CreateAnimation (0);
   anim_props = CreateAnimation (0);
   
   parent2 = AddAnimationComponent  (anim_engturn, 0, 1, &engineturn0);
   
   AddAnimationComponent  (anim_prop, 0, 1, prop1, parent2);
  
   AddAnimationComponent  (anim_prop, 0, 1, prop3, parent2);
   
   AddAnimationComponent  (anim_props, 0, 1, prop1s, parent2);
  
   AddAnimationComponent  (anim_props, 0, 1, prop3s, parent2);
  

   // Prop Blurred Anims
   
   // Props Anim Blurred and Engine Blurred Turn Animations
   
   // Left Eng Turn Blurred
   
   static UINT ETrn0lb[1] = {1};
   
   static MGROUP_ROTATE engineturn0lb (3,ETrn0lb, 1, _V(0,0,0), _V(1,0,0), (float)(-90*RAD));
   
   // Right Eng Turn Blurred
   
   static UINT ETrn0rb[1] = {1};
   
   static MGROUP_ROTATE engineturn0rb (4,ETrn0rb, 1, _V(0,0,0), _V(1,0,0), (float)(-90*RAD));
   
   
    //Left Prop Blurred
   static UINT PropGrp1b[1] = {2};
   
  prop1b = new MGROUP_ROTATE  (3,PropGrp1b, 1, _V(0,0,0), _V(0,0,1), (float)(-360*RAD));
   
  
   //Right Prop Blurred
   static UINT PropGrp3b[1] = {2};
   
   prop3b = new MGROUP_ROTATE  (4,PropGrp3b, 1, _V(0,0,0), _V(0,0,1), (float)(360*RAD));
   
    //Left Prop Blurred Scaled
   static UINT PropGrp1bs[1] = {0};
   
  prop1bs = new MGROUP_SCALE  (3,PropGrp1bs, 1, _V(0,0,0), _V(0.001,0.001,0.001));
   
  
   //Right Prop Blurred Scaled
   static UINT PropGrp3bs[1] = {0};
   
   prop3bs = new MGROUP_SCALE  (4,PropGrp3bs, 1, _V(0,0,0), _V(0.001,0.001,0.001));
   
   
   anim_engturnlb = CreateAnimation (0);
   anim_engturnrb = CreateAnimation (0);
   
   anim_propb = CreateAnimation (0);
   anim_propbs = CreateAnimation (0);
   
   parent2lb = AddAnimationComponent  (anim_engturnlb, 0, 1, &engineturn0lb);
   
   parent2rb = AddAnimationComponent  (anim_engturnrb, 0, 1, &engineturn0rb);
   
   parentb = AddAnimationComponent  (anim_propb, 0, 1, prop1b, parent2lb);
  
   parentb2 = AddAnimationComponent  (anim_propb, 0, 1, prop3b, parent2rb);
   
   AddAnimationComponent  (anim_propbs, 0, 1, prop1bs, parentb);
  
   AddAnimationComponent  (anim_propbs, 0, 1, prop3bs, parentb2);
   
    // Stick Animations
   
   static UINT STGrpL0[2] = {92,96};
   
   stick0 = new MGROUP_ROTATE (2,STGrpL0, 2, _V(-0.8810394,-0.5558859,0.5809922), _V(0,0,1), (float)(-33*RAD));
   
   static UINT STGrpR0[2] = {93,97};
   
   stick1 = new MGROUP_ROTATE (2,STGrpR0, 2, _V(0.8810394,-0.5558859,0.5809922), _V(0,0,1), (float)(-33*RAD));
   
   
   static UINT STGrp1[2] = {94,95};
   
   static MGROUP_ROTATE stick2 (2,STGrp1, 2, _V(0,-0.6160793,0.5809922), _V(1,0,0), (float)(-33*RAD));
   
   anim_stick0 = CreateAnimation (0.5);
   anim_stick1 = CreateAnimation (0.5);
   anim_stick2 = CreateAnimation (0.5);
   
   parent3 = AddAnimationComponent  (anim_stick2, 0, 1, &stick2);
   AddAnimationComponent  (anim_stick0, 0, 1, stick0, parent3);
   AddAnimationComponent  (anim_stick1, 0, 1, stick1, parent3);
   
   
}
	

bool ShuttlePB::clbkLoadVC (int id)
{
	
	SetCameraOffset (_V(0.65,1,3.7));
	
    SetCameraDefaultDirection (_V(0,0,1));

    SetCameraRotationRange (RAD*60, RAD*60, RAD*40, RAD*40);
	
	
	static VCMFDSPEC mfds_left = {2, 99};
    static VCMFDSPEC mfds_right = {2, 100};
    static VCMFDSPEC mfds_mid = {2, 101};
    static VCMFDSPEC mfds_topleft = {2, 102};
    static VCMFDSPEC mfds_topright = {2, 103};

   
	
    
    oapiVCRegisterMFD (MFD_LEFT, &mfds_left);
    oapiVCRegisterMFD (MFD_RIGHT, &mfds_right);
    oapiVCRegisterMFD (MFD_USER1, &mfds_mid);
    oapiVCRegisterMFD (MFD_USER2, &mfds_topleft);
    oapiVCRegisterMFD (MFD_USER3, &mfds_topright);


   
    
    
 return 1;

}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new ShuttlePB (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (ShuttlePB*)vessel;
}
